#include <string.h>
#include "intools.h"
#include "iotools.h"
#include "vector.h"
#include "siftop.h"
#include "import_T3d.h"
#include "alias.h"
#include "global.h"
#include "loadn.h"
#include "dloadn.h"
#include "loadel.h"
#include "input.h"
#include "bocon.h"
#include "inicon.h"
#include "dbcrs.h"
#include "dbmat.h"
#include "edgeload.h"
#include "elsurfload.h"
#include "surfload.h"
#include "pointset.h"
#include "tempload.h"
#include "gfunct.h"
#include "aggregator.h"


siftop    *Top;     ///< Sifel topology
long       Npropn;  ///< number of entries with nodal property
long       Nprope;  ///< number of entries with element property
// Folowing arrays have size equal to number of lines with property description
//  of nodes(elements).
// This numbers are specified in the input file on the first line before section
//  described nodals or elements property
long      *Nnp;     ///< property numbers for nodes
long      *Nep;     ///< property numbers for elements
long      *Ndofn;   ///< array of number of dof for nodes
bocon    **Boc;     ///< array of pointers to the nodal boundary conditions structures
long     **Boccn;   /**< array with bocon squence number (0 : if Bocon[i] == NULL)
                         (positive number : if Bocon[i] != NULL)*/
loadel   **Lel;     ///< array of pointers to the element load structures
loadn    **Lno;     ///< array of pointers to the nodal load structures
// dloadel  **Dlel;   ///< array of pointers to the dynamic element load structures
dloadn   **Dlno;    ///< array of pointers to the dynamic nodal load structures
elemtype  *Tel;     ///< array of element types
strastrestate *Ssse;///< stress strain state for plane element
long      *Sel;     ///< array of shape element number
long      *Nmel;    ///< number of materials on the element
mattype  **Mel;     ///< array of material types
long     **Melid;   ///< array of material indeces
crsectype *Ncs;     ///< array of cross-section types in the nodes
long      *Ncsi;    ///< array of cross-section indeces in the nodes
long     **Nccn;    ///< array of pointers to common code numbers in the nodes
tempload **Ntemp;   ///< array of pointers temperature load of nodes structures
long       Ntempl;  ///< indicator whether temperature load is given in the current problem
crsectype *Ecs;     ///< array of cross-section types in the elements
long      *Ecsi;    ///< array of cross-section indeces in the elements
vector   **Nlsv;    ///< array of vectors of local systems in the nodes
vector   **Elsv;    ///< array of vectors of local systems in the elements
long       Nels;    ///< number of local coordinate systems in the integration points
dbmat     *Dbmat;   ///< database of materials and their parameters
dbcrs     *Dbcrs;   ///< database of cross-sections and their parameters
long      *Tndofn;  ///< array of number of dof for every node
elemtype  *Tet;     ///< array of element type for each element
long      *Nmu;     ///< array with indicators of used material types from material database
long      *Ncu;     ///< array with indicators of used cross-section types from cross-section database
long       Nlc;     ///< number of loading cases
long       Nslc;    ///< number of loading cases in time dependent mechanical problems
long       Nlay;    ///< number of layers for layered problems
long       Nspd;    ///< number of values of static prescribed displacements
long       Ndpd;    ///< number of values of dynamic prescribed displacements
long       Csrin;   ///< number of renumbered index in function wr_nodes
inicon   **Inic;    ///< array of pointers to the nodal initial conditions structures
long       Ninic;   ///< indicator whether initial conditions are given in the current problem
long      *Enip;    ///< array with number of integration point for each element
edgeload **Edgl;    ///< array with edge load
surfload **Surfl;   ///< array with surface load
elsurfload  **Elsurfl; ///< array with surface load on elements
elemposition *Eep;     ///< array with position indicator of point where stresses and strains will be evaluated
pointset    **Eps;     ///< array with position point sets
gfunct       *Tf;      ///< array of time functions used in time dependent mechanical problems for each load case
dynload       Tdynload;///< type of dynamic load
long         *Tfel;    ///< array of indeces of time functions for elements in growing structure problems
/**
 Function   reads problem data from file in a writes it to file out in SIFEL format.

 @param in   - pointer to opened input file with problem description
 @param outf - pointer to string with name of output file

 @retval 0 - on succes
 @retval 1 - unable to open temporary file
 @retval 2 - if fails reading number of loading cases
 @retval 3 - if fails reading topology
 @retval 4 - if fails reading materials
 @retval 5 - if fails reading cross-sections
 @retval 6 - if fails reading property file
 @retval 7 - if fails writing output file
*/
long input(XFILE *in, descrip *d)
{
  long i;
  long npropn, nprope;
  long err, tlc;
  long nl;

  // reading number of loading cases
  xfscanf(in, "%ld", &Nlc);
  tlc = Nlc / 2;
  Nlay = 0;
  switch (Mp->tprob)
  {
    case mat_nonlinear_statics :
    {
      if (tlc * 2 != Nlc)
      {
        fprintf(stderr, "\n\nError reading number of loading cases");
        fprintf(stderr, "\n Number of loading cases is %ld", Nlc);
        fprintf(stderr, "\n Number of loading cases has to be odd");
        fprintf(stderr, "\n because you request to solve mat_nonlinear statics.");
	fprintf (stderr,"\n %s, line %d\n",__FILE__,__LINE__);
        return (2);
      }
      break;
    }
    case eigen_dynamics :
    {
      if (Nlc > 0)
      {
        fprintf(stderr, "\n\nError reading number of load cases");
        fprintf(stderr, "\n Number of load cases is %ld", Nlc);
        fprintf(stderr, "\n Number of load cases has to be zero");
        fprintf(stderr, "\n you request to solve eigen dynamics.");
	fprintf (stderr,"\n %s, line %d\n",__FILE__,__LINE__);
        return (2);
      }
      break;
    }
    case forced_dynamics :
    {
      if (tlc * 2 != Nlc)
      {
        fprintf(stderr, "\n\nError reading number of load cases");
        fprintf(stderr, "\n Number of load cases is %ld", Nlc);
        fprintf(stderr, "\n Number of load cases has to be odd");
        fprintf(stderr, "\n because you request to solve forced dynamics.");
	fprintf (stderr,"\n %s, line %d\n",__FILE__,__LINE__);
        return (2);
      }
      // reading number of loading cases
      xfscanf(in, "%ld", &Nslc);
      if (Nslc > 0)
      {
        if (tlc != 1)
        {
          fprintf(stderr, "\n\nError reading number of load cases");
          fprintf(stderr, "\n Number of load cases is %ld", Nlc);
          fprintf(stderr, "\n Number of load cases has to be 1");
          fprintf(stderr, "\n because you request to solve forced dynamics");
          fprintf(stderr, "\n with subloadcases.");
          fprintf (stderr,"\n %s, line %d\n",__FILE__,__LINE__);
          return (2);
        }
        //  allocation of array of time functions
        Tf = new gfunct [Nslc];
 
        for (i=0;i<Nslc;i++){
          Tf[i].read_prop (in->file);
        }
      }
      else
        xfscanf(in, "%d", &Tdynload);
      break;
    }

  case growing_mech_structure:
  case mech_timedependent_prob:{
    // reading number of loading cases
    xfscanf(in, "%ld", &Nslc);
    //  allocation of array of time functions
    Tf = new gfunct [Nslc];

    for (i=0;i<Nslc;i++){
      Tf[i].read(in);
    }

    break;
  }
    case layered_linear_statics :
    {
      xfscanf(in, "%ld", Nlay);
      if (Nlay == 0)
      {
        fprintf(stderr, "\n\nError reading number of layers");
	fprintf (stderr,"\n %s, line %d\n",__FILE__,__LINE__);
        return(2);
      }
      break;
    }
    default :
      break;
  }

  if (Mp->ssle->prec.pt==boss){
    if (Mp->ssle->prec.agg->impl==2){
      d->seq=0;
    }
  }
  if (Mp->ssle->tlinsol==sfeti){
    d->seq=0;
  }
  
  // reading topology
  err = input_siftop(d);
  if (err)
    return(3);

  // reading material database
  err = input_materials(d->matf);
  if (err)
    return(4);

  // reading cross-section elements
  err = input_crs(d->crf);
  if (err)
    return(5);

  // reading nodal and element properties
  err = input_prop(in, npropn, nprope, nl);
  switch (err)
  {
    case 1 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading property type of node fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 2 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading property ndof of node fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 3 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " property ndof of node hasn't been entered before proprety, which it needs.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 4 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading boundary condition of nodes fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 5 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading nodal load fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 6 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading cross-section of node fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 7 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading transformation vectors of local system in the node fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 8 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading dynamic nodal load fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 9 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading initial nodal conditions fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 10 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading common nodal code numbers fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 11 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading nodal temperature load fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 12 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading nodal time function fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 13 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading nodal time function fails - timefunction number is less than 1.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 21 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading property type of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 22 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading element type property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 23 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading material property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 24 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading cross-section property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 25 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading load property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 26 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading dynamic load property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 27 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading local coordinates system property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 28 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading edge load property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 29 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading surface load property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 32 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading number of entries of section with properties of elements fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    default :
      break;
  }
  in->line += nl;
  Npropn = npropn;
  Nprope = nprope;
  return(0);
}



/**
 Function reads topology file(T3D format) with name stored in in_t3d

 @param d - structure with input data format description

 @retval 0 - on success

 In case of reading alternative file format :
  @retval 1 - if fails reading number of nodes
  @retval 2 - if fails reading nodal coordinates
  @retval 3 - if fails reading number of elements
  @retval 4 - if fails reading number of nodes on element
  @retval 5 - if fails reading nodal number on element
  @retval 6 - if fails reading property number on element
  @retval 7 - if file in alternative format couldn't be opened
*/
long input_siftop(descrip *d)
{
  Top = new siftop;
  if (d->t3d == 1)
  {
    import_T3d_set_topology_pointer(*Top);
    if (d->seq)
      import_T3d(d->t3df, 0);
    else
      import_T3d(d->t3df, 1);
  }
  else
  {
    long ret;
    XFILE *in = xfopen(d->t3df, "rt");
    in->maxlnsize = 1024;
    //in->warrning = 1;
    in->kwdmode = ignore;
    in->ignorecase = 1;
    if (in == NULL)
      return(7);

    if (d->seq){
      ret = Top->read(in, 0, d->redgn);
    }
    else{
      if (Mp->ssle->prec.pt==boss){
	if (Mp->ssle->prec.agg->impl==2){
	  ret = Top->read(in, 2, d->redgn);
	}
      }
      else{
	if (Mp->ssle->tlinsol==sfeti){
	  ret = Top->read(in, 2, d->redgn);
	}
	else{
	  ret = Top->read(in, 1, d->redgn);
	}
      }
    }
    xfclose(in);

    return(ret);
  }
  return(0);
}



/**
 Function reads material database from file with name stored in the in

 @param in - string with database file name

 @retval 0 - on succes
 @retval 1 - if fails opening file
 @retval 2 - if fails reading number of material types
 @retval 3 - if fails reading material parameters
*/
long input_materials(char *in)
{
  FILE *f = NULL;
  f = fopen(in, "rt");
  if (f == NULL)
  {
    fprintf(stderr, "\nError - unable open material file :\n %s\n", in);
    return(1);
  }
  Dbmat = new dbmat;  // allocating new dbmat class
  if (getlong(f, Dbmat->numt)) // reading number of material types
  {
    fprintf(stderr, "\nError reading number of material types\n");
    fclose(f);
    return(2);
  }
  Dbmat->mat = new matr[Dbmat->numt]; // allocating memory for each material type
  Nmu = new long[Dbmat->numt]; // allocating array with number of used materials
  memset(Nmu, 0, sizeof(*Nmu)*Dbmat->numt);
  for (long i = 0; i < Dbmat->numt; i++)
  {
    // reading each index of material types
    if (Dbmat->mat[i].read(f))
    {
      fclose(f);
      return(3);
    }
  }
  return(0);
}



/**
 Function reads cross-section database from file with name stored in the in

 @param in - string with database file name

 @retval 0 - on succes
 @retval 1 - if fails opening file
 @retval 2 - if fails reading number of cross-section types
 @retval 3 - if fails reading cross-section parameters
*/
long input_crs(char *in)
{
  FILE *f = NULL;
  f = fopen(in, "rt");
  if (f == NULL)
  {
    fprintf(stderr, "\nError - unable open cross-section file :\n %s\n", in);
    return(1);
  }
  Dbcrs = new dbcrs; // allocating new dbcrs class
  if (getlong(f, Dbcrs->numt))  // reading number of cross-section types
  {
    fprintf(stderr, "\nError reading number of cross-section types\n");
    fclose(f);
    return(2);
  }
  Dbcrs->crs = new csec[Dbcrs->numt]; // allocating memory for each cross-section types
  Ncu = new long[Dbcrs->numt];        // allocating array with number used cross-sections
  memset(Ncu, 0, sizeof(*Ncu)*Dbcrs->numt);
  for (long i = 0; i < Dbcrs->numt; i++)
  {
    // reading cross-section properties for given cross-section type
    if (Dbcrs->crs[i].read(f))
    {
      fclose(f);
      return(3);
    }
  }
  return(0);
}



/**
 Function reads property data of nodes and elements fro file in.

 @param in  - poinetr to opened input file with property description
 @param npropn - number of property entries in section of nodal properties
 @param nprope - number of property entries in section of element properties
 @param nl - total number of lines scanned in nodal and element section

 @retval 0  : if succes
 @retval 1  : if fails reading property type of node
 @retval 2  : if fails reading property ndof of node
 @retval 3  : if ndof property of node hasn't been entered before node property,
              which needs ndof.
 @retval 4  : if fails reading boundary condition of nodes
 @retval 5  : if fails reading nodal load
 @retval 6  : if fails reading cross-section of node
 @retval 7  : if fails reading transformation vectors of local system in the node
 @retval 8  : if fails reading dynamic nodal load
 @retval 9  : if fails reading initial condition
 @retval 10 : if fails reading common code numbers in the nodes
 @retval 11 : if fails reading temperature load of node
 @retval 12 : if fails reading time function of node
 @retval 21 : if fails reading property type of element
 @retval 22 : if fails reading element type property of element
 @retval 23 : if fails reading material property of element
 @retval 24 : if fails reading cross-section property of element
 @retval 25 : if fails reading load property of element
 @retval 26 : if fails reading dynamic load property of element
 @retval 27 : if fails reading local coordinate system property of element
 @retval 28 : if fails reading edge load property of element
 @retval 29 : if fails reading surface load property of element
 @retval 31 : if fails reading number nodal property entries
 @retval 32 : if fails reading number element property entries
*/
long input_prop(XFILE *xin, long &npropn, long &nprope, long &nl)
{
  long nprop, i, j, numl;
  long nv, nne, ret, ndofn, ccn = 1, ndir, dir;
  proptype tp;
  bocon   *tbc;
  inicon  *tic;
  loadel  *tle;
//  dloadel *tdle;
  loadn   *tln;
  dloadn  *tdln;
  vector  *tlsv;
  tempload *ttln;
  FILE *in = xin->file;

  getgkwid(xin, 0, numl);
  nl = numl;
  nprop = getnprop(xin, numl);
  nl += numl;
  if (nprop < 0) // reading number of lines with nodal propreties
    return(31);
  npropn = nprop;
  Nnp = new long[nprop];   // Array with property numbers
  Ndofn = new long[nprop]; // Array with ndof
  memset(Ndofn, 0, sizeof(*Ndofn)*nprop); // setting all ndof of each property to zero
  for (i = 0; i < nprop; Nnp[i] = -1, i++);
  Boc = new bocon*[nprop];              // Array with bounadry condition
  memset(Boc, 0, sizeof(*Boc)*nprop);   //   setting each bc pointer to NULL
  Ncs = new crsectype[nprop];           // Array of cross-section types in the nodes
  memset(Ncs, 0, sizeof(*Ncs)*nprop);   //   setting each cs type to zero
  Ncsi = new long[nprop];               // Array of cross-section indeces
  memset(Ncsi, 0, sizeof(*Ncsi)*nprop); //   setting each cs to zero
//  Ncsn = 0;
  Nlsv = new vector*[nprop];            // Array with pointers to the
                                        //  local coordinate system base vectors arrays
  memset(Nlsv, 0, sizeof(*Nlsv)*nprop); //   setting each pointer lcs base vector array to NULL
  Lno = new loadn*[nprop];              // Array with pointers to the nodal load
  memset(Lno, 0, sizeof(*Lno)*nprop);   //  setting each pointer to NULL
  Dlno = new dloadn*[nprop];              // Array with pointers to the dynamic nodal load
  memset(Dlno, 0, sizeof(*Dlno)*nprop);   //  setting each pointer to NULL
  Nccn = new long*[nprop];
  memset(Nccn, 0, sizeof(*Nccn)*nprop);
  Inic = new inicon*[nprop];
  memset(Inic, 0, sizeof(*Inic)*nprop);
  Ntemp = new tempload*[nprop];
  memset(Ntemp, 0, sizeof(*Ntemp)*nprop);
  Ntempl = 0;
  Ninic = 0;

  fprintf(stdout, "\nReading nodal properties (number of entries : %ld) . . .", nprop);
  // reading node properties
  for (i = 0; i < nprop; i++)
  {
    // reading property number and type of nodal property (tp)
    if (getlong(in, Nnp[i]))
      return(1);
    tp = proptype(getpkw(in)+1);
    switch(tp)
    {
      case ndofs :  // reading number of degrees of freedom
      {
        fprintf(stdout, "\n reading ndofn");
        if (getlong(in, Ndofn[i]))
          return(2);
        break;
      }
      case boundarycond :  // reading boundary condition and prescribed displacements
      {
        fprintf(stdout, "\n reading boundary condition");
        for (j = 0, ndofn = 0; j < i; j++)
        // searching read properties whether given nodal property has assigned ndof
        {
          if ((Nnp[j] == Nnp[i]) && (Ndofn[j] > 0))
          {
            ndofn = Ndofn[j];
            break;
          }
        }
        if (ndofn == 0) // cannot find property with description of ndof
          return(3);
        tbc = new bocon; // allocating temporary boundary condition class
        if ((ret = tbc->read_prop(in, ndofn))) // reading boundary condition
          return(4);
        Boc[i] = tbc;  // storing pointer to the nodal bc
        break;
      }
      case loadnodes :  // reading nodal load
      {
        fprintf(stdout, "\n reading nodal load");
        for (j = 0, ndofn = 0; j < i; j++)
        // searching read properties whether given nodal property has assigned ndof
        {
          if ((Nnp[j] == Nnp[i]) && (Ndofn[j] > 0))
          {
            ndofn = Ndofn[j];
            break;
          }
        }
        if (ndofn == 0) // cannot find property with description of ndof
          return(3);
        tln = new loadn; // allocating temporary nodal load class
        if ((ret = tln->read_prop(in, ndofn, Nlc))) // reading nodal load
          return(5);
        Lno[i] = tln;  // storing pointer to the nodal load
        break;
      }
      case crosssec :  // reading cross-section in the node
      {
        fprintf(stdout, "\n reading cross-section");
        if (getint(in, (int &)Ncs[i]) || getlong(in, Ncsi[i]))
        // reading cross-section type and index number
          return(6);
        if ((Ncs[i] == 0) || (Ncsi[i] == 0))
          return(6);
        break;
      }
      case localsystem :  // reading local coordinate system
      {
        fprintf(stdout, "\n reading nodal lcs");
        getlong(in, nv);  // reading number of lcs base vectors
        switch (nv)
        {
          case 2 :
            tlsv = new vector[nv];  // allocating temporary array of lcs base vectors
            for (j=0; j<nv; j++)
              allocv(2, tlsv[j]);
            if (readv(in, tlsv[0]))  // reading the first base vector
              return(7);
            if (readv(in, tlsv[1]))  // reading the second base vector
              return(7);
            Nlsv[i] = tlsv;          // storing pointer to the array of lcs base vectors
            break;
          case 3 :
            tlsv = new vector[nv];
            for (j=0; j<nv; j++)
              allocv(3, tlsv[j]);
            if (readv(in, tlsv[0]))  // reading the first base vector
              return(7);
            if (readv(in, tlsv[1]))  // reading the second base vector
              return(7);
            if (readv(in, tlsv[2]))  // reading the third base vector
              return(7);
            Nlsv[i] = tlsv;          // storing pointer to the array of lcs base vectors
            break;
          default :
            return(7);
        }  // end of case number of lcs base vectors
        break;
      }  // end of main case
      case dloadnodes :  // reading nodal load
      {
        fprintf(stdout, "\n reading dynamic nodal load");
        for (j = 0, ndofn = 0; j < i; j++)
        // searching read properties whether given nodal property has assigned ndof
        {
          if ((Nnp[j] == Nnp[i]) && (Ndofn[j] > 0))
          {
            ndofn = Ndofn[j];
            break;
          }
        }
        if (ndofn == 0) // cannot find property with description of ndof
          return(3);
        tdln = new dloadn; // allocating temporary nodal load class
        if (tdln->read_prop(in, ndofn, Nlc)) // reading nodal load
          return(8);
        Dlno[i] = tdln;  // storing pointer to the nodal load
        break;
      }
      case initcond :  // reading initial condition
      {
        Ninic = 1;
        fprintf(stdout, "\n reading initial condition");
        for (j = 0, ndofn = 0; j < i; j++)
        // searching read properties whether given nodal property has assigned ndof
        {
          if ((Nnp[j] == Nnp[i]) && (Ndofn[j] > 0))
          {
            ndofn = Ndofn[j];
            break;
          }
        }
        if (ndofn == 0) // cannot find property with description of ndof
          return(3);
        tic = new inicon; // allocating temporary initial condition class
        if ((ret = tic->read_prop(in, ndofn, Nlc))) // reading initial condition
          return(9);
        Inic[i] = tic;  // storing pointer to the nodal ic
        break;
      }
      case comcodnum :  // reading common code numbers
      {
        fprintf(stdout, "\n reading common code numbers");
        if (Nccn[i] == NULL)
        {
          for (j = 0, ndofn = 0; j < i; j++)
          // searching read properties whether given nodal property has assigned ndof
          {
            if ((Nnp[j] == Nnp[i]) && (Ndofn[j] > 0))
            {
              ndofn = Ndofn[j];
              break;
            }
          }
          if (ndofn == 0) // cannot find property with description of ndof
            return(3);
          Nccn[i] = new long[ndofn];
          memset(Nccn[i], 0, sizeof(*Nccn[i])*ndofn);
          if (getlong(in, ndir))
            return(10);
          for (j = 0; j < ndir; j++)
          {
            if (getlong(in, dir))
              return(10);
            if ((dir < 1) || (dir > ndofn))
              return(10);
            ccn++;
            Nccn[i][j] = ccn;
          }
        }
        else
          return (10);
        break;
      }
      case nodetemp :  // reading nodal temperature
      {
        fprintf(stdout, "\n reading nodal temperature");
        Ntempl = 1;
        ttln = new tempload;
        if (ttln->read(in, Nlc))
          return (11);
        Ntemp[i] = ttln;
        break;
      }
      case nodtimefunc :  // reading nodal dof time functions for growing structure problem
      {
        fprintf(stdout, "\n reading nodal dof time function");
        if (Nccn[i] == NULL)
        {
          for (j = 0, ndofn = 0; j < i; j++)
          // searching read properties whether given nodal property has assigned ndof
          {
            if ((Nnp[j] == Nnp[i]) && (Ndofn[j] > 0))
            {
              ndofn = Ndofn[j];
              break;
            }
          }
          if (ndofn == 0) // cannot find property with description of ndof
            return(3);
          Nccn[i] = new long[ndofn];
          memset(Nccn[i], 0, sizeof(*Nccn[i])*ndofn);
          for (j = 0; j < ndofn; j++)
          {
            if (getlong(in, dir))
              return(12);
            if (dir < 1)
              return(13);
            Nccn[i][j] = dir;
          }
        }
        else
          return (12);
        break;
      }
      default :
        return(1);
    }
  }
  getgkwid(xin, 1, numl); // searchs end
  nl += numl;

  getgkwid(xin,0,numl); // searchs begin
  nl += numl;
  nprop = getnprop(xin, numl);
  nl += numl;
  if (nprop < 0) // reading number of lines with element propreties
    return(32);
  nprope = nprop;
  Nep = new long[nprop]; // Array with element property numbers
  for (i = 0; i < nprop; Nep[i] = -1, i++); // setting each property number to -1
  Lel = new loadel*[nprop];  // Array with pointers to element load
  memset(Lel, 0, sizeof(*Lel)*nprop); // setting each pointer to NULL
/*  Dlel = new dloadel*[nprop];  // Array with pointers to element load
  memset(Dlel, 0, sizeof(*Dlel)*nprop); // setting each pointer to NULL*/
  Tel = new elemtype[nprop];  // Array with element types
  memset(Tel, 0, sizeof(*Tel)*nprop);  // setting each type to zero
  Ssse = new strastrestate[nprop];  // Array with stress, strain state of element
  memset(Ssse, 0, sizeof(*Ssse)*nprop);  // setting each type to zero
  Sel = new long[nprop];  // Array with element shape numbers
  memset(Sel, 0, sizeof(*Sel)*nprop);  // setting each type to zero
  Nmel = new long[nprop];  // Array with number of given material types on the element
  memset(Nmel, 0, sizeof(*Nmel)*nprop);  // setting each number of material to zero
  Mel = new mattype*[nprop];  // Array with material type numbers
  memset(Mel, 0, sizeof(*Mel)*nprop);  // setting each type number to zero
  Melid = new long*[nprop];  // Array with material type indeces
  memset(Melid, 0, sizeof(*Melid)*nprop);  // setting each material index to zero
  Ecs = new crsectype[nprop];  // Array with cross-section type numbers
  memset(Ecs, 0, sizeof(*Ecs)*nprop);  // setting each cs type number to zero
  Ecsi = new long[nprop];      // Array with cross-section type indeces
  memset(Ecsi, 0, sizeof(*Ecsi)*nprop);  // setting each cs index to zero
  Elsv = new vector*[nprop];            // Array with pointers to the
                                        //  local coordinate system base vectors arrays
  memset(Elsv, 0, sizeof(*Elsv)*nprop); //   setting each pointer lcs base vector array to NULL
  Edgl = new edgeload*[nprop];
  memset(Edgl, 0, sizeof(*Edgl)*nprop);
  Surfl = new surfload*[nprop];
  memset(Surfl, 0, sizeof(*Surfl)*nprop);
  Elsurfl = new elsurfload*[nprop];
  memset(Elsurfl, 0, sizeof(*Elsurfl)*nprop);
  Eep = new elemposition[nprop];
  for (i = 0; i < nprop; Eep[i] = elemposition(-1), i++);
  Eps = new pointset*[nprop];
  memset(Eps, 0, sizeof(*Eps)*nprop);
  Tfel = new long[nprop];
  memset(Tfel, 0, sizeof(*Tfel)*nprop);

  Nels = 0;
  // reading element properties
  fprintf(stdout, "\nReading element properties (number of entries : %ld) . . .", nprop);
  for (i = 0; i < nprop; i++)
  {
    if (getlong(in, Nep[i])) // reading property number and type
      return(21);
    tp = proptype(getpkw(in)+1);
    switch(tp)
    {
      case eltype :  // reading element type
      {
        fprintf(stdout, "\n reading element type");
        getlong(in, Sel[i]); // reading element shape number
        getint(in, (int &)Tel[i]); // reading element type number
        if (Tel[i] < 0)
          return(22);
        if ((Tel[i] >= planeelementlt) && (Tel[i] <= planeelementsubqt))
          getint(in, (int &)Ssse[i]); // reading stress strain state
        break;
      }
      case matel :  // reading element material
      {
        fprintf(stdout, "\n reading material");
        getlong(in, Nmel[i]); // reading number of given material types
        if (Nmel[i] == 0)
          return(23);
        Mel[i]   = new mattype[Nmel[i]]; // Allocating material type array
        memset(Mel[i], 0, sizeof(*Mel[i])*Nmel[i]); // setting each type to zero
        Melid[i] = new long [Nmel[i]];   // Allocating material type index array
        memset(Melid[i], 0, sizeof(*Melid[i])*Nmel[i]); // setting each index to zero
        for (j = 0; j < Nmel[i]; j++)
        {
          // reading each material type number and index number
          getint(in, (int &)Mel[i][j]);
          getlong(in, Melid[i][j]);
          if ((Mel[i][j] == 0) || (Melid[i][j] == 0))
            return(23);
        }
        break;
      }
      case crosssec :  // reading element cross-section
      {
        fprintf(stdout, "\n reading cross-section");
        // reading cs type number and cs index number
        getint(in, (int &)Ecs[i]);
        getlong(in, Ecsi[i]);
        if ((Ecs[i] == 0) || (Ecsi[i] == 0))
          return(24);
        break;
      }
      case loadelems :  // reading element load
      {
        fprintf(stdout, "\n reading element load");
        tle = new loadel;  // allocating temporary lodel class
        nne = -1;
        for (j = 0; j < Top->ne; j++)
        // searching for the first element with desired property number
        // to determine number of nodes on the element
        {
          if (Top->elements[j].prop == Nep[i])
          {
            nne = Top->elements[j].nne;
            break;
          }
        }
        if (nne < 0)
	{
          delete tle;
          getnextln(in);
	  break;
        }
	//nne = Top->E[0].size;
	//	nne=8;
        if ((ret = tle->read_prep(xin, Nlc, NULL))) // reading element load
          return(25);
        Lel[i] = tle;  // storing element load
        break;
      }
/*    NOT IMPLEMENTED YET
      case dloadelems : //reading dynamic element load
      {
        tdle = new dloadel;  // allocating temporary lodel class
        for (j = 0; j < Top->elements; j++)
        // searching for the first element with desired property number
        // to determine number of nodes on the element
        {
          if (Top->E[j].property == Nep[i])
          {
            nne = Top->E[j].size;
            break;
          }
        }
        if ((ret = tdle->read_prop(in, nne, Nlc))) // reading element load
          return(26);
        Dlel[i] = tdle;  // storing dynamic element load
        break;
      }*/
      case localsystem :  // reading local coordinate system for integartion points
      {
        fprintf(stdout, "\n reading element lcs");
        getlong(in, nv);  // reading number of lcs base vectors
        tlsv = new vector[nv];  // allocating temporary array of lcs base vectors
        for (j = 0; j < nv; j++)
        {
          allocv(nv, tlsv[j]);
          if (readv(in, tlsv[j]))  // reading the first base vector
             return(27);
        }
        Nels++;
        Elsv[i] = tlsv;          // storing pointer to the array of lcs base vectors
        break;
/*        switch (Mp->nstrcomp)
        {
          case 2 :
            tlsv = new vector[nv](2);  // allocating temporary array of lcs base vectors
            if (readv(in, tlsv[0]))  // reading the first base vector
              return(27);
            if (readv(in, tlsv[1]))  // reading the second base vector
              return(27);
            Nels++;
            Elsv[i] = tlsv;          // storing pointer to the array of lcs base vectors
            break;
          case 4 :
            tlsv = new vector[nv](3);
            if (readv(in, tlsv[0]))  // reading the first base vector
              return(27);
            if (readv(in, tlsv[1]))  // reading the second base vector
              return(27);
            if (readv(in, tlsv[2]))  // reading the third base vector
              return(27);
            Nels++;
            Elsv[i] = tlsv;          // storing pointer to the array of lcs base vectors
            break;
          case 6 :
            tlsv = new vector[nv](3);
            if (readv(in, tlsv[0]))  // reading the first base vector
              return(27);
            if (readv(in, tlsv[1]))  // reading the second base vector
              return(27);
            if (readv(in, tlsv[2]))  // reading the third base vector
              return(27);
            Nels++;
            Elsv[i] = tlsv;          // storing pointer to the array of lcs base vectors
            break;
          default :
            return(27);
        }  // end of case number of lcs base vectors
        break;*/
      }  // end of main case
      case loadedge :
      {
        fprintf(stdout, "\n reading element edge load");
        Edgl[i] = new edgeload;
        if(Edgl[i]->read(in, Nlc))
          return (28);
        break;
      }
      case surfaceload :
      {
        fprintf(stdout, "\n reading surface load");
        Surfl[i] = new surfload;
        if(Surfl[i]->read(in, Nlc))
          return (29);
        break;
      }
      case elsurfaceload :
      {
        fprintf(stdout, "\n reading element surface load");
        Elsurfl[i] = new elsurfload;
        if(Elsurfl[i]->read(in, Nlc))
          return (30);
        break;
      }
      case eltimefunc :  // reading element time function
      {
        fprintf(stdout, "\n reading element time function");
        getlong(in, Tfel[i]); // reading element shape number
        break;
      }
/*      case sscomp :
      {
        fprintf(stdout, "\n reading stres/strain comp. info");
        if (getlong(in, Sel[i]))
          return (30);
        if (getint(in, int(Eep[i])))
          return (30);
        switch(Eep[i])
        {
          case nowhere : break;
          case intpts  : break;
          case enodes  : break;
          case userdefined :
            Eps[i] = new pointset;
            if (Eps[i]->read(in))
              return (30);
            break;
          default :
            return(30);
        }
        break;
      }*/
      default :
        return(21);
    }
  }
//  Mp->nlcsip = Nels;
  getgkwid(xin, 1,numl); // searchs end
  nl += numl;
  return(0);
}
