#include <math.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "adaptivity.h"
#include "global.h"
#include "elemhead.h"
#include "alias.h"
#include "vector.h"
#include "matrix.h"
#include "gtopology.h"
#include "gadaptivity.h"
#include "least_square.h"
#include "z2_smoothing.h"
#include "element.h"
#include "intpoints.h"
#include "loadcase.h"
#include "mechprint.h"
#include "j2flow.h"   // docasne

#include "mechmat.h"



#include "vecttens.h"


/*
predpoklad - na elementu je konstantni matice tuhosti d
funguje pro domenu s jednim materialem

printflags:
1 - print stdout
2 - print bgm
4 - print contoures
8 - print cmeshch.*
16 - print test
32 - print dx
64 - print rderfull
*/


adaptivity::adaptivity ()
{
  dim = Mt->give_dimension (0);
  ord = Mt->give_degree (0);
  ncomp = Mt->give_ncomp (0,0);
  nn = Mt->nn;
  ne = Mt->ne;
  
  tad = Mp->adaptivity;
  printflags = Mp->adaptflag;
  corr = Mp->corr;
  limf = Mp->limf;
  clos = Mp->clos;
  plra = Mp->plra;
  
  otherflags = -1;
  answer = 0;
  
  ni = new char[15];
  body = new long[16];
}

adaptivity::~adaptivity ()
{
  delete ni;
  delete [] body;
}


/**
   This is main function of class adaptivity, it runs in three steps:
   1. preparing strain
   2. computing of refined derivative (strain or stress, determined by otherflag) by spr_smoothing or z2_smoothing
   3. computing of error of FEM solution
   (case 3 = computing of accurate error)
   
   created  3.3.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
*/
long adaptivity::run (long of,int width,long nincr)
{
  long hod,min;
  double sec;
  
  if (Mespr==1){
    //    fprintf (stdout,"\n\n *********************************");
    fprintf (stdout,  "\n\n ***   Adaptivity is started   ***");
    
    sec = clock();
  }
  
  otherflags = of;
  answer = 0;
  
  //if (otherflags & 8)  rderfull = new double [ncomp*nn*2];
  //else                 rderfull = new double [ncomp*nn];
  rderfull = new double [ncomp*nn];
  
  prepare_ni (width,nincr);
  prepare_strain ();
  
  //if (!(otherflags & 8)){
  switch (tad){
  case 1:{
    spr ();
    compute_error ();
    break;
  }
  case 2:{
    z2_smoothing z2_est(ncomp);
    z2_est.run (rderfull);
    compute_error ();
    break;
  }
  case 3:{
    spr ();
    compute_error_fine ();
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown type of error smoothing is required in function");
    fprintf (stderr," adaptivity::run (%s, line %d)",__FILE__,__LINE__);
  }
  }
  //}
  //else{
  //  spr_nonlinear ();
  //  compute_error ();
  //}
  
  
  delete [] rderfull;
  
  if (Mespr==1){
    sec = (clock() - sec) / (double)CLOCKS_PER_SEC;
    hod = (long)sec/3600;  sec -= hod*3600;
    min = (long)sec/60;    sec -= min*60;
    fprintf(stdout,"\n Consumed time by Ada   %2ld:%02ld:%05.2f",hod,min,sec);
    
    fprintf (stdout,"\n ***   Adaptivity is finished  ***");
    //    fprintf (stdout,"\n *********************************");
  }
  return (answer);
}


/**
   
   
 */
void adaptivity::prepare_ni (int width,long nincr)
{
  
  switch (Mp->tprob){
  case linear_statics:   {
    char *fix = strchr(Mp->suffix+1,'.');
    if (fix==NULL)
      ni[0] = '\0';
    else
      strcpy (ni,fix);
    
    break;
  }
  case mat_nonlinear_statics:{
    sprintf (ni,".%0*ld",width,nincr);
    
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown problem type is required in function prepare_ni (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  
}


/**
   Function calls 'mipstrains' for computing strains in main int. points
   and sets up 'body' - indicator of integration points.
   body[i]=0 - unusual int. points , body[i]=1 - usual int. points
   
   created  3.4.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
*/
void adaptivity::prepare_strain (void)
{
  if ( (!(otherflags & 8) && !(otherflags & 4)) || ((otherflags & 8) && !(otherflags & 2)) )
    //Mm->mipstrains (lcid);
  
  
  if (Pelt!=NULL)
  {
    if ( Pelt->nb==1 && Pelt->nip[0][0]==1 && Pelt->ncomp[0]==3 )
      body[0] = 1;
    else{
      fprintf (stderr,"\n\n Warning: Pelt has not usual intpoints\n");  body[0] = 0; }
  }
  
  if (Peqt!=NULL)
  {
    if ( Peqt->nb==2 && Peqt->nip[0][0]==3 && Peqt->nip[1][1]==3 && Peqt->nip[1][0]==0 && Peqt->nip[0][1]==0 && Peqt->ncomp[0]==2 && Peqt->ncomp[1]==1)
      body[2] = 1;
    else{
      fprintf (stderr,"\n\n Warning: Peqt has not usual intpoints\n");  body[2] = 0; }
  }

  if (Pelq!=NULL)
  {
    if ( Pelq->nb==2 && Pelq->nip[0][0]==4 && Pelq->nip[1][1]==1 && Pelq->nip[1][0]==0 && Pelq->nip[0][1]==0 && Pelq->ncomp[0]==2 && Pelq->ncomp[1]==1)
      body[5] = 1;
    else{
      fprintf (stderr,"\n\n Warning: Pelq has not usual intpoints\n");  body[5] = 0; }
  }
  
  if (Peqq!=NULL)
  {
    if ( Peqq->nb==2 && Peqq->nip[0][0]==4 && Peqq->nip[1][1]==4 && Peqq->nip[1][0]==0 && Peqq->nip[0][1]==0 && Peqq->ncomp[0]==2 && Peqq->ncomp[1]==1)
      body[7] = 1;
    else{
      fprintf (stderr,"\n\n Warning: Peqq has not usual intpoints\n");  body[7] = 0; }
  }
  
  if (Cct!=NULL)
  {
    if ( Cct->nb==2 && Cct->nip[0][0]==1 && Cct->nip[1][1]==1 && Cct->nip[1][0]==0 && Cct->nip[0][1]==0 && Cct->ncomp[0]==3 && Cct->ncomp[1]==2)
      body[10] = 1;
    else{
      fprintf (stderr,"\n\n Warning: Cct has not usual intpoints\n");   body[10] = 0; }
  }
  
  if (Ltet!=NULL)
  {
    if ( Ltet->nb==1 && Ltet->nip[0][0]==1 && Ltet->ncomp[0]==6 )
      body[15] = 1;
    else{
      fprintf (stderr,"\n\n Warning: Ltet has not usual intpoints\n");  body[15] = 0; }
  }
}


/**
   Function fills up array rderfull by derivatives(strain or stresses - it depends on otherflags) in nodes.
   It fills up array 'spder' by strain or stresses in sampling points
   and calls spr_smoothing to compute refined derivatives in nodes.
   
   created  3.5.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
*/
void adaptivity::spr (void)
{
  long i;
  double **spder;
  least_square spr(dim,ncomp,Gtm,0);
  
  spder = new double* [ne];
  
  for (i=0;i<ne;i++){
    switch (Mt->elements[i].te){
    case planeelementlt:   { Pelt->elchar  (i,spder[i],otherflags); break;}
    case planeelementqt:   { Peqt->elchar  (i,spder[i]); break;}
    case planeelementlq:   { Pelq->elchar  (i,spder[i]); break;}
    case planeelementqq:   { Peqq->elchar  (i,spder[i]); break;}
    case lineartet:        { Ltet->elchar  (i,spder[i]); break;}
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function spr (%s, line %d)",__FILE__,__LINE__);
    }
    }
  }
  
  spr.spr_default (Out,spder,rderfull);
  
  for (i=0;i<ne;i++)  delete [] spder[i];
  delete [] spder;
}


/**
   Function fills up array rderfull by strain(in first half) and stress(in second half) in nodes.
   1. it fills up array 'spder' by strain in sampling points and calls spr_smoothing to compute refined strain in nodes.
   1. it fills up array 'spder' by stress in sampling points and calls spr_smoothing to compute refined stress in nodes.
   
   created  3.5.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
*/
void adaptivity::spr_nonlinear (void)
{
  long i,of;
  double **spder;
  least_square spr(dim,ncomp,Gtm,0);
  
  spder = new double* [ne];
  
  of = ((otherflags & 4) ? 6 : 2);
  
  for (i=0;i<ne;i++){
    switch (Mt->elements[i].te){
      
    case planeelementlt:   { Pelt->elchar  (i,spder[i],of); break;}
      
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function spr_nonlinear (%s, line %d)",__FILE__,__LINE__);
    }
    }
  }
  
  spr.spr_default (Out,spder,rderfull + ncomp*nn);
  
  for (i=0;i<ne;i++)  delete [] spder[i];
  
  
  for (i=0;i<ne;i++){
    switch (Mt->elements[i].te){
      
    case planeelementlt:   { Pelt->elchar  (i,spder[i],0); break;}
      
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function spr_nonlinear (%s, line %d)",__FILE__,__LINE__);
    }
    }
  }
  
  spr.spr_default (Out,spder,rderfull);
  
  for (i=0;i<ne;i++)  delete [] spder[i];
  delete [] spder;
}


/**
 */
void adaptivity::compute_error (void)
{
  if (otherflags & 1)
    nonlin_error ();
  else
    lin_error ();
}


/**
   Function computes error of FEM solution from difference between
   apriximate derivative fied(from FEM solution) and much more exact derivative fied(from array rderfull).
   Considering 'printflags' printing of any files and information is performed.
   
   created  3.6.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
*/
void adaptivity::lin_error (void)
{
  long i;
  double e2     ,u2     ,volume   ,corr1,corr2;
  vector ei2(ne),ui2(ne),sizel(ne),refsizel(ne);
  
  e2 = u2 = volume = 0.0;
  
  for (i=0;i<ne;i++){
    
    switch (Mt->give_elem_type (i)){
    case planeelementlt:{  Pelt->compute_error (i,volume,ei2[i],ui2[i],sizel[i],rderfull,otherflags);  corr1 = 1.00; corr2 = 1.11;   break; }
    case planeelementlq:{  Pelq->compute_error (i,volume,ei2[i],ui2[i],sizel[i],rderfull);             corr1 = 1.00; corr2 = 1.20;   break; }
    case planeelementqt:{  Peqt->compute_error (i,volume,ei2[i],ui2[i],sizel[i],rderfull);             corr1 = 1.07; corr2 = 1.55;   break; }
    case planeelementqq:{  Peqq->compute_error (i,volume,ei2[i],ui2[i],sizel[i],rderfull);             corr1 = 1.00; corr2 = 1.60;   break; }
    case lineartet:{       Ltet->compute_error (i,volume,ei2[i],ui2[i],sizel[i],rderfull);             corr1 = 1.00; corr2 = 1.00;   break; }
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function");
      fprintf (stderr," adaptivity::compute_error (%s, line %d)",__FILE__,__LINE__);
    }
    }
    
    if (tad == 1) ei2[i] *= corr1*corr1;
    if (tad == 2) ei2[i] *= corr2*corr2;
    
    e2 += ei2[i];
    u2 += ui2[i];
  }
  u2 += e2;
  
  compute_refsizel_lin (sizel.a,ei2.a,e2,u2,ord,ne,refsizel.a);
  
  // docasne
  answer = 1;
  print (volume,e2,u2,ei2.a,ui2.a,refsizel.a);
}


/**
   Function computes new=required characteristic size of elements.
   
   @param sizel - array of old characteristic sizes of element
   @param
   
   created  3.6.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
 */
void adaptivity::compute_refsizel_lin ( double *sizel, double *ei2,double e2,double u2,long ord,long ne,double *refsizel)
{
  long i;
  double em2,specacc;
  vector epsil;
  
  allocv (ne,epsil);
  
  specacc = Mp->adapt_accuracy;
  em2 = specacc*specacc * u2/ne;
  
  //corr = 0.8;  //<0.7;1.0>
  
  for (i=0;i<ne;i++){
    epsil[i] = sqrt(ei2[i]/em2);
    if (epsil[i]<0.05) epsil[i] = 0.05;
    refsizel[i] = sizel[i] / pow(epsil[i] , 1.0/(double)ord * corr);
  }
  
  // ***********************************************
  //char file[255];
  //sprintf (file,"%sepsil.dx%s",Mp->path,ni);
  //print_valel (Gtm,Mp,file,"epsil",epsil.a,'d');
  // ***********************************************
}


/**
   Function computes error of FEM solution from difference between
   apriximate derivative fied(from FEM solution) and much more exact derivative fied(from array rderfull).
   Considering 'printflags' printing of any files and information is performed.
   
   created  3.6.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
*/
void adaptivity::nonlin_error (void)
{
  long i;
  double e2     ,u2     ,volume   ,corr1,corr2;
  vector ei2(ne),ui2(ne),sizel(ne),refsizel(ne),epsil(ne);
  
  u2 = e2 = volume = 0.0;
  
  for (i=0;i<ne;i++){
    
    switch (Mt->give_elem_type (i)){
    case planeelementlt:{  Pelt->compute_error (i,volume,ei2[i],ui2[i],sizel[i],rderfull,otherflags);  corr1 = 1.00; corr2 = 1.11;   break; }
    case planeelementlq:{  Pelq->compute_error (i,volume,ei2[i],ui2[i],sizel[i],rderfull);             corr1 = 1.00; corr2 = 1.20;   break; }
    case planeelementqt:{  Peqt->compute_error (i,volume,ei2[i],ui2[i],sizel[i],rderfull);             corr1 = 1.07; corr2 = 1.55;   break; }
    case planeelementqq:{  Peqq->compute_error (i,volume,ei2[i],ui2[i],sizel[i],rderfull);             corr1 = 1.00; corr2 = 1.60;   break; }
    case lineartet:{       Ltet->compute_error (i,volume,ei2[i],ui2[i],sizel[i],rderfull);             corr1 = 1.00; corr2 = 1.00;   break; }
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function");
      fprintf (stderr," adaptivity::compute_error (%s, line %d)",__FILE__,__LINE__);
    }
    }
    
    if (tad == 1) ei2[i] *= corr1*corr1;
    if (tad == 2) ei2[i] *= corr2*corr2;
    
    e2 += ei2[i];
    u2 += ui2[i];
  }
  
  compute_refsizel_nonlin (refsizel.a,sizel.a,ei2.a,u2,e2,ord,ne);
  
  /*
  compute_epsil_nonlin (ei2.a,ui2.a,ne,epsil.a,e2,u2);
  
  //  double corr = 0.8;  //<0.7;1.0>  test = 0.8
  
  for (i=0;i<ne;i++){
    if (epsil[i]<0.05) epsil[i] = 0.05;
    refsizel[i] = sizel[i] / pow(epsil[i] , 1.0/(double)ord * corr);
  }
  */
  // ***********************************************
  if (answer){    
    char file[255];
    sprintf (file,"%srefsizel.dx.%s",Mp->path,ni);
//    print_valel (Gtm,Mp,file,"refsizel",refsizel.a,'d');
  }
  // ***********************************************
  
  print (volume,e2,u2,ei2.a,ui2.a,refsizel.a);
}

/**
   
   
 */
void adaptivity::compute_refsizel_nonlin (double *refsizel,double *sizel,double *ei2,double u2,double e2,long ord,long ne)
{
  long i,ri,ci,ipp,nle;
  double em2,ff,limf,specacc;
  double *rsemin,*rsemax,*f,*epsil;
  char file[255];
  vector q,sig;
  matrix sigt(3,3);
  
  ri = ci = 0;
  specacc = Mp->adapt_accuracy;
  limf = 0.1;
  
  f = new double [ne];
  epsil = new double [ne];
  rsemin = new double [ne];
  rsemax = new double [ne];
  
  allocv (4,sig);  // only 2D planestress
  allocv (1,q);    // only 2D planestress
  
  if (sqrt(e2/(u2+e2)) > specacc)
    answer = 1;
  
  // epsil
  em2 = specacc*specacc*(u2+e2)/ne;
  
  for (i=0;i<ne;i++){
    epsil[i] = sqrt(ei2[i]/em2);
    if (epsil[i]<0.05) epsil[i] = 0.05;
    rsemax[i] = sizel[i]/pow(epsil[i],1.0/(double)ord);
  }
  
  for (i=0;i<ne;i++){
    epsil[i] = epsil[i]/0.3;
    if (epsil[i]<0.05) epsil[i] = 0.05;
    rsemin[i] = sizel[i]/pow(epsil[i],1.0/(double)ord);
  }
  
  // f
  vector_tensor (sig,sigt,Mm->ip[0].ssst,stress);
  ff = Mm->yieldfunction (0,0,sigt,q);
  
  for (i=0;i<ne;i++){
    ipp = Mt->elements[i].ipp[ri][ci];
    
    Mm->givestress (0,ipp,sig);
    sig[3] = 0.0;
    
    switch (Mm->ip[ipp].tm[0]){
    case jflow:{  q[0] = Mm->ip[ipp].eqother[Mm->ip[ipp].ncompstr+1];  break; }
    default:   {  fprintf (stderr,"\n\n unknown material type is required in function adaptivity::compute_refsizel_nonlin (%s, line %d).\n",__FILE__,__LINE__); }
    }
    
    vector_tensor (sig,sigt,Mm->ip[ipp].ssst,stress);
    f[i] = Mm->yieldfunction (ipp,0,sigt,q);
  }
  
  if (0){
  // ************************************************************
  // print ff.dat
  FILE *fx; sprintf (file,"%sfxlambda.dat",Mp->path);
  fx = fopen (file,"a"); if (fx==NULL) fprintf (stderr,"\n fxlambda file has not been opened.\n\n");
  fprintf (fx,"  %f\n",f[85]); fclose (fx);
  
  // print fo f.dx
  sprintf (file,"%sf.dx",Mp->path);
//  print_valel (Gtm,Mp,file,"f",f,'d');
  
  // print sts  
  double *sts;
  sts = new double [ne];
  
  for (i=0;i<ne;i++){
    if (Mm->ip[i].eqother[4] != 0.0)    sts[i] = 1.0;
    else if (f[i] < limf*ff)          sts[i] = 3.0;
    else                              sts[i] = 2.0;
  }
  sprintf (file,"%ssts.dx",Mp->path);
//  print_valel (Gtm,Mp,file,"sts",sts,'d');
  
  delete [] sts;
  // ************************************************************
  }
  
  nle = 0;
  for (i=0;i<ne;i++){
    
    if (Mm->ip[i].eqother[4] != 0.0)
      refsizel[i] = rsemin[i];  //20.0*rsemax[i];
    else if (f[i] < limf*ff)
      refsizel[i] = rsemax[i];
    else{
      refsizel[i] = rsemin[i] + (rsemax[i] - rsemin[i]) * f[i] / ff / limf ;
      nle++;
    }
  }
  
  if (nle)
    answer = 1;
  
  // print fo f.dx
  sprintf (file,"%srefsizel.dx",Mp->path);
//  print_valel (Gtm,Mp,file,"refsizel",refsizel,'d');
  
  delete [] f;
  delete [] epsil;
  delete [] rsemin;
  delete [] rsemax;
}




/**
   
 */
void adaptivity::compute_epsil_nonlin ( double *ei2, double *ui2,long ne,double *epsil,double &e2,double &u2)
{
  long i,ri,ci,ipp,nelel,nclosel,nresel;
  double f0,limf,em2,um2,sael,sapl;
  vector q,sig,beta;
  char file[255];
  matrix sigt(3,3);
  strastrestate ssst;
  
  e2 = u2 = nelel = nclosel = nresel = ri = ci = 0;
  limf = 0.25;
  sael = Mp->adapt_accuracy;   // elastic accuracy
  sapl = sael / plra;           // plastic accuracy
  
  // predpoklad jednoho typu ssst na domene
  ssst = Mm->ip[0].ssst;
  
  allocv (ne,beta);
  // zde to predelat aby se to alocovalo automaticky, ted je tam nastvrdo 2D planestress
  allocv (4,sig);
  allocv (1,q);
  ipp = 0;
  vector_tensor (sig,sigt,ssst,stress);  // zde predpoklad ze sig a g jsou vynulovane !!
  f0 = Mm->yieldfunction (ipp,0,sigt,q);   // !!!!!!! proc neni ipp vynulovane ??????
  
  for (i=0;i<ne;i++){
    ipp = Mt->elements[i].ipp[ri][ci];
    
    if ( Mm->j2f[Mm->ip[ipp].idm[0]].give_consparam (ipp,0) ) {  beta[i] = 0.0;  continue;  }
    
    Mm->givestress (0,ipp,sig);
    sig[3] = 0.0;
    
    switch (Mm->ip[ipp].tm[0]){
    case jflow:{  q[0] = Mm->ip[ipp].eqother[Mm->ip[ipp].ncompstr+1];  break; }
    default:   {  fprintf (stderr,"\n\n unknown material type is required in function adaptivity::compute_epsil_nonlin (%s, line %d).\n",__FILE__,__LINE__); }
    }
    
    vector_tensor (sig,sigt,ssst,stress);
    beta[i] = Mm->yieldfunction(ipp,0,sigt,q) / f0;
    
    // ***********************************************
    if (i == 85){
      FILE *fx; sprintf (file,"%sfxlambda.dat",Mp->path);
      fx = fopen (file,"a"); if (fx==NULL) fprintf (stderr,"\n fxlambda file has not been opened.\n\n");
      fprintf (fx,"  %f\n",beta[i]*f0); fclose (fx);
    }
    // ***********************************************
    
    e2 += ei2[i];
    u2 += ui2[i];
    nelel++;
  }
  u2 += e2;
  
  um2 = u2/nelel;
  em2 = sael*sael*um2;
  
  for (i=0;i<ne;i++){
    //if      (beta[i] == 0.0)  epsil[i] = 1.0;
    if      (beta[i] == 0.0)  epsil[i] = sqrt(ei2[i] / em2 );
    else if (beta[i] > limf)  epsil[i] = sqrt(ei2[i] / em2 );
    else {
      epsil[i] = sqrt(ei2[i] / em2 );
      epsil[i] = sqrt(ei2[i] / ui2[i] ) / (sapl+(sael-sapl)*beta[i]/limf);
      epsil[i] = sqrt(ei2[i] / um2 ) / (sapl+(sael-sapl)*beta[i]/limf);
      nclosel++;
      if (epsil[i]>1.0) nresel++;
      //if (epsil[i]>2.0)   answer = 1;
    }
  }
  
  fprintf (stdout,"\n nclosel = %ld",nclosel);
  fprintf (stdout,"\n nresel = %ld",nresel);
  
  if (nclosel){
    if (nresel > clos*nclosel)
      answer = 1;
  }
  else
    if ( sael < sqrt(e2/u2) )
      answer = 1;
  
  // ***********************************************
  //if (answer){
  if (answer){
    
    sprintf (file,"%sbeta.dx.%s",Mp->path,ni);
//    print_valel (Gtm,Mp,file,"beta",beta.a,'d');
    
    for (i=0;i<ne;i++){
      if      (beta[i] == 0.0)    beta[i] = 1.0;
      else if (beta[i] < limf)    beta[i] = 2.0;
      else                        beta[i] = 3.0;
    }
    sprintf (file,"%ssts.dx.%s",Mp->path,ni);
//    print_valel (Gtm,Mp,file,"sts",beta.a,'d');
    
    for (i=0;i<ne;i++)  beta[i] = sqrt (ui2[i]);
    sprintf (file,"%sui.dx.%s",Mp->path,ni);
//    print_valel (Gtm,Mp,file,"ui",beta.a,'d');
  
    for (i=0;i<ne;i++)  beta[i] = sqrt (ei2[i]);
    sprintf (file,"%sei.dx.%s",Mp->path,ni);
//    print_valel (Gtm,Mp,file,"ei",beta.a,'d');
    
    for (i=0;i<ne;i++)  beta[i] = 100 * sqrt (ei2[i] / (ui2[i]+ei2[i]));
    sprintf (file,"%serror.dx.%s",Mp->path,ni);
//    print_valel (Gtm,Mp,file,"error",beta.a,'d');

    sprintf (file,"%sepsil.dx.%s",Mp->path,ni);
//    print_valel (Gtm,Mp,file,"epsil",epsil,'d');
    
  }
  
  // ***********************************************  
}


/**
 */
void adaptivity::print (double volume,double e2,double u2, double *ei2, double *ui2, double *refsizel)
{
  long i;
  double e_pere;
  vector valel(ne);
  char name[255];
  
  e_pere = 100.0*sqrt(e2/u2);
  
  // *** output to file.out ***
  fprintf (Out,"\n\n\n ADAPTIVITY\n\n e2 = %f\n u2 = %f\n e_pere = %f",e2,u2,e_pere);
  
  // *** standard output ***
  if (printflags & 1)
    fprintf (stdout,"\n e2celk = %20.10f\n u2celk = %20.10f\n e_pere = %f %%\n volume = %f",e2,u2,e_pere,volume);
  
  // *** print of background file ***
  if (printflags & 2  &&  answer){
    sprintf (name,"%smesh.bgm%s",Mp->path,ni);
//    print_valel (Gtm,Mp,name,"chyba",refsizel,'e');
  }
  
  // *** print of contoures ***
  if (printflags & 4){
    for (i=0;i<ne;i++){
      valel[i] = 100.0*sqrt(ei2[i]/(ui2[i]+ei2[i]));
      //valel[i] = pow(sqrt(ei2[i]/em2),1.0/(double)ord);
    }
    //sprintf (name,"%schyba.el%s",Mp->path,ni);
    //print_valel (Gtm,Mp,name,"chyba",valel.a,'e');
    sprintf (name,"%schyba.dx%s",Mp->path,ni);
//    print_valel (Gtm,Mp,name,"chyba",valel.a,'d');
  }
  
  // *** fine problem ***
  if (printflags & 8)
    print_coarsemesh (ei2);
  
  // *** print of test file ***
  if (printflags & 16)
    print_test (refsizel);
  
  // *** print of dx file ***
  if (printflags & 32  &&  answer){
    sprintf (name,"%s%s.ada.dx%s",Mp->path,Mp->filename,ni);
    print_default_dx (Gtm,Mp,Mt,Mm,0,name);
  }

  // *** print of rderfull in file.dx ***
  if (printflags & 64){
    sprintf (name,"%srderfull.1.dx%s",Mp->path,ni);
//    print_valnod (Gtm,Mp,Mt,name,"rderfull",rderfull,'d');
    

    if (otherflags & 8){
      sprintf (name,"%srderfull.2.dx%s",Mp->path,ni);
//      print_valnod (Gtm,Mp,Mt,name,"rderfull",rderfull + ncomp*nn,'d');
    }
  }
}


/**

   created  3.6.2002, Ladislav Svoboda, termit@cml.fsv.cvut.cz
*/
void adaptivity::print_test ( double *refsizel)
{
  long i,j,ipp;
  char f_name[255];
  FILE *f_test;
  
  sprintf (f_name,"%s%s.test",Mp->path,Mp->filename);
  f_test = fopen (f_name,"a");            if (f_test==NULL) fprintf (stderr,"\n .test file has not been opened.\n\n");
  
  
  // ***************************************************************************  
  fprintf (f_test,"\n\n LEFT HAND SIDE \n\n");
  
  for (i=0;i<Mb->nlc*Ndofm;i++){
    fprintf (f_test," %20.10f ",Lsrs->lhs[i]);
    if ((i+1)%7==0)  fprintf (f_test,"\n");
  }
  
  
  // ***************************************************************************  
  fprintf (f_test,"\n\n STRAINS AND STRESSES ON INTEGRATION POINTS \n\n");
  
  for (i=0;i<ne;i++){
    ipp = Mt->elements[i].ipp[0][0];
    
    switch ((long)Mt->elements[i].te){
    case planeelementlq:{
      for (j=0;j<4;j++){
	fprintf (f_test," %20.10f "  ,Mm->ip[ipp+j].strain[0]);
	fprintf (f_test," %20.10f "  ,Mm->ip[ipp+j].strain[1]);
	fprintf (f_test," %20.10f \n",Mm->ip[ipp+4].strain[2]);
      }
      break;
    }
    case planeelementqq:{
      for (j=0;j<4;j++){
	fprintf (f_test," %20.10f "  ,Mm->ip[ipp+j  ].strain[0]);
	fprintf (f_test," %20.10f "  ,Mm->ip[ipp+j  ].strain[1]);
	fprintf (f_test," %20.10f \n",Mm->ip[ipp+j+4].strain[2]);
	}
      break;
    }
    case planeelementlt:{
      fprintf (f_test," %20.10f "  ,Mm->ip[ipp].strain[0]);
      fprintf (f_test," %20.10f "  ,Mm->ip[ipp].strain[1]);
      fprintf (f_test," %20.10f \n",Mm->ip[ipp].strain[2]);
      break;
    }
    case planeelementqt:
    case planeelementsubqt:{
      for (j=0;j<3;j++){
	fprintf (f_test," %20.10f "  ,Mm->ip[ipp+j  ].strain[0]);
	fprintf (f_test," %20.10f "  ,Mm->ip[ipp+j  ].strain[1]);
	fprintf (f_test," %20.10f \n",Mm->ip[ipp+j+3].strain[2]);
      }
      break;
    }
    case lineartet:{
      for (j=0;j<6;j++)
	fprintf (f_test," %20.10f ",Mm->ip[ipp].strain[j]);
      
      fprintf (f_test,"\n");
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function");
      fprintf (stderr," adaptivity::print_test  (%s, line %d)",__FILE__,__LINE__);
    }
    }
    
  }
  
  if (otherflags & 8){
    fprintf (f_test,"\n");
    
    for (i=0;i<ne;i++){
      ipp = Mt->elements[i].ipp[0][0];
      
      switch ((long)Mt->elements[i].te){
      case planeelementlt:{
	fprintf (f_test," %20.10f "  ,Mm->ip[ipp].stress[0]);
	fprintf (f_test," %20.10f "  ,Mm->ip[ipp].stress[1]);
	fprintf (f_test," %20.10f \n",Mm->ip[ipp].stress[2]);
	break;
      }
      default:{
	fprintf (stderr,"\n\n unknown element type is required in function");
	fprintf (stderr," adaptivity::print_test  (%s, line %d)",__FILE__,__LINE__);
      }
      }
    }
  }
  
  
  // ***************************************************************************  
  fprintf (f_test,"\n\n RDERFULL \n\n");
  
  for (i=0;i<ncomp*nn;i++){
    fprintf (f_test," %20.10f ",rderfull[i]);
    if ((i+1)%7==0)  fprintf (f_test,"\n");
  }
  
  if (otherflags & 8){
    fprintf (f_test,"\n");
    
    for (i=0;i<ncomp*nn;i++){
      fprintf (f_test," %20.10f ",rderfull[i + ncomp*nn]);
      if ((i+1)%7==0)  fprintf (f_test,"\n");
    }
  }
  
  // ***************************************************************************  
  fprintf (f_test,"\n\n REFSIZEL \n\n");
  
  for (i=0;i<Mt->ne;i++){
    fprintf (f_test," %20.10f ",refsizel[i]);
    if ((i+1)%7==0)  fprintf (f_test,"\n");
  }
  
  // ***************************************************************************  
  if (Mm->ip[0].eqother!=NULL){
    fprintf (f_test,"\n\n OTHER \n\n");
    
    for (i=0;i<Mm->tnip;i++){
      for (j=0;j<6;j++)
	fprintf (f_test," %20.10f "  ,Mm->ip[i].eqother[j]);
      
      fprintf (f_test,"\n");
    }
  }
  
  fclose (f_test);
}


//**********************
//***  FINE PROBLEM  ***
//**********************

void adaptivity::compute_error_fine (void)
{
  long i,j,auxl,te,necm,nncm,nnelcm,nesure;
  long *nadjelnod,**adjelnod;
  long *loccfmesh,**etnodes;
  double **xyrr,thetaglob,e2cm,e2cmfine;
  vector ei2(ne),ei2cm,ei2cmfine,theta,pom;
  char auxfile[1023];
  FILE *cmc,*edg,*cor;


  if (Pelt==NULL)  Pelt = new planeelemlt;
  if (Peqt==NULL)  Peqt = new planeelemqt;
  if (Pesqt==NULL)  Pesqt = new planeelemsubqt;
  if (Pelq==NULL)  Pelq = new planeelemlq;
  if (Peqq==NULL)  Peqq = new planeelemqq;

  // reading of edges.in
  sprintf (auxfile,"%sedges.in%s",Mp->path,ni);
  edg=fopen(auxfile,"r"); if (edg==NULL){ fprintf (stderr,"\n edg file has not been specified.\n Try it again!\n"); return ;}

  fscanf (edg,"%ld",&nesure);
  if (nesure!=ne) fprintf (stdout,"\n\n\n nesure!=ne \n\n\n ");
  loccfmesh = new long [ne];

  for (i=0;i<ne;i++){
    fscanf (edg,"%ld",&auxl);
    fscanf (edg,"%ld",loccfmesh+auxl-1);
  }
  fclose(edg);

  // reading of cmchar.in
  sprintf (auxfile,"%sproblem/cmeshchr%s",Mp->path,ni);
  cmc=fopen(auxfile,"r"); if (cmc==NULL){fprintf (stderr,"\n cmc file has not been specified.\n Try it again!\n"); return ;}

  fscanf (cmc,"%ld",&nncm);
  nadjelnod = new long [nncm];
  adjelnod = new long* [nncm];
  xyrr = new double* [nncm];       for (i=0;i<nncm;i++)    xyrr[i] = new double [7];

  for (i=0;i<nncm;i++){
    for (j=0;j<7;j++)
      fscanf (cmc,"%lf",xyrr[i]+j);
    fscanf (cmc,"%ld",nadjelnod+i);
    adjelnod[i] = new long [nadjelnod[i]];
    for (j=0;j<nadjelnod[i];j++)
      fscanf (cmc,"%ld",adjelnod[i]+j);
  }

  fscanf (cmc,"%ld",&necm);
  etnodes = new long* [necm];
  allocv (necm,ei2cm);

  for (i=0;i<necm;i++){
    fscanf (cmc,"%ld %ld",&te,&nnelcm);

    etnodes[i] = new long [nnelcm+2];
    etnodes[i][0] = te;
    etnodes[i][1] = nnelcm;
    for (j=2;j<nnelcm+2;j++)
      fscanf (cmc,"%ld",etnodes[i]+j);
    fscanf (cmc,"%lf",ei2cm.a+i);
  }
  fclose(cmc);

  // computing
  for (i=0;i<ne;i++){
    te = Mt->give_elem_type (i);
    
    switch (te){
    case planeelementqt:{
      Peqt->compute_error_fine (i,etnodes[loccfmesh[i]-1],xyrr,rderfull,ei2[i]);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function");
      fprintf (stderr," adaptivity::compute_error_fine (%s, line %d)",__FILE__,__LINE__);
    }
    }
  }   //fprintf_vector (stdout,ei2,"ei2:",10);
  
  
  allocv (necm,ei2cmfine);
  allocv (necm,theta);
  
  for (i=0;i<ne;i++)
    ei2cmfine[loccfmesh[i]-1] += ei2[i];
  
  e2cm = e2cmfine = 0.0;
  for (i=0;i<necm;i++){
    e2cm     += ei2cm    [i];              fprintf (stdout,"\n [%ld] %f",i,ei2cm[i]);
    e2cmfine += ei2cmfine[i];              fprintf (stdout,"   %f",ei2cmfine[i]);
  }                                        //fprintf (stdout,"\n %f",e2cm); fprintf (stdout,"\n %f",e2cmfine);
  fprintf (stdout,"\n e2cm = %f",e2cm);
  fprintf (stdout,"\n e2cmfine = %f",e2cmfine);
  
  // compute theta
  thetaglob = sqrt(e2cm/e2cmfine);
  fprintf (stdout,"\n thetaglob =  %f",thetaglob);
  fprintf (stdout,"\n correction =  %f",1/thetaglob);
  
  for (i=0;i<necm;i++)
    theta[i] = sqrt(ei2cm[i]/ei2cmfine[i]);
  
  for (i=0;i<necm;i++)
    if (theta[i] > 3.0) theta[i] = 1.5;
  
  //fprintf_vector (stdout,theta,"theta",10);
  sprintf (auxfile,"%stheta%s",Mp->path,ni);
//  print_thetacon (auxfile,nadjelnod,adjelnod,nncm,necm,xyrr,etnodes,theta.a);
  
  //  print theta in interval (-**;0.65> A <1.5;**)
  fprintf(stdout,"\n ");
  for (j=0,i=0;i<necm;i++)
    if (theta[i]>1.5 ){
      fprintf (stdout,"[%3ld] = %5.3f  ",i,theta[i]);   j++;
      if ((j)%6==0) fprintf(stdout,"\n ");
    }
  fprintf (stdout,"\n j =  %ld",j);
  
  //  print correc_file
  sprintf (auxfile,"%scorrec.txt",Mp->path);
  cor = fopen (auxfile,"a"); if (cor==NULL){fprintf(stderr,"\n cor file has not been specified.\n Try it again!\n"); return ;}
  fprintf (cor,"%.3f\n",1/thetaglob);
  fclose (cor);
  
  
  for (i=0;i<necm;i++){ delete [] etnodes[i];     }
  for (i=0;i<nncm;i++){ delete [] adjelnod[i];
                        delete [] xyrr[i];        }
  delete [] etnodes;
  delete [] adjelnod;
  delete [] xyrr;
  delete [] loccfmesh;
  delete [] nadjelnod;

  /*
  for (i=0;i<necm;i++){
    etnodes [i][0] = 1;
    etnodes [i][1] = etnodes [i][2]+1 ;
    etnodes [i][2] = etnodes [i][3]+1 ;
    etnodes [i][3] = etnodes [i][4]+1 ;
  }
  // print deltasigxcon
  for (i=0;i<nncm;i++){
    for (j=0;j<nn;j++)
      if (Gt->gnodes[j].x == xyrr[i][0] && Gt->gnodes[j].y == xyrr[i][1])
	if ((rderfull[j] - xyrr[i][4]) < 0.0)
	  xyrr[i][3] = -(rderfull[j] - xyrr[i][4]);
	else
	  xyrr[i][3] =   rderfull[0*nn+j] - xyrr[i][4] ;

    xyrr[i][2] = 0.0;
    xyrr[i][3] *= 1000.0;
  }
  print_confile ("fine_problem/deltasigx.",nncm,necm,necm,xyrr,etnodes,ni,dim);
 */
}


void adaptivity::print_thetacon ( const char *file, long *nadjelnod,long **adjelnod,long nncm,long necm,double **xyrr,long **etnodes, double *theta)
{
  long i,j,te,nmidp;
  long **auxnod;
  double **auxxyzv;
  vector valnod(nncm);

  auxxyzv = new double* [nncm+necm];
  auxnod = new long* [necm];

  // computing of valnod
  for (i=0;i<nncm;i++){
    for (j=0;j<nadjelnod[i];j++){
      valnod[i] += theta[adjelnod[i][j]]; 
    }
    valnod[i] /= (double)nadjelnod[i]; 
  }

  // assemble aux*
  for (i=0;i<nncm;i++){
    auxxyzv[i] = new double [4];
    auxxyzv[i][0] = xyrr[i][0];
    auxxyzv[i][1] = xyrr[i][1];
    auxxyzv[i][2] = 0.0;
    auxxyzv[i][3] = valnod[i];
  }
  for (i=0;i<necm;i++){
    if (etnodes[i][0] == 23 || etnodes[i][0] == 24){
      auxxyzv[i+nncm] = new double [4];
      auxxyzv[i+nncm][0] = (xyrr[etnodes[i][0+2]][0] + xyrr[etnodes[i][1+2]][0] + xyrr[etnodes[i][2+2]][0] + xyrr[etnodes[i][3+2]][0]) / 4;
      auxxyzv[i+nncm][1] = (xyrr[etnodes[i][0+2]][1] + xyrr[etnodes[i][1+2]][1] + xyrr[etnodes[i][2+2]][1] + xyrr[etnodes[i][3+2]][1]) / 4;
      auxxyzv[i+nncm][2] = 0.0;
      auxxyzv[i+nncm][3] = theta[i];
    }
    else
      auxxyzv[i+nncm] = NULL;
  }

  nmidp = nncm;
  for (i=0;i<necm;i++){
    te = etnodes[i][0];
    nmidp++;

    switch (te){
    case planeelementlt:{
      auxnod[i] = new long [4];
      auxnod[i][0] = 1;   auxnod[i][ 1] = etnodes[i][0+2]+1;   auxnod[i][ 2] = etnodes[i][1+2]+1;   auxnod[i][ 3] = etnodes[i][2+2]+1;

      break;
    }
    case planeelementqt:{
      auxnod[i] = new long [13];
      auxnod[i][0] = 4;   auxnod[i][ 1] = etnodes[i][0+2]+1;   auxnod[i][ 2] = etnodes[i][3+2]+1;   auxnod[i][ 3] = etnodes[i][5+2]+1;
                          auxnod[i][ 4] = etnodes[i][3+2]+1;   auxnod[i][ 5] = etnodes[i][1+2]+1;   auxnod[i][ 6] = etnodes[i][4+2]+1;
                          auxnod[i][ 7] = etnodes[i][4+2]+1;   auxnod[i][ 8] = etnodes[i][2+2]+1;   auxnod[i][ 9] = etnodes[i][5+2]+1;
                          auxnod[i][10] = etnodes[i][3+2]+1;   auxnod[i][11] = etnodes[i][4+2]+1;   auxnod[i][12] = etnodes[i][5+2]+1;

      break;
    }
    case planeelementlq:{
      auxnod[i] = new long [13];
      auxnod[i][0] = 4;   auxnod[i][ 1] = etnodes[i][0+2]+1;   auxnod[i][ 2] = etnodes[i][1+2]+1;   auxnod[i][ 3] = nmidp;
                          auxnod[i][ 4] = etnodes[i][1+2]+1;   auxnod[i][ 5] = etnodes[i][2+2]+1;   auxnod[i][ 6] = nmidp;
                          auxnod[i][ 7] = etnodes[i][2+2]+1;   auxnod[i][ 8] = etnodes[i][3+2]+1;   auxnod[i][ 9] = nmidp;
                          auxnod[i][10] = etnodes[i][3+2]+1;   auxnod[i][11] = etnodes[i][0+2]+1;   auxnod[i][12] = nmidp;

      break;
    }
    case planeelementqq:{
      auxnod[i] = new long [25];
      auxnod[i][0] = 8;   auxnod[i][ 1] = etnodes[i][0+2]+1;   auxnod[i][ 2] = etnodes[i][4+2]+1;   auxnod[i][ 3] = nmidp;
                          auxnod[i][ 4] = etnodes[i][4+2]+1;   auxnod[i][ 5] = etnodes[i][1+2]+1;   auxnod[i][ 6] = nmidp;
                          auxnod[i][ 7] = etnodes[i][1+2]+1;   auxnod[i][ 8] = etnodes[i][5+2]+1;   auxnod[i][ 9] = nmidp;
                          auxnod[i][10] = etnodes[i][5+2]+1;   auxnod[i][11] = etnodes[i][2+2]+1;   auxnod[i][12] = nmidp;
                          auxnod[i][13] = etnodes[i][2+2]+1;   auxnod[i][14] = etnodes[i][6+2]+1;   auxnod[i][15] = nmidp;
                          auxnod[i][16] = etnodes[i][6+2]+1;   auxnod[i][17] = etnodes[i][3+2]+1;   auxnod[i][18] = nmidp;
                          auxnod[i][19] = etnodes[i][3+2]+1;   auxnod[i][20] = etnodes[i][7+2]+1;   auxnod[i][21] = nmidp;
                          auxnod[i][22] = etnodes[i][7+2]+1;   auxnod[i][23] = etnodes[i][0+2]+1;   auxnod[i][24] = nmidp;

      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown element type is required in function");
      fprintf (stderr," adaptivity::print_thetacon (%s, line %d)",__FILE__,__LINE__);
    }
    }
  }

  print_confile (file,nncm,necm,auxxyzv,auxnod,dim);

  for (i=0;i<nncm+necm;i++)
    delete [] auxxyzv[i];
  for (i=0;i<necm;i++)
    delete [] auxnod[i];
  delete [] auxxyzv;  delete [] auxnod;
}


void adaptivity::print_coarsemesh ( double *ei2)
{
  long i,j,te,nne,*cn;
  double r[2];
  ivector nodes;
  char file[1023];
  FILE *cmchar;
  
  sprintf (file,"%scmeshchr%s",Mp->path,ni);
  cmchar = fopen (file,"w");                 if (cmchar==NULL) fprintf (stderr,"\n .cmchar file has not been specified.\n Try it again!\n");
  
  // ** nastaveno pro 2D , lcid = 0  , codenumbers na noudech , d na prvku konstantni!!!!!!!!!!!!!!!!!!
  
  fprintf (cmchar,"%ld\n\n",nn);
  
  for (i=0;i<nn;i++){
    cn = Gtm->gnodes[i].cn;
    for (j=0;j<2;j++){
      if        (cn[j]>0)   r[j]=Lsrs->lhs[cn[j]-1];
      else if   (cn[j]==0)  r[j]=0.0;
           else             r[j]=Mb->lc[0].pd[0-cn[j]-1];
    }
    fprintf (cmchar," %15.7e %15.7e %15.7e %15.7e",Gtm->gnodes[i].x,Gtm->gnodes[i].y,r[0],r[1]);
    fprintf (cmchar," %15.7e %15.7e %15.7e %6ld"  ,rderfull[i],rderfull[nn+i],rderfull[2*nn+i],Gtm->nadjelnod[i]);
    for (j=0;j<Gtm->nadjelnod[i];j++)
      fprintf (cmchar,"%6ld",Gtm->adjelnod[i][j]);
    fprintf (cmchar,"\n");
  }
  
  fprintf (cmchar,"\n\n%ld\n\n",ne);
  
  for (i=0;i<ne;i++){
    te = Mt->give_elem_type (i);
    nne = Mt->give_nne_inner ((elemtype)te);
    allocv (nne,nodes);
    
    Gtm->give_nodes (i,nodes);
    
    fprintf (cmchar,"  %2ld  %2ld",te,nne);
    for (j=0;j<nne;j++)
      fprintf (cmchar,"%6ld ",nodes.a[j]);
    
    fprintf (cmchar,"%15.7f\n",ei2[i]);
    
    destrv (nodes);
  }
  
  fclose (cmchar);
}
