#include "lssolver.h"
#include "global.h"
#include "globmat.h"
#include "loadcase.h"
#include "gmatrix.h"
#include "mechprint.h"
#include "node.h"
#include "elemswitch.h"
#include <string.h>

/**
   function solves linear statics problems
   
   function contains the following steps:
   1. stiffness matrix assembling
   2. right hand side assembling (load vectors assembling)
   3. solution of equation systems by choosen method
   4. strain computation (eligible)
   5. stress computation (eligible)
   6. reaction computation (eligible)
   7. results printing
   
   JK
*/
void solve_linear_statics ()
{
  long i;
  double *lhs,*rhs;
  
  //  stiffness matrix assembling
  stiffness_matrix (0);
  
  //  array is allocated in Lsrs
  lhs = Lsrs->give_lhs (0);
  rhs = Lsrs->give_rhs (0);
  
  //  loop over load cases
  for (i=0;i<Lsrs->nlc;i++){
    //  right hand side of system
    mefel_right_hand_side (i,rhs);
  }
  
  /*
  FILE *out;
  out = fopen ("vektor.txt","w");
  for (i=0;i<Ndofm;i++){
    fprintf (out,"%20.15le\n",rhs[i]);
  }
  fclose (out); 
  Smat->printmat (Out);
 */

  //  solution of equation system
  for (i=0;i<Lsrs->nlc;i++){
    Mp->ssle->solve_system (Gtm,Smat,Lsrs->give_lhs(i),Lsrs->give_rhs(i),Out);
  }
  
  print_init(-1, "wt");    
  for (i=0;i<Lsrs->nlc;i++){
    //  computes required quantities
    compute_req_val (i);
    //  prints required quantities    
    mefel_right_hand_side (i,rhs); // rhs was rewritten by the solver call -> rebuild load vector
    print_step(i, 0, 0.0, Lsrs->give_rhs(i));    
  }
  print_close();

  
  if (Mp->adaptivity)
    Ada->run (2,0,0); // 2 - for testing
  
}

