#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mefelinit.h"

#include "global.h"
#include "globmat.h"
#include "sequent.h"
#include "intpoints.h"
#include "gtopology.h"
#include "meshtransfer.h"
#include "adaptivity.h"
#include "mechprint.h"
#include "nssolver.h"
#include "element.h"
#include "elemswitch.h"
#include "iotools.h"
#include "xfile.h"
#include "saddpoint.h"

/**
   function reads data from input file and initiates all
   necessary variables and arrays
   
   @param argv -
   @param stochd - pointer to %stochdriver
   
*/
void mefel_init (int argc, const char *argv[], stochdriver *stochd)
{
  XFILE *in;
  long merr,i,j;
  char emsg[1000];

  fprintf (stdout,"\n\n ******************************************************\n");
  fprintf (stderr,"\n\n ******************************************************\n");
  
  Mp  = new probdesc;
  Gtm = new gtopology;
  Mt  = new mechtop;
  Mm  = new mechmat;
  Mc  = new mechcrsec;
  Mb  = new mechbclc;
  Outdm = new outdriverm;
  
  if (process_args(argc, argv))
  {
    delete Mp;
    delete Gtm;
    delete Mt;
    delete Mm;
    delete Mc;
    delete Mb;
    delete Outdm;
    delete stochd;
    abort();
  }

  in = xfopen(argv[1],"r");
  if (in==NULL){
    print_err("input file cannot be opened.",__FILE__,__LINE__,__func__);
    delete Mp;
    delete Gtm;
    delete Mt;
    delete Mm;
    delete Mc;
    delete Mb;
    delete Outdm;
    delete stochd;
    abort();
  }
  in->maxlnsize = 1000;
  in->warning = 1;
  if ((Mp->kwdsw == nokwd) || (Mp->kwdsw == kwd_outd))
    in->kwdmode = ignore;
  else
    in->kwdmode = sequent_mode;
  in->ignorecase = 1;
  
  // **************************
  // **  basic problem data  **
  // **************************
  Mp->read (in);

  if (Mp->kwdsw == kwd_full)
    in->kwdmode = sequent_mode;
  else
    in->kwdmode = ignore;

  // **************************************
  // **  input of node and element data  **
  // **************************************
  Mt->read (in);

  // mesh control TKr
  if (Mp->tprob == growing_mech_structure){
    if (Mespr==1){
      merr=Mt->mesh_check();
      if (merr > 0){
        sprintf(emsg, "mesh error = %ld", merr);
	print_err(emsg, __FILE__, __LINE__, __func__);
	abort();
      }
    }
  }

  // zacatek TKo
  if (Mp->tprob == earth_pressure)
    {
      Gtm->bckcn = new long*[Gtm->nn];
      memset(Gtm->bckcn, 0, sizeof(*Gtm->bckcn)*Gtm->nn);
      Gtm->backup_codnum();
    }
  //  konec TKo
  
  Out  = fopen ("mefel.log","wt");
  
  if (Mp->tprob==hemivar_inequal || Mp->tprob==lin_floating_subdomain){
    Gtm->auto_subdom_detection (Out);
    Mt->alloc_enodes ();
    Mt->alloc_edges ();
  }
  
  // *************************************
  //  generation of ordering of unknowns
  //  (generation of code numbers)
  //  number of degrees of freedom
  // *************************************
  Ndofm = Gtm->codenum_generation (Out);

  
  if (Mespr==1)
    fprintf (stdout,"\n number of degrees of freedom    %ld",Ndofm);
  //  zacatek TKr
  //Mt->store_code_num_elem(); // enable combination of plane elements, springs and bar elems with beams//tady??!!!
  //  konec TKr
  
  //  zacatek JK
  if (Mp->tprob==layered_linear_statics){
    //  definition of type of layered problem
    if (Mt->elements[0].te<20)  Mp->tlm=1;
    if (Mt->elements[0].te>20)  Mp->tlm=2;
    
    //  generation of ordering of Lagrange multipliers
    Mt->gencodnumlagrmult (Ndofm);
    Gtm->initiate_elemmult ();
    if (Mespr==1)
      fprintf (stdout,"\n total number of degrees of freedom    %ld",Ndofm);
  }


  if (Mp->tprob==lin_floating_subdomain){
    //  auxiliary informations about ordering of unknowns
    Gtm->flsub.ndof_subdom (Gtm->gnodes);
  }
  //  konec JK  
  
  // ***********************************************
  // **  input of material models and parameters  **
  // ***********************************************

  //  reading of material models and parameters
  //  allocation of integation points
  //  definition of the number of material models (nm) on integration points
  //  definition of material types (tm) on integration points
  //  definition of number of material model (idm) on integration points
  Mm->read (in);

  //  definition of variables ncompstr on intergation points
  //  definition of variables ssst on intergation points
  Mm->init_ip_1 ();
  
  // **********************************************
  // **  input of cross section characteristics  **
  // **********************************************
  Mc->read (in);
  
  // ************************
  // **  input of loading  **
  // ************************
  Mb->read (in);
  
  // ********************************************************
  // **  input of additional data necessary for problems   **
  // **  with changing number of nodes, elements and DOFs  **
  // ********************************************************
  if (Mp->tprob == growing_mech_structure){
    Gtm->read_gf (in);
    Mt->alloc_growstr ();
    Gtm->nm = new long [Gtm->nn];
  }




  //
  if (Mp->ssle->tlinsol == sschur){
    Gtm->stop->compute_multiplicity (Gtm,Out);
    Gtm->stop->node_local_numbers (Out);
    Gtm->stop->node_global_glued_numbers (Out);
    Gtm->stop->node_coarse_numbers (Out);
    Gtm->stop->node_coarse_global_glued_map (Out);
    Gtm->stop->node_coarse_local_map (Out);

    long **schurnodes;
    schurnodes = new long* [Gtm->stop->ns];
    for (i=0;i<Gtm->stop->ns;i++){
      schurnodes[i] = new long [Gtm->stop->nnsd[i]];
    }
    
    //  selection of boundary/interface nodes
    for (i=0;i<Gtm->stop->ns;i++){
      for (j=0;j<Gtm->stop->nnsd[i];j++){
	if (Gtm->stop->nodmultip[i][j]>1)
	  schurnodes[i][j]=1;
	else
	  schurnodes[i][j]=-1;
      }
    }
    
    seqselnodes *selnodschur;
    selnodschur = new seqselnodes (Gtm->stop->ns,Gtm->stop->nnsd,schurnodes,
				   Gtm->stop->nodmultip,Gtm->stop->ggnbn,Gtm->stop->icnbnmas,
				   Gtm->stop->tnbn,Gtm->stop->icmultip,Gtm->stop->lnbncn,Gtm->stop->ggnbncn,Gtm->stop->sid,
				   Gtm->stop->md,Out,1);
    
    //  the following two functions have to be called in this order
    //  otherwise, array cn in the class gnode is rewritten
    //  and wrong results are obtained
    selnodschur->prepare_schur (Gtm,Out);
    Ndofm=Gtm->stop->schur_ordering (Gtm,Out);
    if (Mespr==1){
      fprintf (stdout,"\n number of degrees of freedom    %ld",Ndofm);
    }
    
    Mp->ssle->schur.initiate (selnodschur);
    Mp->ssle->schur.assemble_subdom_unknowns (Gtm,Out);

  }
  if (Mp->ssle->tlinsol == sfeti){
    Gtm->stop->compute_multiplicity (Gtm,Out);
    Gtm->stop->node_local_numbers (Out);
    Gtm->stop->node_global_glued_numbers (Out);
    Gtm->stop->node_coarse_numbers (Out);
    Gtm->stop->node_coarse_global_glued_map (Out);
    Gtm->stop->node_coarse_local_map (Out);
    
    long **fetinodes;
    fetinodes = new long* [Gtm->stop->ns];
    for (i=0;i<Gtm->stop->ns;i++){
      fetinodes[i] = new long [Gtm->stop->nnsd[i]];
    }
    
    for (i=0;i<Gtm->stop->ns;i++){
      for (j=0;j<Gtm->stop->nnsd[i];j++){
	if (Gtm->stop->nodmultip[i][j]>1)
	  fetinodes[i][j]=1;
	else
	  fetinodes[i][j]=-1;
      }
    }
    
    seqselnodes *selnodfeti;
    
    selnodfeti = new seqselnodes (Gtm->stop->ns,Gtm->stop->nnsd,fetinodes,
				   Gtm->stop->nodmultip,Gtm->stop->ggnbn,Gtm->stop->icnbnmas,
				   Gtm->stop->tnbn,Gtm->stop->icmultip,Gtm->stop->lnbncn,Gtm->stop->ggnbncn,Gtm->stop->sid,
				   Gtm->stop->md,Out,1);
    
    Ndofm=selnodfeti->prepare_feti (Mp->ssle->feti.fetiimpl,Gtm,Out);
    
    //Gtm->cngen=1;
    //Ndofm = Gtm->codenum_generation (Out);
    //selnodfeti->contrib_dofs_ln (Gtm,Out);
    //selnodfeti->contrib_dofs_cn (Out);

    Mp->ssle->feti.initiate (selnodfeti,Gtm,Out);

    Mp->ssle->feti.assemble_subdom_unknowns (Gtm,Out);

    //selnodfeti->~seqselnodes ();
  }
  if (Mp->ssle->tlinsol == sfetidef){
    Mp->ssle->feti.assemble_subdom_unknowns (Gtm,Out);
  }




  


  // *************************************************************
  //  vectors on left and right hand side of system of equations
  // *************************************************************
  Lsrs = new lhsrhs;
  //  allocation of arrays lhs, rhs (tdlhs, stdlhs if necessary)
  Lsrs->alloc ();
  //  function reads initial conditons for dynamics
  Lsrs->initcond (in);

  if ((Mp->tprob == mech_timedependent_prob)||(Mp->tprob == growing_mech_structure))
  {
    Mb->alloc_sumcomp ();  // arrays for resultant force components
    if (Mm->csol)
      Gtm->comp_domain_sizes (); // maximum domain sizes used consolidation model
  }

  // ******************************************
  // **  definition of output from the code  **
  // ******************************************
  if ((Mp->kwdsw == nokwd) || (Mp->kwdsw == kwd_pd))
    in->kwdmode = ignore;
  else
    in->kwdmode = sequent_mode;

  Outdm->read(in);

  if (Mp->kwdsw == kwd_full)
    in->kwdmode = sequent_mode;
  else
    in->kwdmode = ignore;
  
  // ************************************************
  // **  assembling of output files from the code  **
  // ************************************************
  
  fclose (Out);
  char fname[1020];
  char *path;
  char *name;
  char *suffix;
  if (Outdm->outfn)
    Mp->filename_decomposition (Outdm->outfn,path,name,suffix);
  else
    Mp->filename_decomposition (argv[1],path,name,suffix);
   
  Mp->path = path;
  Mp->filename=name;
  Mp->suffix=suffix;
  sprintf(fname, "%s%s.log", path, name);
  delete [] path;
  delete [] name;
  delete [] suffix;
  

  Out = fopen(fname,"wt");
  if (Out == NULL)
    print_err("cannot open log file", __FILE__, __LINE__, __func__);

  //  list of adjacent elements
  Gtm->adjacnodes (Out);
  Gtm->adjacelem (Out);
  
  
  //  zacatek JK
  //  toto jsou uzivatelsky definovane body
  //Mm->stra.init(strain);  
  //Mm->stre.init(stress);  
  //  konec JK
  
  //  zacatek TKo
  //  definition of nonlocal models and models for temperature
  Mm->init_ip_2 ();
  //  konec TKo
  
  
  //  allocation of arrays defined on nodes
  Mt->alloc_nodes_arrays ();
  
  //  number of components in arrays used in outdriver
  //  maximum number of components in array strain/stress on nodes and elements
  Mt->give_maxncompstr(Mm->max_ncompstrn, Mm->max_ncompstre);
  //  maximum number of components in array other on nodes and elements
  Mt->give_maxncompo(Mm->max_nncompo, Mm->max_encompo);
  //  konec TKo
  
  //  zacatek JK
  //if (Mp->straincomp==1){
  //Mm->stra.alloc(Mb->nlc);
  //}
  //if (Mp->stresscomp==1){
  //Mm->stre.alloc(Mb->nlc);
  //}
  //  konec JK
  
  
  //  prescribed displacements on elements investigation
  Mt->elemprescdisp ();
  //  prescribed temperatures on elements investigation
  if (Mp->tprob == linear_statics){
    Mt->elempresctemp (0);
  }
  //  reactions on elements investigation
  if (Mp->reactcomp==1){
    Mt->comreac ();
  }



  //  computation of initial values at integration points from initial nodal values
  if (Mb->ico > 0)
    Mb->inicipval();
  
  
  
  if (Mp->matmodel == nonlocal){    
    //  list of adjacent elements
    Gtm->adjacelem (Out);
    //  list of adjacent integration points was not restored 
    //  from file it is necessary to create it from scratch
    if (restore_adjacip() == 0)
      adjacip ();
    save_adjacip();
    ipvolume ();
  }
  
  
  // adaptivity
  if (Mp->adaptivity)
    Ada = new adaptivity ();
  
  //  nondeterministic computation
  if (Mp->stochasticcalc){
    stochd->read (in);
    St = stochd;
  };
  
  //  initiation and computation of eigenstrains
  if (Mp->temperature==1){
    Mm->alloceigstrains ();
    Mm->alloctempstrains ();
    Mm->alloceigstresses ();
    
    Mm->tempr = new double [Mm->tnip];
    Mm->inittempr = new double [Mm->tnip];
    
    for (i=0;i<Mm->tnip;i++){
      Mm->tempr[i]=0.0;
      Mm->inittempr[i]=0.0;
    }    
  }
  
  
  //Mt->nodedisplacements ();
  
  

  xfclose (in);

}



/**
  Function prints help for command line to the opened file out

  @param out - pointer to the opened text file for output of help
  @param prgname - program name

  created 15.10.2007 by Tomas Koudelka koudelka@cml,fsv.cvut.cz
*/
void print_help(FILE *out, const char *prgname)
{
  fflush(out);
  fprintf(out, "\nInput file has not been specified\n");
  fprintf(out, " use : %s infile [-kwd=i]\n",prgname);
  fprintf(out, " where :\n *   infile is input file name\n");
  fprintf(out, " *   -kwd={0,1,2,3,4} is optional switch which controls keyword usage in the input file\n");
  fprintf(out, "           0 means no keywords (default value)\n");
  fprintf(out, "           1 means keywords only in the problem description section\n");
  fprintf(out, "           2 means keywords only in the outdriver section\n");
  fprintf(out, "           3 means keywords in the problem description and outdriver sections\n");
  fprintf(out, "           4 means full usage of keywords\n\n");
  abort();
}



/**
  Function processes command line switches.

  @param argc - number of command line arguments
  @param argv - pointer to string array with particular arguments

  @retval 0 - on success
  @retval 1 - unknown  switch is required

  created 15.10.2007 by Tomas Koudelka koudelka@cml.fsv.cvut.cz
 */
long process_args(int argc, const char *argv[])
{
  const char *pkwd;
  
  switch (argc)
  {
    case 0:
      fflush(stdout);
      fprintf(stderr, "\n\nError - too few arguments is used\n\n");
      print_help(stderr,"pmefel|mefel");
      return 1;
    case 1:
      fflush(stdout);
      fprintf(stderr, "\n\nError - too few arguments is used\n\n");
      print_help(stderr,argv[0]);
      return 1;
    case 2:
      Mp->kwdsw = nokwd;
      break;
    case 3:
      // processing of optional keyword switch -kwd={0,1,2,3,4}
      pkwd = strstrcis(argv[2], "-kwd=");
      if (pkwd)
      {
        Mp->kwdsw = pkwd_sw(-1);
        sscanf(pkwd+5, "%d", (int *)&Mp->kwdsw);
        if ((Mp->kwdsw < 0) || (Mp->kwdsw > 4))
	      { 
          fflush(stdout);
          fprintf(stderr, "\n\nError - unknown -kwd value is required\n\n");
          print_help(stderr,argv[0]);
          return 1;
        }
      }
      else
      {
        fflush(stdout);
        fprintf(stderr, "\n\nError - unknown switch is used\n\n");
        print_help(stderr,argv[0]);
        return 1;
      }
      break;
    default:
      fflush(stdout);
      fprintf(stderr, "\n\nError - too many arguments is used\n\n");
      print_help(stderr,argv[0]);
      return 1;
  }
  return 0;
}
