#include <math.h>
#include <string.h>
#include "mechmat.h"
#include "global.h"
#include "globmat.h"
#include "elemhead.h"
#include "vecttens.h"
#include "intpoints.h"
#include "element.h"
#include "node.h"
#include "elastisomat.h"
#include "elastgmat3d.h"
#include "splas1d.h"
#include "j2flow.h"
#include "j2flow2.h"
#include "microM4.h"
#include "microSIM.h"
#include "microfiber.h"
#include "mohrc.h"
#include "mohrcparab.h"
#include "boermat.h"
#include "drprag.h"
#include "camclay.h"
#include "shefplast.h"
#include "glasgmech.h"
#include "glasgowdam.h"
#include "creepdam.h"
#include "timeswmat.h"
#include "simviscous.h"
#include "svispl.h"
#include "lemaitre.h"
#include "ortodam.h"
#include "ortodamrot.h"
#include "anisodam.h"
#include "anisodamrot.h"
#include "scaldam.h"
#include "scaldamcc.h"
#include "scaldamtime2.h"
#include "aci78.h"
#include "cebfip78.h"
#include "graphmat.h"
#include "graphmattime.h"
#include "geoelast.h"
#include "creep.h"
#include "creep_b3.h"
#include "creep_rspec.h"
#include "creepb.h"
#include "creep_dpl.h"
#include "creep_effym.h"
#include "consol.h"
#include "winpast.h"
#include "nonlocmicroM4.h"
#include "therisomat.h"
#include "therisomattime.h"
#include "nonlocplast.h"
#include "nonlocdamg.h"
#include "damplast.h"
#include "tensor.h"
#include "visplast.h"
#include "elasttime.h"
#include "normmat.h"
#include "chen.h"
#include "lenjonesmat.h"
#include "contactmat.h"
#include "elemswitch.h"



/**
  Constructor initialzes data members to zero or default values.
*/
mechmat::mechmat ()
{
  //  number of material types
  nmt=0;
  //  total number of integration points
  tnip=0;

  //  indicator of plastic regime in domain
  plast=0;

  // maximum number of component of other array at nodes
  max_nncompo=0;
  // maximum number of component of eqother array on elements
  max_encompo=0;
  // maximum number of component of strain/stress arrays at nodes
  max_ncompstrn=0;
  // maximum number of component of strain/stress array on elements
  max_ncompstre=0;
  
  //  intergation points
  ip=NULL;
  //  initial conditions at integration points
  ic=NULL; 
  //  element - integration point map
  elip = NULL;


  eliso=NULL;  elgm3d=NULL;
  spl1d=NULL;  j2f=NULL;  j2f2=NULL;  mcoul=NULL;  mcpar=NULL;  
  boerm=NULL;  drprm=NULL;  cclay=NULL;  shpl=NULL;
  glasgmat=NULL;  glasgdam=NULL;  crdam=NULL;  tswmat=NULL;
  mpM4=NULL;  mpSIM=NULL;  mpfib=NULL;  nmpM4=NULL;
  svis=NULL;  svipl=NULL;   lmtr=NULL;
  scdam=NULL; scdamtime=NULL; anidam=NULL; anidamrot=NULL;  ortdam=NULL;
  ortdam=NULL; ortdamrot=NULL;
  aci78mod=NULL;  cebfip78mod=NULL;
  grmat=NULL;  grmattime=NULL;   geoel=NULL;
  crb3=NULL;
  crrs=NULL;
  crdpl=NULL;
  crbaz=NULL;  csol=NULL;   wpast=NULL;
  tidilat=NULL;
  tidilattime=NULL;
  nlplast=NULL;   nldamg=NULL;

  dampl = NULL;  visplas = NULL;  eltimemat = NULL;  normm = NULL;  
  chplast = NULL;
  lenjon = NULL;
  conmat=NULL;

  ipv = NULL;
  
  //  eigenstrains at integration points
  eigstrains = NULL;
  //  eigenstresses at integration points
  eigstresses = NULL;
  //  temperature strains at integration points
  tempstrains = NULL;
  //  temperature at integration points
  tempr = NULL;
  //  initial temperature at integration points
  inittempr = NULL;
  //  moisture at integration points
  moist = NULL;
}



/**
  Destructor releases allocated memory of the mechmat object.
*/
mechmat::~mechmat ()
{
  long i;

  delete [] ip;
  delete [] elip;
  delete [] eliso;     delete [] elgm3d;
  delete [] spl1d;     delete [] j2f;         delete [] j2f2;   
  delete [] mcoul;     delete [] mcpar;  
  delete [] boerm;     delete [] drprm;       delete [] cclay;  delete [] shpl;
  delete [] glasgmat;  delete [] glasgdam;    delete [] crdam;  delete [] tswmat;
  delete [] mpM4;      delete [] mpSIM;       delete [] mpfib;  delete [] nmpM4;
  delete [] svis;      delete [] svipl;       delete [] lmtr;
  delete [] scdam;     delete [] scdamtime;   delete [] anidam; delete [] anidamrot;
  delete [] ortdam;    delete [] ortdamrot;
  
  delete [] aci78mod;  delete [] cebfip78mod;
  delete [] grmat;     delete [] grmattime;   delete [] geoel;
  delete [] crb3;      delete [] crrs;        delete [] crdpl;
  delete [] crbaz;     delete [] csol;        delete [] wpast;
  delete [] tidilat;   delete [] tidilattime;
  delete [] nlplast;    delete [] nldamg;
  delete [] dampl;     delete [] visplas;     delete [] eltimemat;  
  delete [] normm;     delete [] chplast;
  delete [] lenjon;    delete [] conmat;

  if (ic != NULL)
  {
    for (i = 0; i < tnip; i++)
      delete [] ic;
  }
  delete [] ic;
  
  delete [] ipv;
  delete [] tempr;
  delete [] inittempr;
  delete [] moist;
  
  if (eigstrains != NULL){
    for (i=0;i<tnip;i++){
      delete [] eigstrains[i];
    }
    delete [] eigstrains;
  }
  if (eigstresses != NULL){
    for (i=0;i<tnip;i++){
      delete [] eigstresses[i];
    }
    delete [] eigstrains;
  }
  if (tempstrains != NULL){
    for (i=0;i<tnip;i++){
      delete [] tempstrains[i];
    }
    delete [] tempstrains;
  }
}



/**
   Function returns number of integration points and determines pointers
   to integration points on elements.
   
   8.7.2001
*/
long mechmat::intpnum (void)
{
  long i,j,k,n,nb;
  
  n=0;
  for (i=0;i<Mt->ne;i++){
    nb=Mt->give_nb (i);
    Mt->elements[i].ipp = new long* [nb];
    for (j=0;j<nb;j++){
      Mt->elements[i].ipp[j] = new long [nb];
    }
    for (j=0;j<nb;j++){
      for (k=0;k<nb;k++){
        Mt->elements[i].ipp[j][k]=n;
        n+=Mt->give_nip (i,j,k);
      }
    }
  }
  
  return n;
}



/**
   Function allocates integration points.
   
   JK, 28.11.2006
*/
void mechmat::intpointalloc ()
{
  ip = new intpoints [tnip];
}



/**
   Function reads material types and material id.

   @param in - input stream
   
   8.7.2001
*/
void mechmat::readip (FILE *in)
{
  long i;
  for (i=0;i<tnip;i++){
    ip[i].read (in);
  }
}



/**
   Function initiates integration point variables.
   
   21.8.2001
*/
void mechmat::init_ip_1 (void)
{
  long i,j,ne,ii,jj,ipp,nb,nip,ncomp;
  strastrestate ssst;
  
  ne=Mt->ne;

  for (i=0;i<ne;i++){
    nb=Mt->give_nb (i);
    for (ii=0;ii<nb;ii++){
      //  number of components in arrays strains/stresses
      ncomp=Mt->give_ncomp (i,ii);
      //  strain/stress state (beam, plane stress, plane strain, etc.
      ssst=Mt->give_ssst (i,ii);
      for (jj=0;jj<nb;jj++){
        //  number of the first integration point in the ii,jj block
        ipp=Mt->elements[i].ipp[ii][jj];
        //  number of integration points in the ii,jj block
        nip=Mt->give_nip (i,ii,jj);
        for (j=0;j<nip;j++){
          ip[ipp].ncompstr=ncomp;
          ip[ipp].ssst=ssst;
          
          if ((ssst == planestrain) || (ssst == planestress))
            ip[ipp].ncompstr=4;
          
          ipp++;
        }
      }
    }
  }
}



/**
   Function initiates integration point variables.
   
   JK, 21.8.2001
*/
void mechmat::init_ip_2 (void)
{
  long i,j,ne,ii,nlmid;
  
  ne=Mt->ne;

  for (i=0;i<tnip;i++){
    ip[i].alloc(Mb->nlc,i);

    ii = ip[i].hmt & 2; // the material is nonlocal
    if (ii > 0)
    // Additional allocation for the nonlocal material models is neccessary
    {
      nlmid = givenonlocid(i);
      switch (ip[i].tm[nlmid])
      {
  	case nonlocplastmat:
	  nlplast->allocation(i);
  	  break;
        case nonlocdamgmat:
	  nldamg->allocation(i,nlmid);
          break;
 	case nonlocalmicroM4:
	  nmpM4->allocation(i);
          break;
        default:
          print_err("unknown type of nonlocal material is required", __FILE__, __LINE__, __func__);
          abort();
      }
    }
    ii = ip[i].hmt & 1; // presence of thermal dilatation material model
    if (ii > 0){
      if (tempr==NULL){
	tempr = new double [tnip];
	for (j=0;j<tnip;j++){
	  tempr[j]=0.0;
	}
      }
    }
  }
}



/**
   Function defines material models on integration points.
   Material models are read on elements.
   
   JK, 28.11.2006
 */
void mechmat::intpointinit ()
{
  long i, j, ii, jj, k, nb, nip, ipp;
  
  if ((scdam != NULL) || (scdamtime != NULL) || (anidam != NULL) || (anidamrot != NULL) || (ortdam != NULL) || (ortdamrot != NULL))
    elip = new long[tnip];
  
  for (i=0;i<Mt->ne;i++){
    nb=Mt->give_nb (i);
    for (ii=0;ii<nb;ii++){
      for (jj=0;jj<nb;jj++){
        nip=Mt->give_nip(i,ii,jj);
        ipp=Mt->elements[i].ipp[ii][jj];
        for (j=0;j<nip;j++){
          if (elip != NULL)
            elip[ipp] = i;
          ip[ipp].nm  = Mt->elements[i].nm;
          ip[ipp].tm  = new mattype[ip[ipp].nm];
          ip[ipp].idm = new long[ip[ipp].nm];
          for (k = 0; k < ip[ipp].nm; k++)
          {
            ip[ipp].tm[k]  = Mt->elements[i].tm[k];
            ip[ipp].idm[k] = Mt->elements[i].idm[k];
            switch (ip[ipp].tm[k])
            {
              case therisodilat :
                ip[ipp].hmt ^= 1;
                break;
	      case therisodilattime :
                ip[ipp].hmt ^= 1;
                break;
              case nonlocplastmat :
                ip[ipp].hmt ^= 2;
                break;
              case nonlocdamgmat :
                ip[ipp].hmt ^= 2;
                break;
              case nonlocalmicroM4 :
                ip[ipp].hmt ^= 2;
                break;
              default :
                break;
	    }
	  } 
          ipp++;
        }
      }
    }
  }
}



/**
   Function reads material characteristics from file.
   
   @param in - input file stream
   
   8.7.2001
*/
void mechmat::readmatchar (XFILE *in)
{
  long i,j,k;
  long numtype;
  mattype mtype;

  xfscanf (in,"%k%ld","number_of_material_types",&nmt);
  if (nmt<1)
    print_err("wrong number of material types",__FILE__,__LINE__,__func__);
  
  if (Mespr==1)  fprintf (stdout,"\n number of different types of materials  %ld",nmt);

  for (i=0;i<nmt;i++)
  {
    xfscanf (in,"%d %ld",(int*)&mtype,&numtype);
    if (numtype<1)
      print_err("wrong number of material characteristics",__FILE__,__LINE__,__func__);
    
    switch (mtype){
    case elisomat:{
      if (Mespr==1)  fprintf (stdout,"\n number of elastic isotropic materials  %ld",numtype);
      eliso = new elastisomat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of elastic isotropic material",__FILE__,__LINE__,__func__);
        eliso[k-1].read (in);
      }
      break;
    }
    case simplas1d:{
      if (Mespr==1)  fprintf (stdout,"\n number of simple plastic 1D materials   %ld",numtype);
      spl1d = new splas1d [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of simple plastic 1D material",__FILE__,__LINE__,__func__);
        spl1d[k-1].read (in);
      }
      break;
    }
    case jflow:{
      if (Mespr==1)  fprintf (stdout,"\n number of J2 flow materials   %ld",numtype);
      j2f = new j2flow [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of J2 flow material",__FILE__,__LINE__,__func__);
        j2f[k-1].read (in);
      }
      break;
    }
    case jflow2:{
      if (Mespr==1)  fprintf (stdout,"\n number of J2 flow materials   %ld",numtype);
      j2f2 = new j2flow2 [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of J2 flow material",__FILE__,__LINE__,__func__);
        j2f2[k-1].read (in);
      }
      break;
    }
    case microplaneM4:{
      if (Mespr==1)  fprintf (stdout,"\n number of microplane materials   %ld",numtype);
      mpM4 = new microM4 [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of microplane material",__FILE__,__LINE__,__func__);
        mpM4[k-1].read (in);
      }
      break;
    }
    case nonlocalmicroM4:{
      if (Mespr==1)  fprintf (stdout,"\n number of nonlocal microplane materials   %ld",numtype);
      nmpM4 = new nonlocmicroM4 [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of nonlocal microplane material",__FILE__,__LINE__,__func__);
        nmpM4[k-1].read (in);
      }
      break;
    }
    case microsimp:{
      if (Mespr==1)  fprintf (stdout,"\n number of microplane materials   %ld",numtype);
      mpSIM = new microSIM [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of microplane material",__FILE__,__LINE__,__func__);
        mpSIM[k-1].read (in);
      }
      break;
    }
    case microfibro:{
      if (Mespr==1)  fprintf (stdout,"\n number of microplane materials   %ld",numtype);
      mpfib = new microfiber [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of microplane material",__FILE__,__LINE__,__func__);
        mpfib[k-1].read (in);
      }
      break;
    }
    case mohrcoul:{
      if (Mespr==1)  fprintf (stdout,"\n number of Mohr-Coulomb materials   %ld",numtype);
      mcoul = new mohrcoulomb [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of Mohr-Coulomb material",__FILE__,__LINE__,__func__);
        mcoul[k-1].read (in);
      }
      break;
    }
    case mohrcoulparab:{
      if (Mespr==1)  fprintf (stdout,"\n number of parabolic Mohr-Coulomb materials   %ld",numtype);
      mcpar = new mohrcoulombpar [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of parabolic Mohr-Coulomb material",__FILE__,__LINE__,__func__);
        mcpar[k-1].read (in);
      }
      break;
    }
    case boermaterial:{
      if (Mespr==1)  fprintf (stdout,"\n number of Boer materials   %ld",numtype);
      boerm = new boermat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of Boer material",__FILE__,__LINE__,__func__);
        boerm[k-1].read (in);
      }
      break;
    }
    case druckerprager:{
      if (Mespr==1)  fprintf (stdout,"\n number of Drucker-Prager materials   %ld",numtype);
      drprm = new drprag [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of Drucker-Prager material",__FILE__,__LINE__,__func__);
        drprm[k-1].read (in);
      }
      break;
    }
    case modcamclaymat:{
      if (Mespr==1)  fprintf (stdout,"\n number of modified cam-clay materials   %ld",numtype);
      cclay = new camclay [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of modified cam-clay material",__FILE__,__LINE__,__func__);
        cclay[k-1].read (in);
      }
      break;
    }
    case shefpl:{
      if (Mespr==1)  fprintf (stdout,"\n number of Sheffield plasticity materials   %ld",numtype);
      shpl = new shefplast [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of Sheffiled material",__FILE__,__LINE__,__func__);
        shpl[k-1].read (in);
      }
      break;
    }
    case chenplast:{
      if (Mespr==1)  fprintf (stdout,"\n number of Chen plasticity materials   %ld",numtype);
      chplast = new chen [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of Chen material",__FILE__,__LINE__,__func__);
        chplast[k-1].read (in);
      }
      break;
    }
    case glasgowmechmat:{
      if (Mespr==1)  fprintf (stdout,"\n number of Glasgow plasticity materials   %ld",numtype);
      glasgmat = new glasgmech [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of Glasgow material",__FILE__,__LINE__,__func__);
        glasgmat[k-1].read (in);
      }
      break;
    }
    case glasgowdamage:{
      if (Mespr==1)  fprintf (stdout,"\n number of Glasgow damage materials   %ld",numtype);
      glasgdam = new glasgowdam [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of Glasgow damage material",__FILE__,__LINE__,__func__);
        glasgdam[k-1].read (in);
      }
      break;
    }
    case creep_damage:{
      if (Mespr==1)  fprintf (stdout,"\n number of artificial material creep-damage %ld",numtype);
      crdam = new creepdam [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of artificial material creep-damge",__FILE__,__LINE__,__func__);
      }
      break;
    }
    case time_switchmat:{
      if (Mespr==1)  fprintf (stdout,"\n number of artificial time-switched material %ld",numtype);
      tswmat = new timeswmat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of artificial time-switched material",__FILE__,__LINE__,__func__);
      }
      break;
    }
    case simvisc:{
      if (Mespr==1)  fprintf (stdout,"\n number of simple viscous materials   %ld",numtype);
      svis = new simviscous [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of simple viscous material in function",__FILE__,__LINE__,__func__);
        svis[k-1].read (in);
      }
      break;
    }
    case simvisplas:{
      if (Mespr==1)  fprintf (stdout,"\n number of simple visco-plastic materials   %ld",numtype);
      svipl = new svisplas [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of simple visco-plastic material",__FILE__,__LINE__,__func__);
        svipl[k-1].read (in);
      }
      break;
    }
    case lemaitr:{
      if (Mespr==1)  fprintf (stdout,"\n number of Lemaitre visco materials   %ld",numtype);
      lmtr = new lemaitre [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of Lemaitre visco material",__FILE__,__LINE__,__func__);
        lmtr[k-1].read (in);
      }
      break;
    }
    case scaldamage:{
      if (Mespr==1)  fprintf (stdout,"\n number of simple damage materials   %ld",numtype);
      scdam = new scaldam [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of simple damage material",__FILE__,__LINE__,__func__);
        scdam[k-1].read (in);
      }
      break;
    }
    case scaldamagecc:{
      if (Mespr==1)  fprintf (stdout,"\n number of simple damage materials with crack closure  %ld",numtype);
      scdamcc = new scaldamcc [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of simple damage material with crack closure",__FILE__,__LINE__,__func__);
        scdamcc[k-1].read (in);
      }
      break;
    }
    case scaldamage_time:{
      if (Mespr==1)  fprintf (stdout,"\n number of simple damage time dependent materials   %ld",numtype);
      scdamtime = new scaldamtime [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of simple damage time dependent material",__FILE__,__LINE__,__func__);
        scdamtime[k-1].read (in);
      }
      break;
    }
    case ortodamage:{
      if (Mespr==1)  fprintf (stdout,"\n number of ortotropic damage materials   %ld",numtype);
      ortdam = new ortodam [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of ortotropic damage material",__FILE__,__LINE__,__func__);
        ortdam[k-1].read (in);
      }
      break;
    }
    case ortodamagerot:{
      if (Mespr==1)  fprintf (stdout,"\n number of orthotropic damage materials with crack rotation   %ld",numtype);
      ortdamrot = new ortodamrot [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of orthotropic damage material with crack rotation",__FILE__,__LINE__,__func__);
        ortdamrot[k-1].read (in);
      }
      break;
    }
    case anisodamage:{
      if (Mespr==1)  fprintf (stdout,"\n number of anisotropic damage materials   %ld",numtype);
      anidam = new anisodam [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of anisotropic damage material",__FILE__,__LINE__,__func__);
        anidam[k-1].read (in);
      }
      break;
    }
    case anisodamagerot:{
      if (Mespr==1)  fprintf (stdout,"\n number of rotational anisotropic damage materials with crack rotation   %ld",numtype);
      anidamrot = new anisodamrot [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of anisotropic damage material with crack rotation",__FILE__,__LINE__,__func__);
        anidamrot[k-1].read (in);
      }
      break;
    }

    case aci:{
      if (Mespr==1)  fprintf (stdout,"\n number of aci78 materials   %ld",numtype);
      aci78mod = new aci78 [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of aci78 material",__FILE__,__LINE__,__func__);
        aci78mod[k-1].read (in);
      }
      break;
    }
    case cebfip:{
      if (Mespr==1)  fprintf (stdout,"\n number of cebfip78 materials   %ld",numtype);
      cebfip78mod = new cebfip78 [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of cebfip78 material in function",__FILE__,__LINE__,__func__);
        cebfip78mod[k-1].read (in);
      }
      break;
    }
    case graphm:{
      if (Mespr==1)  fprintf (stdout,"\n number of graphic material materials   %ld",numtype);
      grmat = new graphmat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of graphic material",__FILE__,__LINE__,__func__);
        grmat[k-1].read (in);
      }
      break;
    }
    case graphmtime:{
      if (Mespr==1)  fprintf (stdout,"\n number of graphic material time dependent materials   %ld",numtype);
      grmattime = new graphmattime [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of graphic time dependent material",__FILE__,__LINE__,__func__);
        grmattime[k-1].read (in);
      }
      break;
    }
    case geoelast:{
      if (Mespr==1)  fprintf (stdout,"\n number of geoelast material materials   %ld",numtype);
      geoel = new geoelastmat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
	xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of graphic material",__FILE__,__LINE__,__func__);
        geoel[k-1].read (in);
      }
      break;
    }
    case creepb3:{
      if (Mespr==1)  fprintf (stdout,"\n number of b3 creep material materials   %ld",numtype);
      crb3 = new b3mat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of b3 creep material",__FILE__,__LINE__,__func__);
        crb3[k-1].read (in);
      }
      break;
    }
    case creeprs:{
      if (Mespr==1)  fprintf (stdout,"\n number of b3 creep material materials   %ld",numtype);
      crrs = new rspecmat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of b3 creep material",__FILE__,__LINE__,__func__);
        crrs[k-1].read (in);
      }
      break;
    }
    case creepdpl:{
      if (Mespr==1)  fprintf (stdout,"\n number of double-power law creep material materials   %ld",numtype);
      crdpl = new dplmat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of double-power law creep material",__FILE__,__LINE__,__func__);
        crdpl[k-1].read (in);
      }
      break;
    }
    case creepbaz:{
      if (Mespr==1)  fprintf (stdout,"\n number of creep material materials   %ld",numtype);
      crbaz = new creepb [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of creep material in function",__FILE__,__LINE__,__func__);
        crbaz[k-1].read (in);
      }
      break;
    }
    case creepeffym:{
      if (Mespr==1)  fprintf (stdout,"\n number of creep_effym materials   %ld",numtype);
      creffym = new creep_effym [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err ("wrong number of creep_effym material in function",__FILE__,__LINE__,__func__);
      }
      break;
    }
    case consolidation:{
      if (Mespr==1)  fprintf (stdout,"\n number of consolidation material materials   %ld",numtype);
      csol = new consol [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of consolidation material model",__FILE__,__LINE__,__func__);
        csol[k-1].read (in);
      }
      break;
    }
    case winklerpasternak:{
      if (Mespr==1)  fprintf (stdout,"\n number of Winkler-Pasternak material materials   %ld",numtype);
      wpast = new winpast [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of Winkler-Pasternak material model",__FILE__,__LINE__,__func__);
        wpast[k-1].read (in);
      }
      break;
    }
    case therisodilat:{
      if (Mespr==1)  fprintf (stdout,"\n number of isotropic thermal dilatancy materials  %ld",numtype);
      tidilat = new therisomat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of isotropic thermal dilatancy material",__FILE__,__LINE__,__func__);
        tidilat[k-1].read (in);
      }
      break;
    }
    case therisodilattime:{
      if (Mespr==1)  fprintf (stdout,"\n number of isotropic thermal dilatancy materials with time dependent temperature  %ld",numtype);
      tidilattime = new therisomattime [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of isotropic thermal dilatancy material with time dependent temperature",__FILE__, __LINE__,__func__);
        tidilattime[k-1].read (in);
      }
      break;
    }

    case nonlocplastmat:{
      if (Mespr==1)  fprintf (stdout,"\n number of nonlocal plasticity materials   %ld",numtype);
      nlplast = new nonlocplast [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of nonlocal plasticity material",__FILE__,__LINE__,__func__);
        nlplast[k-1].read (in);
      }
      // nonlocal material model flag is set
      Mp->matmodel = nonlocal;
      break;
    }
    case nonlocdamgmat:{
      if (Mespr==1)  fprintf (stdout,"\n number of nonlocal damage materials   %ld",numtype);
      nldamg = new nonlocdamg [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of nonlocal damage material", __FILE__,__LINE__,__func__);
        nldamg[k-1].read (in);
      }
      // nonlocal material model flag is set
      Mp->matmodel = nonlocal;
      break;
    }

    case damage_plasticity:{
      if (Mespr==1)  fprintf (stdout,"\n number of artificial material damage-plasticity  %ld",numtype);
      dampl = new damplast [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of artificial material damge-plasticity",__FILE__,__LINE__,__func__);
      }
      break;
    }

    case viscoplasticity:{
      if (Mespr==1)  fprintf (stdout,"\n number of artificial material of viscoplasticity  %ld",numtype);
      visplas = new visplast [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of artificial material of viscoplasticity",__FILE__,__LINE__,__func__);
      }
      break;
    }

    case elasttimemat:{
      if (Mespr==1)  fprintf (stdout,"\n number of artificial elastic isotropic material for time dependent problems %ld",numtype);
      eltimemat = new elasttime [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of artificial elastic isotropic material for time dependent problems",__FILE__,__LINE__,__func__);
	eltimemat[k-1].read (in);
      }
      break;
    }
      
    case rcmatmodnorm:{
      if (Mespr==1)  fprintf (stdout,"\n number of artificial material of viscoplasticity  %ld",numtype);
      normm = new normmat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of artificial material of viscoplasticity",__FILE__,__LINE__,__func__);
	normm[k-1].read (in);
      }
      break;
    }
    
    case lenjonespot:{
      if (Mespr==1)  fprintf (stdout,"\n number of Lennard-Jones interatomic potentials   %ld",numtype);
      lenjon = new lenjonesmat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of Lennard-Jones interatomic potentials",__FILE__,__LINE__,__func__);
	lenjon[k-1].read (in);
      }
      break;
    }
    
    case contmat:{
      if (Mespr==1)  fprintf (stdout,"\n number of materials for contact problems   %ld",numtype);
      conmat = new contactmat [numtype];
      for (j=0;j<numtype;j++){
        k=numtype+1;
        xfscanf (in,"%ld",&k);
        if (k>numtype || k<1)
          print_err("wrong number of materials for contact problems",__FILE__,__LINE__,__func__);
	conmat[k-1].read (in);
      }
      break;
    }
      
    default:
      print_err("unknown material type is required",__FILE__,__LINE__,__func__);

    }
  }
}



/**
   Function reads integration points, material characteristics,
   auxiliary points for strain and stress computation.
   
   @param in - input stream
   
   21.7.2001
*/
void mechmat::read (XFILE *in)
{

  //  reading of material characteristics
  readmatchar (in);

  //  computation of number of all integration points
  tnip = intpnum ();
  if (Mespr==1){
    fprintf (stdout,"\n number of integration points  %ld",tnip);
  }

  //  allocation of integration points
  intpointalloc ();

  //  reading of integration points
  //readip (in);
  intpointinit ();

/*
  //  reading of strain and stress points
  if (Mp->straincomp==1){
    stra.read(in);
  }
  if (Mp->stresscomp==1){
    stre.read(in);
  }
  */
}



/**
   Function returns actual value of the Young modulus for given material and
   in the required integration point.
   
   @param ipp - number of integration point
   @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
   @param ido - index of internal variables in the ip's ipp eqother array. The standard value is zero.
   
   Returns:
     Function returns value of Young moduls.

   TKo, 8.8.2005
*/
double mechmat::give_actual_ym(long ipp, long im, long ido)
{
  long ncompo;
  double e = 0.0;

  switch (ip[ipp].tm[im])
  {
    case elisomat:
      e = eliso[ip[ipp].idm[im]].e;
      break;
    case elasttimemat:
      e = eltimemat[ip[ipp].idm[im]].actual_modulus();
      break;
    case winklerpasternak:
    case rcmatmodnorm:
    case simplas1d:
    case jflow:
    case jflow2:
    case microplaneM4:
    case microsimp:
    case microfibro:
    case mohrcoul:
    case mohrcoulparab:
    case boermaterial:
    case druckerprager:
    case chenplast:
    case modcamclaymat:
    case shefpl:
    case glasgowmechmat:
    case glasgowdamage:
    case simvisc:
    case simvisplas:
    case lemaitr:
    case scaldamage:
    case scaldamagecc:
    case scaldamage_time:
    case ortodamage:
    case ortodamagerot:
    case anisodamage:
    case anisodamagerot:
    case creepdpl:
    case creepbaz:
    case creepeffym:
    case consolidation:
      ncompo = givencompeqother(ipp, im); // this material should be last one before elastic
      e = give_actual_ym(ipp, im+1, ido+ncompo);
      break;
    case damage_plasticity:
      ncompo = givencompeqother(ipp, im+1); // ido index shift of the first nonartificial material
      e= give_actual_ym(ipp, im+2, ido+ncompo); // Young modulus will be obtained from the second nonartificial material
      break;
    case viscoplasticity:
      ncompo = givencompeqother(ipp, im+1); // ido index shift of the first nonartificial material
      e = give_actual_ym(ipp, im+2, ido+ncompo); // Young modulus will be obtained from the second nonartificial material
      break;
    case nonlocplastmat:
    case nonlocdamgmat:
    case nonlocalmicroM4:
    case creep_damage:
      e = give_actual_ym(ipp, im+1, ido); // artificial material has no internal variables -> no ido index shift
      break;
    case creepb3:
    case creeprs:
      e = creep_give_actual_ym(ipp, im, ido);
      break;
    case time_switchmat:
      e = tswmat[ip[ipp].idm[im]].give_actual_ym(ipp, im, ido);
      break;
    default:
      print_err("unknown elastic material type is required",__FILE__,__LINE__,__func__);
  }

  return e;
}



/**
   Function returns initial value of the Young modulus for given material and
   in the required integration point.
   
   @param ipp - number of integration point
   @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
   @param ido - index of internal variables in the ip's ipp eqother array. The standard value is zero.

   Returns:
     Function returns value of initial Young modulus.

   TKo, 8.2008
*/
double mechmat::give_initial_ym(long ipp, long im, long ido)
{
  long ncompo;
  double e = 0.0;

  switch (ip[ipp].tm[im])
  {
    case elisomat:
      e = eliso[ip[ipp].idm[im]].e;
      break;
    case elasttimemat:
      e = eltimemat[ip[ipp].idm[im]].initial_modulus();
      break;
    case winklerpasternak:
    case rcmatmodnorm:
    case simplas1d:
    case jflow:
    case jflow2:
    case microplaneM4:
    case microsimp:
    case microfibro:
    case mohrcoul:
    case mohrcoulparab:
    case boermaterial:
    case druckerprager:
    case chenplast:
    case modcamclaymat:
    case shefpl:
    case glasgowmechmat:
    case glasgowdamage:
    case simvisc:
    case simvisplas:
    case lemaitr:
    case scaldamage:
    case scaldamagecc:
    case scaldamage_time:
    case ortodamage:
    case ortodamagerot:
    case anisodamage:
    case anisodamagerot:
    case creepdpl:
    case creepbaz:
    case creepeffym:
    case consolidation:
      ncompo = givencompeqother(ipp, im); // this material should be last one before elastic
      e = give_initial_ym(ipp, im+1, ido+ncompo);
      break;
    case damage_plasticity:
      ncompo = givencompeqother(ipp, im+1); // ido index shift of the first nonartificial material
      e= give_initial_ym(ipp, im+2, ido+ncompo); // Young modulus will be obtained from the second nonartificial material
      break;
    case viscoplasticity:
      ncompo = givencompeqother(ipp, im+1); // ido index shift of the first nonartificial material
      e = give_initial_ym(ipp, im+2, ido+ncompo); // Young modulus will be obtained from the second nonartificial material
      break;
    case nonlocplastmat:
    case nonlocdamgmat:
    case nonlocalmicroM4:
    case creep_damage:
      e = give_initial_ym(ipp, im+1, ido); // artificial material has no internal variables -> no ido index shift
      break;
    case creepb3:
    case creeprs:
      e = creep_compute_inital_ym(ipp, im, ido);
      break;
    case time_switchmat:
      e = tswmat[ip[ipp].idm[im]].give_initial_ym(ipp, im, ido);
      break;
    default:
      print_err("unknown elastic material type is required",__FILE__,__LINE__,__func__);
  }
  return e;
}



/**
   Function returns actual value of the Poissons ratio for given material and
   in the required integration point.
   
   @param ipp - number of integration point
   @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
   @param ido - index of internal variables in the ip's ipp eqother array. The standard value is zero.
   
   Returns:
    Function returns value of Poissons ratio.

   TKo, 7.2008
*/
double mechmat::give_actual_nu(long ipp, long im, long ido)
{
  long ncompo;
  double nu = 0.0;

  switch (ip[ipp].tm[im])
  {
    case elisomat:
      nu = eliso[ip[ipp].idm[im]].nu;
      break;
    case elasttimemat:
      nu = eltimemat[ip[ipp].idm[im]].nu;
      break;
    case winklerpasternak:
    case rcmatmodnorm:
    case simplas1d:
    case jflow:
    case jflow2:
    case microplaneM4:
    case microsimp:
    case microfibro:
    case mohrcoul:
    case mohrcoulparab:
    case boermaterial:
    case druckerprager:
    case chenplast:
    case modcamclaymat:
    case shefpl:
    case glasgowmechmat:
    case glasgowdamage:
    case simvisc:
    case simvisplas:
    case lemaitr:
    case scaldamage:
    case scaldamagecc:
    case scaldamage_time:
    case ortodamage:
    case ortodamagerot:
    case anisodamage:
    case anisodamagerot:
    case creepdpl:
    case creepbaz:
    case creepb3:
    case creeprs:
    case creepeffym:
    case consolidation:
      ncompo = givencompeqother(ipp, im); // this material should be last one before elastic
      nu = give_actual_nu(ipp, im+1, ido+ncompo);
      break;
    case damage_plasticity:
      ncompo = givencompeqother(ipp, im+1); // ido index shift of the first nonartificial material
      nu = give_actual_nu(ipp, im+2, ido+ncompo); // Young modulus will be obtained from the second nonartificial material
      break;
    case viscoplasticity:
      ncompo = givencompeqother(ipp, im+1); // ido index shift of the first nonartificial material
      nu = give_actual_nu(ipp, im+2, ido+ncompo); // Young modulus will be obtained from the second nonartificial material
      break;
    case creep_damage:
      ncompo = givencompeqother(ipp, im+1); // ido index shift of the first nonartificial material
      nu = give_actual_nu(ipp, im+2, ido+ncompo); // Young modulus will be obtained from the second nonartificial material
      break;
    case nonlocplastmat:
    case nonlocdamgmat:
    case nonlocalmicroM4:
      nu = give_actual_nu(ipp, im+1, ido); // artificial material has no internal variables -> no ido index shift
      break;
    case time_switchmat:
      nu = tswmat[ip[ipp].idm[im]].give_actual_nu(ipp, im, ido);
      break;
    default:
      print_err("unknown elastic material type is required",__FILE__,__LINE__,__func__);
  }
  return nu;
}



/**
   Function returns actual value of the tensile strenght for given material and
   in the required integration point.
   
   @param ipp - number of integration point
   @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
   @param ido - index of internal variables in the ip's ipp eqother array. The standard value is zero.

   Returns:
     Function returns tensile strength.
   
   TKo, 8.8.2005
*/
double mechmat::give_actual_ft(long ipp, long im, long ido)
{
  double ft = 0.0;

  switch (ip[ipp].tm[im])
  {
    case scaldamage:
      ft = scdam[ip[ipp].idm[im]].give_actual_ft(ipp, im, ido);
      break;
    case scaldamage_time:
      ft = scdamtime[ip[ipp].idm[im]].give_actual_ft(ipp, im, ido);
      break;
    case ortodamage:
      ft = ortdam[ip[ipp].idm[im]].give_actual_ft(ipp);
      break;
    case ortodamagerot:
      ft = ortdamrot[ip[ipp].idm[im]].give_actual_ft(ipp);
      break;
    case anisodamage:
      ft = anidam[ip[ipp].idm[im]].give_actual_ft(ipp);
      break;
    case anisodamagerot:
      ft = anidamrot[ip[ipp].idm[im]].give_actual_ft(ipp);
      break;
    case nonlocdamgmat:
      ft = give_actual_ft(ipp,im+1,ido);
      break;
    case creep_damage:
      ft = crdam[ip[ipp].idm[im]].give_actual_ft(ipp, im, ido);
      break;
    case creepb3:
    case creeprs:
      ft = creep_give_actual_ft(ipp, im, ido);
      break;
    case creepeffym:
      ft = creffym[ip[ipp].idm[im]].give_actual_ft(ipp, im, ido);
      break;
    case time_switchmat:
      ft = tswmat[ip[ipp].idm[im]].give_actual_ft(ipp, im, ido);
      break;
    default:
      print_err("unknown material type is required",__FILE__,__LINE__,__func__);
  }
  return ft;
}



/**
   Function returns actual value of the compressive strenght for given material and
   in the required integration point.
   
   @param ipp - number of integration point
   @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
   @param ido - index of internal variables in the ip's ipp eqother array. The standard value is zero.

   Returns:
     Function returns tensile strength.
   
   TKo, 8.8.2005
*/
double mechmat::give_actual_fc(long ipp, long im, long ido)
{
  double fc = 0.0;

  switch (ip[ipp].tm[im])
  {
    case ortodamage:
      fc = ortdam[ip[ipp].idm[im]].give_actual_fc(ipp);
      break;
    case ortodamagerot:
      fc = ortdamrot[ip[ipp].idm[im]].give_actual_fc(ipp);
      break;
    case anisodamage:
      fc = anidam[ip[ipp].idm[im]].give_actual_fc(ipp);
      break;
    case anisodamagerot:
      fc = anidamrot[ip[ipp].idm[im]].give_actual_fc(ipp);
      break;
    case nonlocdamgmat:
      fc = give_actual_fc(ipp,im+1,ido);
      break;
    case creep_damage:
      fc = crdam[ip[ipp].idm[im]].give_actual_fc(ipp, im, ido);
      break;
    default:
      print_err("unknown material type is required",__FILE__,__LINE__,__func__);
  }
  return fc;
}



/**
   Function computes stiffness %matrix of the material
   in the required integration point.
   
   @param d - stiffness %matrix
   @param ipp - number of integration point
   @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
   @param ido - index of internal variables in the ip's ipp other array
   
   JK, 17.7.2001
*/
void mechmat::matstiff (matrix &d,long ipp,long im,long ido)
{
  long i;
  switch (ip[ipp].tm[im]){
  case elisomat:{
    i=ip[ipp].idm[im];
    eliso[i].matstiff (d,ip[ipp].ssst);
    break;
  }
  case simplas1d:{
    i=ip[ipp].idm[im];
    spl1d[i].matstiff (d,ipp,ido);
    break;
  }
  case jflow:{
    i=ip[ipp].idm[im];
    j2f[i].matstiff (d,ipp,ido);
    break;
  }
  case jflow2:{
    i=ip[ipp].idm[im];
    j2f2[i].matstiff (d,ipp,ido);
    break;
  }
  case microplaneM4:{
    i=ip[ipp].idm[im];
    mpM4[i].matstiff (d,ipp,ido);
    break;
  }
  case microsimp:{
    i=ip[ipp].idm[im];
    mpSIM[i].matstiff (d,ipp,ido);
    break;
  }
  case microfibro:{
    i=ip[ipp].idm[im];
    mpfib[i].matstiff (d,ipp,ido);
    break;
  }
  case mohrcoul:{
    i=ip[ipp].idm[im];
    mcoul[i].matstiff (d,ipp,ido);
    break;
  }
  case mohrcoulparab:{
    i=ip[ipp].idm[im];
    mcpar[i].matstiff (d,ipp,ido);
    break;
  }
  case boermaterial:{
    i=ip[ipp].idm[im];
    boerm[i].matstiff (d,ipp,ido);
    break;
  }
  case druckerprager:{
    i=ip[ipp].idm[im];
    drprm[i].matstiff (d,ipp,ido);
    break;
  }
  case chenplast:{
    i=ip[ipp].idm[im];
    chplast[i].matstiff (d,ipp,ido);
    break;
  }
  case modcamclaymat:{
    i=ip[ipp].idm[im];
    cclay[i].matstiff (d,ipp,ido);
    break;
  }
  case shefpl:{
    i=ip[ipp].idm[im];
    shpl[i].matstiff (d,ipp,ido);
    break;
  }
  case glasgowmechmat:{
    i=ip[ipp].idm[im];
    glasgmat[i].matstiff (d,ipp,ido);
    break;
  }
  case glasgowdamage:{
    i=ip[ipp].idm[im];
    glasgdam[i].matstiff (d,ipp,ido);
    break;
  }
  case creep_damage:{
    i=ip[ipp].idm[im];
    crdam[i].matstiff (d,ipp,im,ido);
    break;
  }
  case time_switchmat:{
    i=ip[ipp].idm[im];
    tswmat[i].matstiff (d,ipp,im,ido);
    break;
  }
  case simvisplas:{
    i=ip[ipp].idm[im];
    svipl[i].matstiff (d,ipp,ido);
    break;
  }
  case lemaitr:{
    i=ip[ipp].idm[im];
    lmtr[i].matstiff (d,ipp,ido);
    break;
  }
  case scaldamage:{
    i=ip[ipp].idm[im];
    scdam[i].matstiff (d,ipp,ido);
    break;
  }
  case scaldamagecc:{
    i=ip[ipp].idm[im];
    scdamcc[i].matstiff (d,ipp,ido);
    break;
  }
  case scaldamage_time:{
    i=ip[ipp].idm[im];
    scdamtime[i].matstiff (d,ipp,ido);
    break;
  }
  case ortodamage:{
    i=ip[ipp].idm[im];
    ortdam[i].matstiff (d,ipp,ido);
    break;
  }
  case ortodamagerot:{
    i=ip[ipp].idm[im];
    ortdamrot[i].matstiff (d,ipp,ido);
    break;
  }
  case anisodamage:{
    i=ip[ipp].idm[im];
    anidam[i].matstiff (d,ipp,ido);
    break;
  }
  case anisodamagerot:{
    i=ip[ipp].idm[im];
    anidamrot[i].matstiff (d,ipp,ido);
    break;
  }
  case graphm:{
    i=ip[ipp].idm[im];
    grmat[i].matstiff (d,ipp);
    break;
  }
  case graphmtime:{
    i=ip[ipp].idm[im];
    grmattime[i].matstiff (d,ipp);
    break;
  }
  case geoelast:{
    i=ip[ipp].idm[im];
    geoel[i].matstiff (d,ipp,ido);
    break;
  }
  case creeprs:
  case creepb3:{
    creep_matstiff (d,ipp,im,ido);
    break;
  }
  case creepdpl:{
    creep_matstiff (d,ipp,im,ido);
    break;
  }
  case creepbaz:{
    i=ip[ipp].idm[im];
    crbaz[i].matstiff (d,ipp);
    break;
  }
  case creepeffym:{
    i=ip[ipp].idm[im];
    creffym[i].matstiff (d,ipp,im,ido);
    break;
  }
  case consolidation:{
    i=ip[ipp].idm[im];
    csol[i].matstiff (d,ipp);
    break;
  }
  case winklerpasternak:{
    i=ip[ipp].idm[im];
    wpast[i].matstiff (d,ipp);
    break;
  }

  case nonlocdamgmat:{
    matstiff (d,ipp,im+1,ido);
    break;
  }
  case nonlocplastmat:{
    matstiff (d,ipp,im+1,ido);
    break;
  }
  case nonlocalmicroM4:{
    matstiff (d,ipp,im+1,ido);
    break;
  }

  case damage_plasticity:{
    i=ip[ipp].idm[im];
    dampl[i].matstiff (d,ipp);
    break;
  }

  case viscoplasticity:{
    i=ip[ipp].idm[im];
    visplas[i].matstiff (d,ipp,im,ido);
    break;
  }

  case elasttimemat:{
    i=ip[ipp].idm[im];
    eltimemat[i].matstiff (d,ip[ipp].ssst);
    break;
  }
  case rcmatmodnorm:{
    i=ip[ipp].idm[im+1];
    eliso[i].matstiff (d,ip[ipp].ssst);
    break;
  }

  case contmat:{
    i=ip[ipp].idm[im+1];
    conmat[i].matstiff (d,ipp);
    break;
  }

  default:
    print_err("unknown material type is required",__FILE__,__LINE__,__func__);

  }

}



/**
   Function computes elastic stiffness %matrix of the material
   in the required integration point.
   
   @param d - stiffness %matrix
   @param ipp - number of integration point
   
   JK, 17.7.2001
   TKo 7.2008
*/
void mechmat::elmatstiff (matrix &d,long ipp)
  //  27.10.2001
{
  long i,idem;

  idem = ip[ipp].gemid();
  switch (ip[ipp].tm[idem]){
  case elisomat:{
    i=ip[ipp].idm[idem];
    eliso[i].elmatstiff (d,ip[ipp].ssst);
    break;
  }
  case elasttimemat:{
    i=ip[ipp].idm[idem];
    eltimemat[i].elmatstiff (d,ip[ipp].ssst);
    break;
  }
  case elgmat3d:{
    i=ip[ipp].idm[idem];
    elgm3d[i].elmatstiff (d);
    break;
  }
  default:
    print_err("Unknown elastic material type is required",__FILE__,__LINE__,__func__);

  }

}



/**
   Function computes elastic compliance %matrix of the material
   in the required integration point.
   
   @param c - compliance %matrix
   @param ipp - number of integration point
   
   JK, 3.2.2003
*/
void mechmat::elmatcompl (matrix &c,long ipp)
  //  27.10.2001
{
  long i, idem;

  idem = ip[ipp].gemid();
  switch (ip[ipp].tm[idem]){
  case elisomat:{
    i=ip[ipp].idm[idem];
    eliso[i].matcompl (c,ip[ipp].ssst);
    break;
  }
  case elasttimemat:{
    i=ip[ipp].idm[idem];
    eltimemat[i].matcompl (c,ip[ipp].ssst);
    break;
  }
  default:
    print_err("unknown elastic material type is required",__FILE__,__LINE__,__func__);

  }

}


// *****************************************************************
// *****************************************************************
//   PART CONTAINING FUNCTIONS DEALING WITH PLASTICITY
// *****************************************************************
// *****************************************************************



/**
   Function evaluates yield function for stresses and internal variables.
   
   @param ipp - integration point pointer
   @param idpm - id of plasticity model
   @param sig - stress components
   @param q - internal variables (hardening parameters)
   
   28.10.2001
*/
double mechmat::yieldfunction (long ipp, long idpm, matrix &sig,vector &q)
{
  double f;
  
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    f = spl1d[ip[ipp].idm[idpm]].yieldfunction (sig,q);
    break;
  }
  case jflow:{
    f = j2f[ip[ipp].idm[idpm]].yieldfunction (sig,q);
    break;
  }
  case jflow2:{
    f = j2f2[ip[ipp].idm[idpm]].yieldfunction (sig,q);
    break;
  }
  case mohrcoulparab:{
    f = mcpar[ip[ipp].idm[idpm]].yieldfunction (sig);
    break;
  }
  case boermaterial:{
    f = boerm[ip[ipp].idm[idpm]].yieldfunction (sig);
    break;
  }
  case druckerprager:{
    f = drprm[ip[ipp].idm[idpm]].yieldfunction (sig,q);
    break;
  }
  case chenplast:{
    f = chplast[ip[ipp].idm[idpm]].yieldfunction (sig,q);
    break;
  }
  case modcamclaymat:{
    f = cclay[ip[ipp].idm[idpm]].yieldfunction (sig,q);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
  return f;
}



/**
   Function evaluates derivates of yield function with respect to stresses.
   Stresses must be assembled in tensor notation and stored in 3x3 %matrix.
   Function overwrites array sig, some material models use the array sig for auxiliary computations.
   Function stores derivatives in %vector dfds.

   @param ipp - integration point pointer
   @param idpm - id of plasticity model
   @param sig - stress components
   @param q - internal parameters (hardening parameters)
   @param dfds - derivatives of yield function with respect to stresses
   
   28.10.2001
*/
void mechmat::dfdsigma (long ipp, long idpm, matrix &sig, vector &q, vector &dfds)
{
  matrix dfdst(3,3);
  
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    spl1d[ip[ipp].idm[idpm]].dfdsigma (sig,dfdst);
    break;
  }
  case jflow:{
    j2f[ip[ipp].idm[idpm]].deryieldfsigma (sig,dfdst);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].deryieldfsigma (sig,dfdst);
    break;
  }
  case mohrcoulparab:{
    mcpar[ip[ipp].idm[idpm]].deryieldfsigma (sig,dfdst);
    break;
  }
  case boermaterial:{
    boerm[ip[ipp].idm[idpm]].deryieldfsigma (sig,dfdst);
    break;
  }
  case druckerprager:{
    drprm[ip[ipp].idm[idpm]].deryieldfsigma (sig,dfdst);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].deryieldfdsigma (sig,q,dfdst);
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].deryieldfsigma (sig, q, dfdst);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
  
  //  conversion of tensor notation to vector notation
  tensor_vector (dfds,dfdst,ip[ipp].ssst,stress);
  
}



/**
   Function evaluates derivates of plastic potential with respect to stresses.
   Stresses must be assembled in tensor notation and stored in 3x3 %matrix.
   Function overwrites array sig, some material models use the array sig for auxiliary computations.
   Function stores derivatives in %vector dgds.

   @param ipp - integration point pointer
   @param idpm - id of plasticity model
   @param sig - stress components
   @param q - internal parameters (hardening parameters)
   @param dfds - derivatives of plastic potential with respect to stresses
   
   28.10.2001
*/
void mechmat::dgdsigma (long ipp, long idpm, matrix &sig, vector &q, vector &dgds)
{
  matrix dgdst(3,3);
  
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    spl1d[ip[ipp].idm[idpm]].dfdsigma (sig,dgdst);
    break;
  }
  case jflow:{
    j2f[ip[ipp].idm[idpm]].deryieldfsigma (sig,dgdst);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].deryieldfsigma (sig,dgdst);
    break;
  }
  case mohrcoulparab:{
    mcpar[ip[ipp].idm[idpm]].derplaspotsigma (sig,dgdst);
    break;
  }
  case boermaterial:{
    boerm[ip[ipp].idm[idpm]].deryieldfsigma (sig,dgdst);
    break;
  }
  case druckerprager:{
    drprm[ip[ipp].idm[idpm]].deryieldfsigma (sig,dgdst);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].deryieldfdsigma (sig,q,dgdst);
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].deryieldfsigma (sig, q, dgdst);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
  
  //  conversion of tensor notation to vector notation
  tensor_vector (dgds,dgdst,ip[ipp].ssst,stress);
  
}



/**
   Function evaluates second derivates of yield function with respect to stresses.   
   Stresses must be assembled in tensor notation and stored in 3x3 %matrix,
   function stores derivatives in %matrix dfdsds.
   
   @param ipp - integration point pointer
   @param idpm - id of plasticity model
   @param sig - stress components
   @param dfdsds - second derivatives of yield function with respect to stresses
   
   28.10.2001
*/
void mechmat::dfdsigmadsigma (long ipp, long idpm, matrix &sig, matrix &dfdsds)
{
  matrix dfdsdst(6,6);
  
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    spl1d[ip[ipp].idm[idpm]].dfdsigmadsigma (dfdsdst);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].deryielddsigmadsigma (sig,dfdsdst);
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].dderyieldfsigma (dfdsdst);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].deryieldfdsigmadsigma (sig,dfdsdst);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
  
  //  conversion from tensor notation to matrix notation
  tensor4_matrix (dfdsds,dfdsdst,ip[ipp].ssst);
  
}



/**
   Function evaluates second derivates of plastic potential with respect to stresses.
   Stresses must be assembled in tensor notation and stored in 3x3 %matrix,
   function stores derivatives in %matrix dgdsds.
   
   @param ipp - integration point pointer
   @param idpm - id of plasticity model
   @param sig - stress components
   @param dfdsds - second derivatives of plastic potential with respect to stresses
   
   28.10.2001
*/
void mechmat::dgdsigmadsigma (long ipp, long idpm, matrix &sig, matrix &dgdsds)
{
  matrix dgdsdst(6,6);
  
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    spl1d[ip[ipp].idm[idpm]].dfdsigmadsigma (dgdsdst);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].deryielddsigmadsigma (sig,dgdsdst);
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].dderyieldfsigma (dgdsdst);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].deryieldfdsigmadsigma (sig,dgdsdst);
    break;
  }
  default:
    print_err("unknown plasticity model is required", __FILE__, __LINE__, __func__);

  }
  
  //  conversion from tensor notation to matrix notation
  tensor4_matrix (dgdsds,dgdsdst,ip[ipp].ssst);
  
}



/**
   Function evaluates derivates of yield function with respect to internal variables (hardening parameters).
   Stresses must be assembled in tensor notation and stored in 3x3 %matrix,
   function stores derivatives in %vector dq.
   
   @param ipp - integration point pointer
   @param idpm - id of plasticity model
   @param sig - stress components
   @param q - internal variables (hardening parameters)
   @param dq - derivatives of yield function with respect to internal variables
   
   28.10.2001
*/
void mechmat::dfdqpar (long ipp, long idpm,matrix &sig,vector &q,vector &dq)
{
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    spl1d[ip[ipp].idm[idpm]].dfdqpar (dq);
    break;
  }
  case jflow:{
    j2f[ip[ipp].idm[idpm]].deryieldfq (dq);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].deryieldfq (dq);
    break;
  }
  case mohrcoulparab:{
    break;
  }
  case boermaterial:{
    break;
  }
  case druckerprager:{
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].deryieldfq (sig, dq);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].deryieldfdq (sig,q,dq);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
}



/**
   Function evaluates derivates of plastic potential with respect to internal variables (hardening parameters).
   Stresses must be assembled in tensor notation and stored in 3x3 %matrix,
   function stores derivatives in %vector dq.
   
   @param ipp - integration point pointer
   @param idpm - id of plasticity model
   @param sig - stress components
   @param q - internal variables (hardening parameters)
   @param dq - derivatives of plastic potential with respect to internal variables
   
   28.10.2001
*/
void mechmat::dgdqpar (long ipp, long idpm,matrix &sig,vector &q,vector &dq)
{
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    spl1d[ip[ipp].idm[idpm]].dfdqpar (dq);
    break;
  }
  case jflow:{
    j2f[ip[ipp].idm[idpm]].deryieldfq (dq);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].deryieldfq (dq);
    break;
  }
  case mohrcoulparab:{
    break;
  }
  case boermaterial:{
    break;
  }
  case druckerprager:{
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].deryieldfq (sig, dq);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].deryieldfdq (sig,q,dq);
    break;
  }
  default:
    print_err ("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
}



/**
   Function evaluates derivates of yield function with respect to stresses and
   internal variables (hardening parameters).

   Stresses must be assembled in tensor notation and stored in 3x3 %matrix,
   function stores derivatives in %matrix dfdsdq.
   
   @param ipp - integration point pointer
   @param idpm - id of plasticity model
   @param sig - stress components
   @param q - internal variables (hardening parameters)
   @param dfdsdq - derivatives of yield function with respect to stresses and internal variables
   
   28.10.2001
*/
void mechmat::dfdsigmadq (long ipp, long idpm,matrix &sig,vector &q,matrix &dfdsdq)
{
  //  number of hardening parameters
  long nh=dfdsdq.n;
  
  matrix dfdsdqt(6,nh);

  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    spl1d[ip[ipp].idm[idpm]].dfdsigmadq (dfdsdqt);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].deryieldfdsigmadq (dfdsdqt);
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].deryieldfdsdq (dfdsdqt);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].deryieldfdsigmadq (sig,q,dfdsdqt);
    break;
  }
  default:
    print_err("unknown plasticity model is required", __FILE__,__LINE__,__func__);

  }
  
  //  conversion from tensorial notation to matrix notation
  gentensor_matrix (dfdsdq,dfdsdqt,ip[ipp].ssst);

}



/**
   Function evaluates derivates of plastic potential with respect to stresses and
   internal variables (hardening parameters).

   Stresses must be assembled in tensor notation and stored in 3x3 %matrix,
   function stores derivatives in %matrix dgdsdq.
   
   @param ipp - integration point pointer
   @param idpm - id of plasticity model
   @param sig - stress components
   @param q - internal variables (hardening parameters)
   @param dgdsdq - derivatives of plastic potential with respect to stresses and internal variables
   
   28.10.2001
*/
void mechmat::dgdsigmadq (long ipp, long idpm,matrix &sig,vector &q,matrix &dgdsdq)
{
  //  number of hardening parameters
  long nh=dgdsdq.n;
  
  matrix dgdsdqt(6,nh);

  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    spl1d[ip[ipp].idm[idpm]].dfdsigmadq (dgdsdqt);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].deryieldfdsigmadq (dgdsdqt);
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].deryieldfdsdq (dgdsdqt);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].deryieldfdsigmadq (sig,q,dgdsdqt);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
  
  //  conversion from tensorial notation to matrix notation
  gentensor_matrix (dgdsdq,dgdsdqt,ip[ipp].ssst);

}



/**
   Function computes derivatives of hardening function with respect to stresses.
   
   @param ipp - id of integration point
   @param idpm - id of plasticity model
   @param sigt - stress components stored in 3x3 %matrix
   @param q - internal variables stored in %vector
   @param dhds - derivatives of hardening function with respect to stresses stored in 6 x ncomphard %matrix
   
   JK, 7.8.2005
*/
void mechmat::dhdsigma (long ipp,long idpm,matrix &sigt,vector &q,matrix &dhds)
{
  //  number of hardening parameters
  long nh=dhds.m;
  
  matrix dhdst(6,nh);

  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    //spl1d[ip[ipp].idm[idpm]].dfdsigmadq (dgdsdqt);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].dhdsigma (sigt,q,dhdst);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }

  //  conversion from tensorial notation to matrix notation
  gentensor_matrix (dhds,dhdst,ip[ipp].ssst);
}



/**
   Function computes derivatives of hardening function with respect to consistency parameter.
   
   @param ipp - id of integration point
   @param idpm - id of plasticity model
   @param dhdc - derivatives of hardening function with respect to consistency parameter stored in ncomphard x 1 %vector
   
   JK, 7.8.2005
*/
void mechmat::dhdgamma (long ipp,long idpm,vector &epsp,matrix &sig,vector &q,vector &dhdc)
{
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    spl1d[ip[ipp].idm[idpm]].dhdgamma (ipp,epsp,sig,dhdc);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].dhdgamma (dhdc);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
}



/**
   Function computes derivatives of hardening function with respect to hardening parameters.
   
   @param ipp - id of integration point
   @param idpm - id of plasticity model
   @param sigt - stress components stored in 3x3 %matrix
   @param q - internal variables stored in %vector
   @param dhdq - derivatives of hardening function with respect to hardening parameters stored in ncomphard x ncomphard %matrix
   
   JK, 7.8.2005
*/
void mechmat::dhdqpar (long ipp,long idpm,matrix &sigt,vector &q,matrix &dhdq)
{
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    //spl1d[ip[ipp].idm[idpm]].dfdsigmadq (dgdsdqt);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].dhdqpar (sigt,q,dhdq);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
}



void mechmat::plasticmoduli (long ipp,long idpm,vector &epsp,matrix &sig,matrix &h)
{
  switch (ip[ipp].tm[idpm]){
  case jflow:{
    j2f[ip[ipp].idm[idpm]].plasmod (h);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].plasmod (h);
    break;
  }
  case chenplast:{
    //chplast[ip[ipp].idm[idpm]].plasmod (h);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }

}



/*
void mechmat::dfunctdqdq (long ipp, long idpm,matrix &sig,vector &q,matrix &dfdqdq)
{
  switch (ip[ipp].tm[idpm]){
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].deryieldfdqdq (dfdqdq);
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].deryieldfdqdq (dfdqdq);
    break;
  }
  case chenplast:{
    //chplast[ip[ipp].idm[idpm]].deryieldfdqdq (dfdqdq);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown plasticity model is required in function dfdqdq (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
}
*/



void mechmat::dhardfdq (long ipp, long idpm,long ido, double dgamma, vector &q,matrix &dqdq)
{
  switch (ip[ipp].tm[idpm]){
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].dhardfdq (ipp, ido, dgamma, q, dqdq);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
}



void mechmat::dqdsigma (long ipp, long idpm,matrix &sig,vector &q,matrix &dqds)
{
  switch (ip[ipp].tm[idpm]){
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].dqdsigma (sig, q, dqds);
    break;
  }
  default:{
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);
  }
  }
}



/**
  Function assembleses contribution from hardening to the denominator in cutting plane method.

  @param ipp - integration point id
  @param idpm - id of plastic material
  @param ido - index of internal variables for given material in the ipp other array
  @param gamma - consistency parameter
  @param sig  - %vector of actial stresses
  @param eps  - %vector of actual strains
  @param epsp - %vector of actual plastic strains
  @param qtr  - %vector of trial hardening parameters

  Retunrs:
    Function returns contribution from hardening to the denominator in cutting plane method.
*/
/*
double mechmat::plasmodscalar(long ipp, long idpm, long ido, matrix &sig, vector &eps, vector &epsp, vector &qtr,double gamma)
{
  double ret = 0.0;
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    //ret = spl1d[ip[ipp].idm[idpm]].plasmodscalar (qtr);
    ret = spl1d[ip[ipp].idm[idpm]].plasmodscalar (sig,epsp,qtr,gamma);
    break;
  }
  case jflow:{
    ret = j2f[ip[ipp].idm[idpm]].plasmodscalar (qtr);
    break;
  }
  case jflow2:{
    ret = j2f2[ip[ipp].idm[idpm]].plasmodscalar (qtr);
    break;
  }
  case mohrcoulparab:{
    break;
  }
  case boermaterial:{
    break;
  }
  case druckerprager:{
    ret = drprm[ip[ipp].idm[idpm]].plasmodscalar(qtr);
    break;
  }
  case chenplast:{
    ret = chplast[ip[ipp].idm[idpm]].plasmodscalar (ipp,sig,epsp,qtr);
    //ret = 0.0;
    break;
  }
  case modcamclaymat:{
    ret = cclay[ip[ipp].idm[idpm]].plasmodscalar(ipp, ido, sig, qtr);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown plasticity model is required in function");
    fprintf (stderr,"\n plasmodscalar (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return ret;
}
*/



/**
  Function assembleses contribution from hardening to the denominator in cutting plane method.

  @param denom - resulting contribution from hardening (output parameter)
  @param ipp - integration point id
  @param idpm - id of plastic material
  @param ido - index of internal variables for given material in the ipp other array
  @param gamma - consistency parameter
  @param sig  - %vector of actial stresses
  @param eps  - %vector of actual strains
  @param epsp - %vector of actual plastic strains
  @param qtr  - %vector of trial hardening parameters
*/
void mechmat::plasmodscalar(double &denom,long ipp, long idpm, long ido, matrix &sig, vector &eps, vector &epsp, vector &qtr,double gamma)
{
  double ret = 0.0;
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    //ret = spl1d[ip[ipp].idm[idpm]].plasmodscalar (qtr);
    ret = spl1d[ip[ipp].idm[idpm]].plasmodscalar (sig,epsp,qtr,gamma);
    break;
  }
  case jflow:{
    ret = j2f[ip[ipp].idm[idpm]].plasmodscalar (qtr);
    break;
  }
  case jflow2:{
    ret = j2f2[ip[ipp].idm[idpm]].plasmodscalar (qtr);
    break;
  }
  case mohrcoulparab:{
    break;
  }
  case boermaterial:{
    break;
  }
  case druckerprager:{
    ret = drprm[ip[ipp].idm[idpm]].plasmodscalar(qtr);
    break;
  }
  case chenplast:{
    //ret = chplast[ip[ipp].idm[idpm]].plasmodscalar (ipp,sig,epsp,qtr);
    if (ret<1.0e-6)
      denom*=2.0;
    //else
    //if (epsp[0]<1.0e-6)
    //denom*=2.0;
    //else
    //denom+=ret;
    return;
    //break;
  }
  case modcamclaymat:{
    ret = cclay[ip[ipp].idm[idpm]].plasmodscalar(ipp, ido, sig, qtr);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
  denom = ret;
//  return ret;
}



/**
  Function computes (updates) new hardening parameters depending on the actual 
  values of the internal parameters (plastic strains) of the given plasticity model.

   @param ipp - integration point id
   @param idpm - id of plastic material
   @param ido - index of internal variables for given material in the ipp other array
   @param dgamma - increment of consistency parameter
   @param eps  - %vector of actual strains
   @param epsp - %vector of actual plastic strains
   @param sig  - %vector of actial stresses
   @param q    - %vector of resulting hardening parameters (output parameter)
*/
void mechmat::updateq (long ipp, long idpm, long ido,double dgamma, vector &eps, vector &epsp, matrix &sig, vector &q)
{
  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    //spl1d[ip[ipp].idm[idpm]].updateq(dgamma, q);
    spl1d[ip[ipp].idm[idpm]].updateq (ipp,dgamma, epsp,q);
    break;
  }
  case jflow:{
    j2f[ip[ipp].idm[idpm]].updateq (dgamma, q);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[idpm]].updateq (dgamma, q);
    break;
  }
  case mohrcoul:{
    break;
  }
  case mohrcoulparab:{
    break;
  }
  case boermaterial:{
    break;
  }
  case druckerprager:{
    drprm[ip[ipp].idm[idpm]].updateq(ipp, epsp, q);
    break;
  }
  case chenplast:{
    //chplast[ip[ipp].idm[idpm]].updateq (ipp,dgamma,epsp,sig,q);
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[idpm]].updateq(ipp, ido, eps, epsp, sig, q);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
}



/**
   Function computes hardening/softening function for required parameters.
   
   @param ipp - integration point id
   @param idpm - id of plastic material
   @param sigt - stress components stored in 3x3 %matrix
   @param q - internal parameters
   @param h - values of hardening function stored in %vector
   
   JK, 3.2.2007
*/
void mechmat::hardsoftfunction (long ipp,long idpm,matrix &sigt,vector &q,vector &h)
{
  switch (ip[ipp].tm[idpm]){
  case chenplast:{
    chplast[ip[ipp].idm[idpm]].hvalues (sigt,q,h);
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }  
}



/**
  Function returns irreversible strains for given material im.
  
  @param ipp    - integration point number
  @param im     - index of given material
  @param ido    - index of the first internal variable for given material in the ipp eqother array
  @param epsirr - %vector of irreversible strains

  Returns :
   Function returns vector of irreversible strains via parameter epsirr.
*/
void mechmat::giveirrstrains(long ipp, long im, long ido, vector &epsirr)
{
  fillv(0.0, epsirr);
  switch (ip[ipp].tm[im])
  {
    case simplas1d:{
      spl1d[ip[ipp].idm[im]].giveirrstrains(ipp, ido, epsirr);
      break;
    }
    case jflow:{
      j2f[ip[ipp].idm[im]].giveirrstrains(ipp, ido, epsirr);
      break;
    }
    case jflow2:{
      j2f2[ip[ipp].idm[im]].giveirrstrains(ipp, ido, epsirr);
      break;
    }
    case mohrcoul:{
      mcoul[ip[ipp].idm[im]].giveirrstrains(ipp, ido, epsirr);
      break;
    }
    case mohrcoulparab:{
      mcpar[ip[ipp].idm[im]].giveirrstrains(ipp, ido, epsirr);
      break;
    }
    case boermaterial:{
      boerm[ip[ipp].idm[im]].giveirrstrains(ipp, ido, epsirr);
      break;
    }
    case druckerprager:{
      drprm[ip[ipp].idm[im]].giveirrstrains(ipp, ido, epsirr);
      break;
    }
    case chenplast:{
      //chplast[ip[ipp].idm[im]].giveirrstrains(ipp, ido, epsirr);
      break;
    }
    case modcamclaymat:{
      cclay[ip[ipp].idm[im]].giveirrstrains(ipp, ido, epsirr);
      break;
    }
    case creepbaz:{
      crbaz[ip[ipp].idm[im]].giveirrstrains(ipp, ido, epsirr);
      break;
    }
    case creepeffym:{
      creffym[ip[ipp].idm[im]].giveirrstrains(ipp, im, ido, epsirr);
      break;
    }
    case creepb3:
    case creeprs:
    case creepdpl:{
      creep_giveirrstrains(ipp, im, ido, epsirr);
      break;
    }
    default:
      print_err("unknown material model is required",__FILE__,__LINE__,__func__);
  }
}



/**
   Function returns damage parameter for given material im.
   
   @param ipp    - integration point number
   @param im     - index of given material
   @param ido    - index of the first internal variable for given material in the ipp eqother array
   
   Returns :
   Function returns damage parameter.
*/
double mechmat::givedamage(long ipp, long im, long ido)
{
  double dam = 0.0;
  switch (ip[ipp].tm[im])
  {
    case scaldamage_time:{
      dam = scdamtime[ip[ipp].idm[im]].givedamage(ipp, ido);
      break;
    }
    case scaldamage:{
      dam = scdam[ip[ipp].idm[im]].givedamage(ipp, ido);
      break;
    }
    case nonlocdamgmat:
      dam = givedamage(ipp, im+1, ido);
      break;
    default:{
      print_err("unknown material model is required",__FILE__,__LINE__,__func__);
    }
  }
  return dam;
}



// *****************************************************************
// *****************************************************************
// *****************************************************************
// *****************************************************************



/**
   function stores all strains.
   
   @param lcid - load case id
   @param ipp - integration point pointer
   @param eps - %vector containing strain components
   
   JK, 17.7.2001
*/
void mechmat::storestrain (long lcid,long ipp,vector &eps)
{
  long i,j,m,n;
  m=lcid*ip[ipp].ncompstr;
  n=eps.n;
  j=0;
  for (i=m;i<m+n;i++){
    ip[ipp].strain[i]=eps[j];
    j++;
  }
}



/**
   Function stores part of strains.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param fi - first index
   @param eps - %vector containing strain components
   
   JK, 17.7.2001
*/
void mechmat::storestrain (long lcid,long ipp,long fi,vector &eps)
{
  long i,j,m,n;
  m=lcid*ip[ipp].ncompstr;
  n=eps.n;
  j=0;
  for (i=m+fi;i<m+fi+n;i++){
    ip[ipp].strain[i]=eps[j];
    j++;
  }
}



/**
   Function stores part of strains.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param fi - first index
   @param ncomp - number of components
   @param eps - %vector containing strain components
   
   ncomp is explicitly defined in order to save part of the %vector eps
   
   JK, 17.7.2001
*/
void mechmat::storestrain (long lcid,long ipp,long fi,long ncomp,vector &eps)
{
  long i,j,m;
  m=lcid*ip[ipp].ncompstr;
  j=fi;
  for (i=m+fi;i<m+fi+ncomp;i++){
    ip[ipp].strain[i]=eps[j];
    j++;
  }
}



/**
   Function restores all strains to %vector eps.
   
   @param lcid - load case id
   @param ipp - integration point pointer
   @param eps - %vector containing strain components
   
   JK, 5.8.2001
*/
void mechmat::givestrain (long lcid,long ipp,vector &eps)
{
  long i,j,m,n;
  m=lcid*ip[ipp].ncompstr;
  n=eps.n;
  j=0;
  for (i=m;i<n+m;i++){
    eps[j]=ip[ipp].strain[i];
    j++;
  }
}



/**
   Function restores part of strains to %vector eps.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param fi - first index
   @param eps - %vector containing strain components
   
   JK, 5.8.2001
*/
void mechmat::givestrain (long lcid,long ipp,long fi,vector &eps)
{
  long i,j,m,n;
  m=lcid*ip[ipp].ncompstr;
  n=eps.n;
  j=0;
  for (i=m+fi;i<m+fi+n;i++){
    eps[j]=ip[ipp].strain[i];
    j++;
  }
}



/**
   Function restores part of strains to %vector eps.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param fi - first index
   @param ncomp - number of components
   @param eps - %vector containing strain components
   
   ncomp is explicitly defined in order to restore part of the %vector eps
   
   JK, 5.8.2001
*/
void mechmat::givestrain (long lcid,long ipp,long fi,long ncomp,vector &eps)
{
  long i,j,m;
  m=lcid*ip[ipp].ncompstr;
  j=fi;
  for (i=m+fi;i<m+fi+ncomp;i++){
    eps[j]=ip[ipp].strain[i];
    j++;
  }
}



/**
   Function cleans array of strains.
   It works for nonlinear problems where only one load case can be used.
   
   JK
*/
void mechmat::cleanstrain ()
{
  long i;
  
  for (i=0;i<tnip;i++){
    //  it works for nonlinear problems where only one load case can be used
    ip[i].clean_strains (1);
  }
}


// *****************************************************************
// *****************************************************************



/**
   Function stores all stresses.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param sig - %vector containing stress components
   
   JK, 17.7.2001
*/
void mechmat::storestress (long lcid,long ipp,vector &sig)
{
  long i,j,m,n;
  m=lcid*ip[ipp].ncompstr;
  n=sig.n;
  j=0;
  for (i=m;i<m+n;i++){
    ip[ipp].stress[i]=sig[j];
    j++;
  }
}



/**
   Function stores part of stresses.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param fi - first index
   @param sig - %vector containing stress components

   JK, 17.7.2001
*/
void mechmat::storestress (long lcid,long ipp,long fi,vector &sig)
{
  long i,j,m,n;
  m=lcid*ip[ipp].ncompstr;
  n=sig.n;
  j=0;
  for (i=m+fi;i<m+fi+n;i++){
    ip[ipp].stress[i]=sig[j];
    j++;
  }
}



/**
   Function stores part of stresses.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param fi - first index
   @param ncomp - number of components
   @param sig - %vector containing stress components

   ncomp is explicitly defined in order to save part of the %vector sig

   JK, 17.7.2001
*/
void mechmat::storestress (long lcid,long ipp,long fi,long ncomp,vector &sig)
{
  long i,j,m;
  m=lcid*ip[ipp].ncompstr;
  j=fi;
  for (i=m+fi;i<m+fi+ncomp;i++){
    ip[ipp].stress[i]=sig[j];
    j++;
  }
}



/**
   Function restores all stresses to %vector sig.
   
   @param lcid - load case id
   @param ipp - integration point pointer
   @param sig - %vector containing stress components
   
   JK, 5.8.2001
*/
void mechmat::givestress (long lcid,long ipp,vector &sig)
{
  long i,j,m,n;
  m=lcid*ip[ipp].ncompstr;
  n=sig.n;
  j=0;
  for (i=m;i<m+n;i++){
    sig[j]=ip[ipp].stress[i];
    j++;
  }
}



/**
   Function restores part of stresses to %vector sig.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param fi - first index
   @param sig - %vector containing stress components
   
   JK, 5.8.2001
*/
void mechmat::givestress (long lcid,long ipp,long fi,vector &sig)
{
  long i,j,m,n;
  m=lcid*ip[ipp].ncompstr;
  n=sig.n;
  j=0;
  for (i=m+fi;i<m+fi+n;i++){
    sig[j]=ip[ipp].stress[i];
    j++;
  }
}



/**
   Function restores part of stresses to %vector sig.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param fi - first index
   @param ncomp - number of components
   @param sig - %vector containing stress components
   
   ncomp is explicitly defined in order to restore part of the %vector sig
   
   JK, 5.8.2001
*/
void mechmat::givestress (long lcid,long ipp,long fi,long ncomp,vector &sig)
{
  long i,j,m;
  m=lcid*ip[ipp].ncompstr;
  j=fi;
  for (i=m+fi;i<m+fi+ncomp;i++){
    sig[j]=ip[ipp].stress[i];
    j++;
  }
}



// *****************************************************************
// *****************************************************************



/**
   Function restores part of stress increments to %vector sig.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param im  - index of the material in the tm and idm arrays on integration point
   @param ido - index of internal variables in the ip's ipp eqother array
   @param sig - %vector containing stress components
   
   JK, TKo, 29.4.2008
*/
void mechmat::givestressincr (long lcid,long ipp,long im,long ido,vector &sig)
{
  long i;
  
  switch (ip[ipp].tm[im]){
  case creep_damage:{
    givestressincr (lcid, ipp, im+1, ido, sig);
    break;
  }
  /*
  case time_switchmat:{
    tswmat[ip[ipp].idm[im]].givestressincr (ipp,ido,sig);
    break;
  }
  */
  case simvisplas:{
    i=ip[ipp].idm[im];
    svipl[i].givestressincr (ipp,ido,sig);
    break;
  }
  case creepb3:
  case creeprs:
  case creepdpl:{
    creep_givestressincr (ipp,im,ido,sig);
    break;
  }
    /*
  case consolidation:{
    csol[ip[ipp].idm[im]].give_dstresses_eqother (ipp,ido,sig);
    break;
  }
  */

  /*
  case lemaitr:{
    lmtr[ip[ipp].idm[im]].givestressincr (ipp,ido,sig);
    break;
  }
  case creepb3:{
    crb3[ip[ipp].idm[im]].givestressincr (ipp,ido,sig);
    break;
  }
  case creeprs:{
    crrs[ip[ipp].idm[im]].givestressincr (ipp,ido,sig);
    break;
  }
  case creepdpl:{
    crdpl[ip[ipp].idm[im]].givestressincr (ipp,ido,sig);
    break;
  }
  case creepbaz:{
    crbaz[ip[ipp].idm[im]].givestressincr (ipp,ido,sig);
    break;
  }
  case viscoplasticity:{
    visplas[ip[ipp].idm[im]].givestressincr (ipp,ido,sig);
    break;
  }
  */
  case elisomat:
  case simplas1d:
  case jflow:
  case jflow2:
  case microplaneM4:
  case microsimp:
  case microfibro:
  case mohrcoul:
  case mohrcoulparab:
  case boermaterial:
  case druckerprager:
  case chenplast:
  case modcamclaymat:
  case shefpl:
  case glasgowmechmat:
  case glasgowdamage:
  case scaldamage:
  case scaldamagecc:
  case scaldamage_time:   // it can be deleted probably
  case ortodamage:
  case ortodamagerot:
  case anisodamage:
  case anisodamagerot:
  case graphm:
  case graphmtime:        // it can be deleted probably
  case geoelast:
  case creepeffym: 
  case winklerpasternak:
  case nonlocdamgmat:
  case nonlocplastmat:
  case nonlocalmicroM4:
  case damage_plasticity:
  case elasttimemat:      // it can be deleted probably
  case therisodilattime:  // it can be deleted probably
  case contmat:
    if (im == 0)
      fillv(0.0, sig);
    break;    
  default:{
    print_err("unknown material model is required",__FILE__,__LINE__,__func__);
  }
  }
}



/**
   Function stores part of stress increments.
   
   @param lcid - load case id
   @param ipp - integration point number
   @param im  - index of the material in the tm and idm arrays on integration point
   @param ido - index of internal variables in the ip's ipp eqother array
   @param sig - %vector containing stress components
   
   JK, TKo, 29.4.2008
*/
void mechmat::storestressincr (long lcid,long ipp,long im,long ido,vector &sig)
{
  long i;
  
  switch (ip[ipp].tm[im]){
  case simvisplas:{
    i=ip[ipp].idm[im];
    svipl[i].storestressincr (ipp,ido,sig);
    break;
  }
  default:{
    print_err("unknown material model is required",__FILE__,__LINE__,__func__);
  }
  }
}



// *****************************************************************
// *****************************************************************



/**
   Function stores eigenstrains at integration point.
   
   @param ipp - number of integration point
   @param eps - %vector of eigenstrains
   
   3.3.2004, JK
*/
void mechmat::storeeigstrain (long ipp,vector &eps)
{
  long i,nc;
  
  nc=ip[ipp].ncompstr;
  if (est==eigstrain){
    for (i=0;i<nc;i++){
      eigstrains[ipp][i]=eps[i];
    }
  }
  if (est==tempstrain){
    for (i=0;i<nc;i++){
      tempstrains[ipp][i]=eps[i];
    }
  }
}



/**
   Function stores eigenstrains.
   
   @param ipp - integration point pointer
   @param fi - first index
   @param ncomp - number of required components
   @param eps - %vector containing eigenstrain components
   
   JK, 17.7.2001
*/
void mechmat::storeeigstrain (long ipp,long fi,long ncomp,vector &eps)
{
  long i,j;

  if (est==eigstrain){
    j=0;
    for (i=fi;i<fi+ncomp;i++){
      eigstrains[ipp][i] = eps[j];
      j++;
    }
  }
  if (est==tempstrain){
    j=0;
    for (i=fi;i<fi+ncomp;i++){
      tempstrains[ipp][i] = eps[j];
      j++;
    }
  }
}



/**
   Function returns eigenstrains.
   
   @param ipp - integration point pointer
   @param eps - %vector containing strain components
   
   17.7.2001, JK
*/
void mechmat::giveeigstrain (long ipp,vector &eps)
{
 long i,n;
  
  n=eps.n;  
  if (est==eigstrain){
    for (i=0;i<n;i++){
      eps[i] = eigstrains[ipp][i];
    }
  }
  if (est==tempstrain){
    for (i=0;i<n;i++){
      eps[i] = tempstrains[ipp][i];
    }
  }
}



/**
   Function restores eigenstrains.
   
   @param ipp - integration point pointer
   @param fi - first index
   @param ncomp - number of selected components
   @param eps - %vector containing strain components
   
   17.7.2001, JK
*/
void mechmat::giveeigstrain (long ipp,long fi,long ncomp,vector &eps)
{
  long i,j;
  
  if (est==eigstrain){
    j=0;
    for (i=fi;i<fi+ncomp;i++){
      eps[j] = eigstrains[ipp][i];
      j++;
    }
  }
  if (est==tempstrain){
    j=0;
    for (i=fi;i<fi+ncomp;i++){
      eps[j] = tempstrains[ipp][i];
      j++;
    }
  }
}



// *****************************************************************
// *****************************************************************



/**
   Function stores eigenstresses at integration point.
   
   @param ipp - number of integration point
   @param sig - %vector of eigenstresses
   
   JK, 27.11.2006
*/
void mechmat::storeeigstress (long ipp,vector &sig)
{
  long i,nc;
  
  nc=sig.n;
  for (i=0;i<nc;i++){
    eigstresses[ipp][i]=sig[i];
  }
  
}



/**
   Function stores eigenstresses.
   
   @param ipp - integration point pointer
   @param fi - first index
   @param ncomp - number of required components
   @param sig - %vector containing eigenstress components
   
   27.11.2006, JK
*/
void mechmat::storeeigstress (long ipp,long fi,long ncomp,vector &sig)
{
  long i,j;

  j=0;
  for (i=fi;i<fi+ncomp;i++){
    eigstresses[ipp][i] = sig[j];
    j++;
  }
}



/**
   Function returns eigenstresses.
   
   @param ipp - integration point pointer
   @param sig - %vector containing stress components
   
   27.11.2006, JK
*/
void mechmat::giveeigstress (long ipp,vector &sig)
{
  long i,n;
  
  n=sig.n;  
  for (i=0;i<n;i++){
    sig[i] = eigstresses[ipp][i];
  }
}



/**
   Function restores eigenstress.
   
   @param ipp - integration point pointer
   @param fi - first index
   @param sig - %vector containing stress components
   
   29.4.2008, JK
*/
void mechmat::giveeigstress (long ipp,long fi,vector &sig)
{
  long i,j,ncomp;
  ncomp=sig.n;
  j=0;
  for (i=fi;i<fi+ncomp;i++){
    sig[j] = eigstresses[ipp][i];
    j++;
  }
}



/**
   Function restores eigenstress.
   
   @param ipp - integration point pointer
   @param fi - first index
   @param ncomp - number of selected components
   @param sig - %vector containing eigenstress components
   
   29.4.2008, JK
*/
void mechmat::giveeigstress (long ipp,long fi,long ncomp,vector &sig)
{
  long i,j;
  j=fi;
  for (i=fi;i<fi+ncomp;i++){
    sig[j] = eigstresses[ipp][i];
    j++;
  }
}

// *****************************************************************
// *****************************************************************



/**
   Function stores components of array eqother.
   
   @param ipp - integration point pointer
   @param fi - first index
   @param ncomp - number of required components
   @param comp - %vector containing components
   
   5.11.2002, JK
*/
void mechmat::storeeqother (long ipp,long fi,long ncomp,double *comp)
{
  long i,j;
  j=0;
  for (i=fi;i<fi+ncomp;i++){
    ip[ipp].eqother[i]=comp[j];
    j++;
  }
}



/**
   Function restores array eqother to %vector comp.
   
   @param ipp - integration point pointer
   @param fi - first index
   @param ncomp - number of required components
   @param sig - %vector containing components
   
   5.11.2002, JK
*/
void mechmat::giveeqother (long ipp,long fi,long ncomp,double *comp)
{
  long i,j;
  j=0;
  for (i=fi;i<fi+ncomp;i++){
    comp[j]=ip[ipp].eqother[i];
    j++;
  }
}



// *****************************************************************
// *****************************************************************
// *****************************************************************
// *****************************************************************



/**
   Function assembles required quantity stored at integration point.
   
   @param iq - id of required quantity
   @param lcid - load case id
   @param ipp - number of integration point
   @param fi - first index
   @param ipv - array of required values from integration point
   
   29.4.2008
*/
void mechmat::givequantity (integratedquant iq,long lcid,long ipp,long fi,vector &ipv)
{
  switch (iq){
  case locstress:{
    //  stress reading from integration point
    givestress (lcid,ipp,fi,ipv);
    break;
  }
  case nonlocstress:{
    //  stress reading from integration point
    givestress (lcid,ipp,fi,ipv);
    break;
  }
  case stressincr:{
    //  stress increments from integration point
    givestressincr (lcid,ipp,0,0,ipv);
    break;
  }
  case eigstress:{
    //  eigenstress reading from integration point
    giveeigstress (ipp,fi,ipv);
    break;
  }
  default:{
    print_err("unknown type of quantity is required",__FILE__,__LINE__,__func__);
  }
  }
  
}




// *****************************************************************
// *****************************************************************
// *****************************************************************
// *****************************************************************
// *****************************************************************
// *****************************************************************
// *****************************************************************
// *****************************************************************



/**
  Function returns number of internal parameters for material models of plasticity.

  @param ipp - integration point number in the mechmat ip array.
  @param idpm - index of material which is plasticty.

  @retval Function returns number of material parameters. 
*/
long mechmat::give_num_interparam (long ipp,long idpm)
{
  long np;

  switch (ip[ipp].tm[idpm]){
  case simplas1d:{
    np = spl1d[ip[ipp].idm[idpm]].give_num_interparam ();
    break;
  }
  case jflow:{
    np = j2f[ip[ipp].idm[idpm]].give_num_interparam ();
    break;
  }
  case mohrcoul:{
    break;
  }
  case mohrcoulparab:{
    break;
  }
  case boermaterial:{
    break;
  }
  case druckerprager:{
    break;
  }
  case modcamclaymat:{
    break;
  }
  default:
    print_err("unknown plasticity model is required",__FILE__,__LINE__,__func__);

  }
  
  return np;
}



/**
   Function gives internal parameters of the required material model.
   
   @param ipp - number of integration point
   @param im - index of the material
   @param ido - index in the eqother array
   @param q - array containing internal parameters
   
   
*/
void mechmat::give_interparam (long ipp,long im,long ido,vector &q)
{
  switch (ip[ipp].tm[im]){
  case simplas1d:{
    spl1d[ip[ipp].idm[im]].give_interparam (ipp,ido,q);
    break;
  }
  case jflow:{
    j2f[ip[ipp].idm[im]].give_interparam (ipp,ido,q);
    break;
  }
  case mohrcoul:{
    break;
  }
  case mohrcoulparab:{
    break;
  }
  case boermaterial:{
    break;
  }
  case druckerprager:{
    break;
  }
  case modcamclaymat:{
    break;
  }

  case simvisplas:{
    svipl[ip[ipp].idm[im]].give_interparam (ipp,ido,q);
    break;
  }
  case lemaitr:{
    break;
  }
  default:
    print_err("unknown material type is required",__FILE__,__LINE__,__func__);

  }
  
}



/**
  Function stores internal parameters of the vsicous materials.

  @param ipp - integration point number in the mechmat ip array.
  @param im  - material index
  @param ido - index of internal variables for given material in the ipp other array
  @param q   - %vector containing internal parameters

*/
void mechmat::store_interparam (long ipp,long im,long ido,vector &q)
{
  switch (ip[ipp].tm[im]){
    /*
  case vis:{
    svipl[ip[ipp].idm[im]].store_interparam (ipp,ido,q);
    break;
  }
*/
  case simvisplas:{
    svipl[ip[ipp].idm[im]].store_interparam (ipp,ido,q);
    break;
  }
  case lemaitr:{
    break;
  }
  default:
    print_err("unknown material type is required",__FILE__,__LINE__,__func__);
  }
  
}



/**
   This function returns the number of components of ipp's other array.
   Material type is given by the im parameter, which means index of material
   in the integration point ipp tm(idm) arrays.
   
   @param ipp - integration point pointer
   @param im - index of material
   
   TK, 5.11.2003
*/
long mechmat::givencompother (long ipp,long im)
{
  long ncompo = 0;
  
  switch (ip[ipp].tm[im])
    {
    case elisomat:
    case elgmat3d:
    case winklerpasternak:
    case graphm:
    case rcmatmodnorm:
    case geoelast:
    case simplas1d:
    case jflow:
    case jflow2:
    case microplaneM4:
    case microsimp:
    case microfibro:
    case mohrcoul:
    case mohrcoulparab:
    case boermaterial:
    case druckerprager:
    case chenplast:
    case modcamclaymat:
    case shefpl:
    case scaldamage:
    case scaldamagecc:
    case ortodamage:
    case ortodamagerot:
    case anisodamage:
    case anisodamagerot:
    case creepbaz:
    case creepeffym:
    case consolidation:
    case nonlocplastmat:
    case nonlocdamgmat:
    case nonlocalmicroM4:
    case glasgowdamage:
    case damage_plasticity:
    case therisodilat:
    case simvisc:
    case lenjonespot:{
      ncompo = givencompeqother (ipp,im);
      break;
    }
    case creeprs:
    case creepb3:
    case creepdpl:{
      ncompo=creep_ncompo(ipp,im);//the same as in eqother
      break;
    }
    case creep_damage:
      ncompo += givencompother (ipp, im+1); // creep
      ncompo += givencompother (ipp, im+2); // damage
      break;
    case simvisplas:
    case lemaitr:
    case glasgowmechmat:
    case viscoplasticity:
    case scaldamage_time:
    case graphmtime:
    case therisodilattime:
    case elasttimemat:{
      ncompo=0;
      break;
    case time_switchmat:
      ncompo = tswmat[ip[ipp].idm[im]].givencompother (ipp,im);
      break;
    }
    default:
      print_err("unknown material type is required", __FILE__, __LINE__, __func__);
    }
  
  return ncompo;
}



/**
   This function returns the number of components of ipp's eqother array.
   Material type is given by the im parameter, which means index of material
   in the integration point ipp tm(idm) arrays.
   
   @param ipp - integration point pointer
   @param im - index of material
   
   TK, 5.11.2003
*/
long mechmat::givencompeqother (long ipp,long im)
{
  long ncompo = 0;
  long ncompstr = ip[ipp].ncompstr;
  long n;
  
  switch (ip[ipp].tm[im])
    {
    case elisomat:
      break;
    case elgmat3d:
      break;
      //case geoelmat:
      //ncompo=ncompstr+1;
      //break;
    case winklerpasternak:
      break;
    case rcmatmodnorm:
      ncompo=6;
      break;
    case graphm:
      break;
    case graphmtime:
      ncompo = 2*ncompstr;
      break;
    case geoelast:
      ncompo  = 1;
      ncompo += givencompeqother(ipp, im+1); 
      break;
    case simplas1d:
      ncompo=3;
      break;
    case jflow:
      ncompo=ncompstr+2;
      break;
    case jflow2:
      ncompo=ncompstr+2;
      break;
    case microplaneM4:
      n=mpM4[ip[ipp].idm[im]].numberOfMicroplanes;
      ncompo=3*n+1+ncompstr;
      break;
    case microsimp:
      n=mpSIM[ip[ipp].idm[im]].numberOfMicroplanes;
      ncompo=n+1+6;
      break;
    case microfibro:
      n=mpfib[ip[ipp].idm[im]].numberOfMicroplanes;
      ncompo=5*n+1+6;
      break;
    case mohrcoul:
      ncompo=ncompstr+1;
      break;
    case mohrcoulparab:
      ncompo=ncompstr+1;
      break;
    case boermaterial:
      ncompo=ncompstr+1;
      break;
    case druckerprager:
      ncompo=ncompstr+4;
      break;
    case chenplast:
      ncompo=ncompstr+3;
      break;
    case modcamclaymat:
      ncompo=ncompstr+2+3+5;
      break;
    case shefpl:
      ncompo=ncompstr+2;
      break;
    case glasgowmechmat:
      ncompo=ncompstr*4+5;
      break;
    case glasgowdamage:
      ncompo=2;
      break;
    case creep_damage:
      ncompo += givencompeqother (ipp, im+1); // creep
      ncompo += givencompeqother (ipp, im+2); // damage
      break;
    case time_switchmat:
      ncompo = tswmat[ip[ipp].idm[im]].givencompeqother (ipp,im);
      break;
    case simvisc:
      ncompo=3*ncompstr;
      break;
    case simvisplas:
      ncompo=3*ncompstr+1;
      break;
    case lemaitr:
      ncompo=3*ncompstr+2;
      break;
    case scaldamage:
      ncompo=3;
      break;
    case scaldamagecc:
      ncompo=2;
      break;
    case scaldamage_time:
//      ncompo=4*ncompstr+3;
      ncompo=2;
      break;
    case ortodamage:
      ncompo=19;
      break;
    case ortodamagerot:
      ncompo=23;
      break;
    case anisodamage:
//      ncompo=10;
      ncompo=26;
      break;
    case anisodamagerot:
//      ncompo=10;
      ncompo=22;
      break;
    case creeprs:
    case creepb3:
    case creepdpl:{
	ncompo=creep_ncompo(ipp,im);
	break;
    }
    case creepbaz:
      //ncompo=ncompstr*9+3+ncompstr;
      n=crbaz[ip[ipp].idm[im]].numberOfCreepb();
      ncompo=ncompstr*(n+3)+3+1;
      break;
    case creepeffym:
      ncompo=ncompstr;
      break;
    case consolidation:
      n=csol[ip[ipp].idm[im]].numberOfConsol();
      if(ncompstr==6) ncompo=ncompstr+2*n+1;
      if(ncompstr==5) ncompo=ncompstr+3*n+1;
      if(ncompstr==3) ncompo=ncompstr+3*n+1;
      //if(ncompstr==6) ncompo=ncompstr+2*7+1;
      //if(ncompstr==3) ncompo=ncompstr+3*7+1;
      break;
      
    case nonlocplastmat:
      ncompo = givencompeqother (ipp, im+1);
      break;
    case nonlocdamgmat:
      ncompo = givencompeqother (ipp, im+1);
      break;
    case nonlocalmicroM4:
      n=mpM4[ip[ipp].idm[im]].numberOfMicroplanes;
      ncompo=3*n+1+ncompstr+ncompstr;
      break;
    case damage_plasticity:
      ncompo = givencompeqother (ipp, im+1);
      ncompo += givencompeqother (ipp, im+2);
      break;
    case viscoplasticity:
      ncompo = givencompeqother (ipp, im+1);
      ncompo += givencompeqother (ipp, im+2);
      break;
    case elasttimemat:
      ncompo = 1;
      break;
      
    case therisodilat:{
      break;
    }
    case therisodilattime:{
      ncompo = 2;
      break;
    }
    case lenjonespot:{
      ncompo = 0;
      break;
    }
    default:{
      print_err("unknown material type is required", __FILE__, __LINE__, __func__);
    }
    }
  
  if (((ip[ipp].hmt & 1) && (im < ip[ipp].nm-1)) && (im == 0))
    // thermal dilatancy is present and ncompeqother for different 
    // material type (than thermal dilatancy material) is required
    // and index of this material is 0 (i.e it is main controle material)
    {
      switch (ip[ipp].tm[ip[ipp].nm-1])
	{
	case therisodilat:
	  break;
	case therisodilattime:
	  ncompo += 2;
	  break;
	default:{
	  print_err("unknown thermal material type is required", __FILE__, __LINE__, __func__);
	}
	}
    }
  return ncompo;
}



// *******************
// *******************
// ****  STRAINS  ****
// *******************
// *******************



void mechmat::compute_averstrains ()
{
  long i,ne;
  double volume;
  elemtype te;
  vector averstra (6);
  
  volume=0.0;
  fillv (0.0,averstra);
  
  ne=Mt->ne;
  for (i=0;i<ne;i++){
    if (Gtm->leso[i]==1){
      
      te = Mt->give_elem_type (i);
      
      switch (te){
      case linearhex:{
	Lhex->aver_strains (0,i,0,0,averstra,volume);
	break;
      }
      case quadrhex:{
	Qhex->aver_strains (0,i,0,0,averstra,volume);
	break;
      }
	
      default:{
	
      }
      }
    }
  }
  
  for (i=0;i<6;i++){
    averstra[i]/=volume;
  }
  
  printf ("\n\n prumerne deformace");
  for (i=0;i<6;i++){
    printf ("\n slozka %ld   %f",i+1,averstra[i]);
  }
  
}



/**
   Function computes strains caused by temperature.
   
   @param lcid - load case id
   
*/
void mechmat::temprstrains (long lcid)
{
  long i,nm,ii, ido;
  
  for (i=0;i<tnip;i++){
    
    ii = ip[i].hmt & 1; // presence of thermal dilatation material model
    if (ii > 0){
      nm=ip[i].nm-1;
      switch (ip[i].tm[nm]){
      case therisodilat:{
	tidilat[ip[i].idm[nm]].temprstrains (lcid,i);
	break;
      }
      case therisodilattime:{
        ido = givencompeqother(i, 0);
        ido -= givencompeqother(i, nm);
	tidilattime[ip[i].idm[nm]].temprstrains (i, ido);
	break;
      }
      default:
	print_err("unknown thermal dilatation material is required",__FILE__,__LINE__,__func__);
      }
    }

  }
}



// ********************
// ********************
// ****  STRESSES  ****
// ********************
// ********************



/**
   Function computes correct stresses for computed strains.
   
   @param ipp - integration point pointer
   @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
   @param ido - index of the first internal variable in the ipp other arrays for given material model
                The standard value is zero.
   
   4.8.2001
*/
void mechmat::computenlstresses (long ipp,long im,long ido)
{
  switch (ip[ipp].tm[im]){
  case elisomat:{
    eliso[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case simplas1d:{
    spl1d[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case jflow:{
    j2f[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case microplaneM4:{
    mpM4[ip[ipp].idm[im]].nlstresses (ipp,ido);
    break;
  }
  case microsimp:{
    mpSIM[ip[ipp].idm[im]].nlstresses (ipp,ido);
    break;
  }
  case microfibro:{
    mpfib[ip[ipp].idm[im]].nlstresses (ipp,ido);
    break;
  }
  case mohrcoul:{
    mcoul[ip[ipp].idm[im]].nlstresses (ipp,ido);
    break;
  }
  case mohrcoulparab:{
    mcpar[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case boermaterial:{
    boerm[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case druckerprager:{
    drprm[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case shefpl:{
    shpl[ip[ipp].idm[im]].nlstresses (ipp,ido);
    break;
  }
  case glasgowmechmat:{
    glasgmat[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case glasgowdamage:{
    glasgdam[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case creep_damage:{
    crdam[ip[ipp].idm[im]].nlstresses (ipp, im, ido);
    break;
  }
  case time_switchmat:{
    tswmat[ip[ipp].idm[im]].nlstresses (ipp, im, ido);
    break;
  }
  case simvisplas:{
    svipl[ip[ipp].idm[im]].nlstresses (ipp,ido);
    break;
  }
  case lemaitr:{
    lmtr[ip[ipp].idm[im]].nlstresses (ipp,ido);
    break;
  }
  case scaldamage:{
    scdam[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case scaldamagecc:{
    scdamcc[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case scaldamage_time:{
    scdamtime[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case ortodamage:{
    ortdam[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case ortodamagerot:{
    ortdamrot[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case anisodamage:{
    anidam[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case anisodamagerot:{
    anidamrot[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case graphm:{
    grmat[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case graphmtime:{
    grmattime[ip[ipp].idm[im]].nlstresses (ipp, im, ido);
    break;
  }
  case geoelast:{
    geoel[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case creepb3:
  case creeprs:
  case creepdpl:{
    creep_nlstresses(ipp,im,ido);
    break;
  }
  case creepbaz:{
    crbaz[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case creepeffym:{
    creffym[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case consolidation:{
    csol[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case winklerpasternak:{
    wpast[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }

  case nonlocdamgmat:{
    computenlstresses (ipp,im+1,ido);
    break;
  }
  case nonlocplastmat:{
    computenlstresses (ipp,im+1,ido);
    break;
  }
  case nonlocalmicroM4:{
    computenlstresses (ipp,im+1,ido);
    break;
  }
  case damage_plasticity:{
    dampl[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case viscoplasticity:{
    visplas[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case elasttimemat:{
    eltimemat[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case therisodilattime:{
    tidilattime[ip[ipp].idm[im]].nlstresses (ipp,ido);
    break;
  }

  case contmat:{
    conmat[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }

  default:{
    print_err("unknown material type is required", __FILE__,__LINE__,__func__);
  }
  }

}



/**
   Function computes correct increments of stresses for computed strains.
   
   @param ipp - integration point pointer
   @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
   @param ido - index of the first internal variable in the ipp other arrays for given material model
                The standard value is zero.
   
   29.4.2008
*/
void mechmat::computenlstressesincr (long ipp,long im,long ido)
{
  switch (ip[ipp].tm[im]){
   
  case creep_damage:{
    crdam[ip[ipp].idm[im]].nlstressesincr (ipp, im, ido);
    break;
  }
  /*
  case time_switchmat:{
    tswmat[ip[ipp].idm[im]].nlstresses (ipp, im, ido);
    break;
  }
  */
  case simvisplas:{
    svipl[ip[ipp].idm[im]].nlstressesincr (ipp,ido);
    break;
  }
  case creepb3:
  case creeprs:
  case creepdpl:{
    creep_nlstressesincr (ipp,im,ido);
    break;
  }
    
    /*
      case lemaitr:{
      lmtr[ip[ipp].idm[im]].nlstresses (ipp,ido);
      break;
      }
      case creepbaz:{
      crbaz[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
      break;
      }
    */
  case consolidation:{
    csol[ip[ipp].idm[im]].nlstressesincr (ipp);
    break;
  }

  /*
  case viscoplasticity:{
    visplas[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  */
  case elisomat:
  case simplas1d:
  case jflow:
  case jflow2:
  case microplaneM4:
  case microsimp:
  case microfibro:
  case mohrcoul:
  case mohrcoulparab:
  case boermaterial:
  case druckerprager:
  case chenplast:
  case modcamclaymat:
  case shefpl:
  case glasgowmechmat:
  case glasgowdamage:
  case scaldamage:
  case scaldamagecc:
  case scaldamage_time:   // it can be deleted probably
  case ortodamage:
  case ortodamagerot:
  case anisodamage:
  case anisodamagerot:
  case graphm:
  case graphmtime:        // it can be deleted probably
  case geoelast:
  case creepeffym:
  case winklerpasternak:
  case nonlocdamgmat:
  case nonlocplastmat:
  case nonlocalmicroM4:
  case damage_plasticity:
  case elasttimemat:      // it can be deleted probably
  case therisodilattime:  // it can be deleted probably
  case contmat:
    if (im == 0)
      ip[ipp].clean_stresses(Mb->nlc);
    break;    
  default:{
    print_err("unknown material type is required",__FILE__,__LINE__,__func__);
  }
  }

}



/**
   Function computes correct stresses for computed strains for non-local material models.

   @param ipp - integration point pointer
   @param im  - index of the material in the ipp tm and idm arrays. The standard value is zero.
   @param ido - index of the first internal variable in the ipp other arrays for given material model
                The standard value is zero.

   4.8.2001
*/
void mechmat::compnonloc_nlstresses (long ipp,long im,long ido)
{
  switch (ip[ipp].tm[im]){
  case elisomat:{
    eliso[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case simplas1d:{
    if (ip[ipp].hmt & 2) 
      spl1d[ip[ipp].idm[im]].nonloc_nlstresses (ipp,im,ido);
    else
      spl1d[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case jflow:{
    if (ip[ipp].hmt & 2) 
      j2f[ip[ipp].idm[im]].nonloc_nlstresses (ipp,im,ido);
    else
      j2f[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case microplaneM4:{
    if (ip[ipp].hmt & 2) 
      mpM4[ip[ipp].idm[im]].nonloc_nlstresses (ipp,ido);
    else
      mpM4[ip[ipp].idm[im]].nlstresses (ipp,ido);
    break;
  }
  case microsimp:{
//    mpSIM[ip[ipp].idm[0]].nlstresses (ipp);
    break;
  }
  case microfibro:{
//    mpfib[ip[ipp].idm[0]].nlstresses (ipp);
    break;
  }
  case mohrcoul:{
    if (ip[ipp].hmt & 2) 
      mcoul[ip[ipp].idm[im]].nonloc_nlstresses (ipp,ido);
    else
      mcoul[ip[ipp].idm[im]].nlstresses (ipp,ido);
    break;
  }
  case mohrcoulparab:{
    if (ip[ipp].hmt & 2) 
      mcpar[ip[ipp].idm[im]].nonloc_nlstresses (ipp,im,ido);
    else
      mcpar[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case boermaterial:{
    if (ip[ipp].hmt & 2) 
      boerm[ip[ipp].idm[im]].nonloc_nlstresses (ipp,im,ido);
    else
      boerm[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case druckerprager:{
    if (ip[ipp].hmt & 2) 
      drprm[ip[ipp].idm[im]].nonloc_nlstresses (ipp,im,ido);
    else
      drprm[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case modcamclaymat:{
    //    cclay[ip[ipp].idm[im]].nonloc_nlstresses (ipp);
    break;
  }
  case simvisplas:{
//    svipl[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case lemaitr:{
//    lmtr[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case scaldamage:{
    scdam[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case scaldamagecc:{
    scdamcc[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case ortodamage:{
    ortdam[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case ortodamagerot:{
    ortdamrot[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case anisodamage:{
    anidam[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case anisodamagerot:{
    anidamrot[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case graphm:{
    grmat[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case graphmtime:{
    grmattime[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case creepbaz:{
//    crbaz[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case consolidation:{
    csol[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case winklerpasternak:{
    wpast[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }
  case elasttimemat:{
    eltimemat[ip[ipp].idm[im]].nlstresses (ipp);
    break;
  }

  case creep_damage:{
    crdam[ip[ipp].idm[im]].nlstresses (ipp, im, ido);
    break;
  }
  case creepb3:
  case creeprs:
  case creepdpl:{
    creep_nlstresses(ipp,im,ido);
    break;
  }
  case creepeffym:{
    creffym[ip[ipp].idm[im]].nlstresses (ipp,im,ido);
    break;
  }
  case nonlocdamgmat:{
    compnonloc_nlstresses (ipp,im+1,ido);
    break;
  }
  case nonlocplastmat:{
    compnonloc_nlstresses (ipp,im+1,ido);
    break;
  }
  case nonlocalmicroM4:{
    compnonloc_nlstresses (ipp,im+1,ido);
    break;
  }
    
  default:{
    print_err("unknown material type is required",__FILE__,__LINE__,__func__);
  }
  }

}



/**
   Function averages local values and stores them in nonlocal values.

   @param ipp - integration point pointer
   @param im - material index
   @param ido - index of internal variables for given material in the ipp other array

   4.8.2003
*/
void mechmat::nonlocaverage (long ipp,long im,long ido)
{
  if (ip[ipp].hmt & 2)
  {
    switch (ip[ipp].tm[im]){
    case nonlocplastmat :
      nlplast[ip[ipp].idm[im]].average(ipp, ido);
      break;
    case nonlocdamgmat :
      nldamg[ip[ipp].idm[im]].average(ipp,im,ido);
      break;
    case nonlocalmicroM4 :
      nmpM4[ip[ipp].idm[im]].average(ipp);
      break;
    case creep_damage:
      long tncompo;
      long nldamncompo;
      tncompo = givencompeqother(ipp,im); // number of components for creepdam
      nldamncompo = givencompeqother(ipp,im+2); // number of components for nonlocdam    
      nonlocaverage(ipp, im+2, ido+tncompo-nldamncompo);
      break;
    default:{
      print_err("unknown type of averaging model is required", __FILE__, __LINE__, __func__);
    }
    }
  }
}



/**
   Function averages local values and stores them in nonlocal values.

   @param ipp - integration point pointer
   @param im  - material index
   @param ido - index of internal variables for given material in the ipp other array

   Created by TKo 10.2009
*/
long mechmat::give_num_averq(long ipp,long im,long ido)
{
  long ret = 0;
  if (ip[ipp].hmt & 2)
  {
    switch (ip[ipp].tm[im]){
    case nonlocplastmat :
      ret = nlplast[ip[ipp].idm[im]].give_num_averq(ipp);
      break;
    case nonlocdamgmat :
      ret = nldamg[ip[ipp].idm[im]].give_num_averq(ipp, im);
      break;
    case creep_damage:
      long tncompo;
      long nldamncompo;
      tncompo = givencompeqother(ipp,im); // number of components for creepdam
      nldamncompo = givencompeqother(ipp,im+2); // number of components for nonlocdam    
      ret = give_num_averq(ipp, im+2, ido+tncompo-nldamncompo);
      break;
    default:{
      print_err("unknown type of averaging model is required", __FILE__, __LINE__, __func__);
    }
    }
  }
  return ret;
}



/**
   Function returns vector of averaged quantities

   @param ipp - integration point pointer
   @param im  - material index
   @param ido - index of internal variables for given material in the ipp other array

   Created by TKo 10.2009
*/
void mechmat::give_aver_quantv(long ipp,long im,long ido, vector &qv)
{
  if (ip[ipp].hmt & 2)
  {
    switch (ip[ipp].tm[im]){
    case nonlocplastmat :
      nlplast[ip[ipp].idm[im]].give_aver_quantv(ipp, im, ido, qv);
      break;
    case nonlocdamgmat :
      nldamg[ip[ipp].idm[im]].give_aver_quantv(ipp, im, ido, qv);
      break;
    case creep_damage:
      long tncompo;
      long nldamncompo;
      tncompo = givencompeqother(ipp,im); // number of components for creepdam
      nldamncompo = givencompeqother(ipp,im+2); // number of components for nonlocdam    
      give_aver_quantv(ipp, im+2, ido+tncompo-nldamncompo, qv);
      break;
    default:{
      print_err("unknown type of averaging model is required", __FILE__, __LINE__, __func__);
    }
    }
  }
  return;
}



/**
  Function returns stresses on sufrace of plasticity.
  Cutting plane method is used.

  gamma,epsp and q will be replaced by new values

  @param ipp - integration point pointer
  @param im - material index
  @param ido - index of internal variables for given material in the ipp other array
  @param gamma - consistency parameter
  @param epsn - total strain components
  @param epsp - plastic strain components
  @param q - hardening parameters
  @param ni - maximum number of iterations
  @param err - required error
   
  4.8.2001
*/
void mechmat::cutting_plane (long ipp,long im,long ido,double &gamma,vector &epsn,vector &epsp,vector &q,long ni,double err)
{
  long i,j,ncomp=epsn.n,idem;
  double f,denom,denomh,dgamma,nu;
  vector epsa(ncomp),sig(ncomp),dfds(ncomp),dgds(ncomp);
  matrix d(ncomp,ncomp),sigt(3,3);

  //  initialization
  dgamma=0.0;

  //  elastic stiffness matrix
  elmatstiff (d,ipp);
  if (ip[ipp].ssst == planestrain)
  {
     d[0][3] = d[0][1]; d[1][3] = d[1][0];
     d[3][0] = d[1][0]; d[3][1] = d[1][0]; d[3][3] = d[1][1];
  }
  idem = ip[ipp].gemid();
  if (ip[ipp].ssst == planestress)
  {
    nu = Mm->give_actual_nu(ipp);
    epsn[3] = -nu / (1.0 - nu) * (epsn[0]-epsp[0]+epsn[1]-epsp[1]);
  }
  
  
  //fprintf (Out,"\n\n int. point %ld \n",ipp);

  //  main iteration loop
  for (i=0;i<ni;i++){

    //fprintf (Out,"\n cutting plane, iterace  %ld",i);

    //  elastic strain
    subv (epsn,epsp,epsa);
    //  trial stress computation
    mxv (d,epsa,sig);
    if (ic)
      {
	for (j=0; j < ncomp; j++)
	  sig[j] += ic[ipp][3+j];
      }
    
    vector_tensor (sig,sigt,ip[ipp].ssst,stress);
    // updating hardening variables from previous step
    updateq(ipp, im, ido, dgamma, epsn, epsp, sigt, q);
    // checking yield function
    f = yieldfunction (ipp,im,sigt,q);
    
    //if (f>0.0) printf ("\n plasticity at int. point %ld",ipp);
    //fprintf (Out,"\n step %ld   yield function  %le",i,f);
    
    if (f<err)  break;
    if (i==ni-1 && f>err){
      print_err("yield surface was not reached",__FILE__,__LINE__,__func__);
      fprintf (Out,"\n yield surface was not reached");
    }

    dfdsigma (ipp, im, sigt, q, dfds);
    dgdsigma (ipp, im, sigt, q, dgds);

    mxv (d,dgds,epsa);
    scprd (dfds,epsa,denom);

    //fprintf (Out,"  denom %le",denom);

    //denom += plasmodscalar(ipp, im, ido, sigt, epsn, epsp, q,gamma);
    denomh=0;
    plasmodscalar(denomh,ipp, im, ido, sigt, epsn, epsp, q,gamma);
    denom += denomh;

    //fprintf (Out,"  denom %le",denom);

    //  new increment of consistency parameter
    dgamma = f/denom;
    //  new internal variables
    gamma+=dgamma;
    // updating plastic strains
    for (j=0;j<ncomp;j++){
      epsp[j]+=dgamma*dgds[j];
    }
    if (ip[ipp].ssst == planestress)
      epsp[3] = epsp[0]+epsp[1];
  }
  
  //  elastic strain
  subv (epsn,epsp,epsa);
  //  trial stress computation
  mxv (d,epsa,sig);
  //  storing resulting stresses
  storestress (0,ipp,sig);
}



/**
   Function returns stresses on surface of plasticity in multi-surface
   plasticity with hardening is assumed. Cutting plane method is used.   
   Vectors epsp,gamma and q will be overwritten by new values.
   
   @param ipp - integration point integer
   @param epsn - new total strain components
   @param epsp - plastic strain components
   @param gamma - %vector of consistency parameters
   @param q - %vector of hardening parameters
   
   ncomp - number of components of strain tensors
   nyf - number of yield functions
   nhp - number of hardening parameters
   
   16.8.2001
*/
void mechmat::mult_surf_cutting_plane (long ipp,vector &epsn,vector &epsp,vector &gamma,vector &q)
{
  long ni,i,j,k,nas,ncomp=epsn.n,nhp=q.n,nyf=gamma.n,*stat;
  double err;
  vector epsptr(ncomp),epsa(ncomp),sig(ncomp),f(nyf),qtr(nhp);
  vector dq,dgamma,ff;
  matrix d(ncomp,ncomp),h(nhp,nhp);
  matrix fsig,fq,cpm,amq,am,acpm;
  
  //ni = Mp->nicp;
  //err = Mp->errcp;
  ni=0;
  err=10.0;
  
  //  initialization
  copyv (epsp,epsptr);
  copyv (q,qtr);
  
  stat = new long [nyf];
  
  //  main iteration loop
  for (i=0;i<ni;i++){

    subv (epsn,epsptr,epsa);
    
    //  elastic stiffness matrix
    elmatstiff (d,ipp);
    
    //  trial stress computation
    mxv (d,epsa,sig);
    


    /*
    switch (ip[ipp].tm[0]){
      //case mohrcoul3d:{
      //mc3d[ip[ipp].idm[0]].yieldfunction (sig,f);
      //break;
    }
    default:{
      fprintf (stderr,"\n\n unknown material type is required in function mult_surf_cutting_plane (mechmat.cpp).\n");
    }
    }
    */

    //  yield function control
    nas=0;
    for (j=0;j<nyf;j++){
      if (f[j]<err){      stat[j]=0;  }
      else{  nas++;       stat[j]=1;  }
    }
    if (nas==0)  break;

    
    allocm (nas,6,fsig);  allocm (nas,6,am);  allocm (nas,nas,cpm);
    allocm (nas,nhp,fq);  allocm (nhp,nas,amq);  allocm (nas,nas,acpm);
    allocv (nas,ff);  allocv (nas,dgamma);  allocv (nas,dq);
    
    /*
    switch (ip[ipp].tm[0]){

	case mohrcoul3d:{
        k=0;
	for (j=0;j<6;j++){
        if (stat[j]==1){
	mc3d[ip[ipp].idm].deryieldfsigma (sig,j,fsig[k]);
	ff[k]=f[j];
	k++;
        }
	}
	break;

    }
    default:{
      fprintf (stderr,"\n\n unknown material type is required in function mult_surf_cutting_plane (mechmat.cpp).\n");
    }
    }
    */
    
    mxm (fsig,d,am);
    mxmt (am,fsig,cpm);
    

    mxmt (h,fq,amq);
    mxm (fq,amq,acpm);

    addm (cpm,acpm,cpm);
    
    //  solution of auxiliary system of equations
    gemp (cpm.a,dgamma.a,ff.a,nas,1,Mp->zero,1);

    //  new internal variables
    k=0;
    for (j=0;j<nyf;j++){
      if (stat[j]==1){
        gamma[j]+=dgamma[k];
        k++;
      }
    }
    
    mtxv (fsig,dgamma,sig);
    mxv (amq,dgamma,dq);

    //  new plastic strains
    for (j=0;j<ncomp;j++){
      epsptr[j]+=sig[j];
    }
    //  new hardening parameters
    for (j=0;j<nhp;j++){
      qtr[j]-=dq[j];
    }


    destrm (fsig);  destrm (am);  destrm (cpm);
    destrm (fq);  destrm (amq);  destrm (acpm);
    destrv (ff);  destrv (dgamma);  destrv (dq);

  }

  copyv (epsptr,epsp);
  copyv (qtr,q);


  //  correct elastic strain
  subv (epsn,epsp,epsa);

  //  elastic stiffness matrix
  elmatstiff (d,ipp);
  
  //  correct stress components
  mxv (d,epsa,sig);

  storestress (0,ipp,sig);

}



/**
   Function returns stresses on sufrace of plasticity.
   Newton method is used.

   gamma,epsp and q will be replaced by new values

   @param ipp - integration point id
   @param im - id of material type (for combination of material models, default value is 0)
   @param ido - first index in eqother array (for combination of material models, default value is 0)
   @param gamma - consistency parameter
   @param epsn - total strain components
   @param epsp - plastic strain components
   @param q - hardening parameters
   @param ni - maximum number of iterations
   @param err - required error
   
   JK, 16.3.2005, revised 7.8.2005
*/
void mechmat::newton_stress_return (long ipp,long im,long ido,double &gamma,vector &epsn,vector &epsp,vector &q,long ni,double err)
{
  long i,j,k,n,ncompstr,ncomphard;
  double zero,f,dgamma,norstr,norstrincr,norconspar,norhardpar,norhardparincr;
  
  //  computer zero
  zero=Mp->zero;

  //  number of strain components
  ncompstr = epsn.n;
  //  number of hardening components
  ncomphard = q.n;
  //  number of unknowns in the Newton method
  n = ncompstr + 1 + ncomphard;
  //  unknowns in the Newton-Raphson method are organized as follows:
  //  increments of stresses (ncompstr components)
  //  increment of consistency parameter (1 component)
  //  increments of hardening/softening parameters (ncomphard components)
  //
  //  hardening parameters may be omitted
  
  //  array containing matrix of the system (Jacobian matrix)
  double *a;
  //  array containing solution of the system
  double *x;
  //  array containing right hand side
  double *y;
  //  stiffness matrix of the material
  matrix d(ncompstr,ncompstr);
  //  compliance matrix of the material
  matrix c(ncompstr,ncompstr);
  //  vector of elastic strains
  vector epse(ncompstr);
  //  vector of stresses
  vector sig(ncompstr);
  //  stress tensor
  matrix sigt(3,3);
  //  backup of the trial stresses
  vector sigtrial(ncompstr);
  //  backup of attained hardening parameters
  //vector qold(ncomphard);
  //  matrix of second derivatives of plastic potential with respect to stresses
  matrix dgdsds(ncompstr,ncompstr);
  //  vector of derivatives of plastic potential with respect to stresses
  vector dgds(ncompstr);
  //  vector of derivatives of yield function with respect to stresses
  vector dfds(ncompstr);
  //  matrix of second derivatives of plastic potential with respect to stresses and hardening parameters
  matrix dgdsdq(ncompstr,ncomphard);
  //  vector of derivatives of yield function with respect to hardening parameters
  vector dfdq(ncomphard);
  //  matrix of derivatives of hardening function with respect to stresses
  matrix dhds (ncomphard,ncompstr);
  //  vector of derivatives of hardening function with respect to consistency parameter
  vector dhdc (ncomphard);
  //  matrix of derivatives of hardening function with respect to hardening parameters
  matrix dhdq(ncomphard,ncomphard);
  //  hardening/softening parameters
  vector hs(ncomphard);

  //  auxiliary matrices
  matrix auxm(ncompstr,ncompstr),auxm2(ncompstr,ncomphard);
  //  auxiliary vector
  vector auxv(ncompstr),auxv2(ncompstr);


  //  allocation of memory for Newton method
  a = new double [n*n];
  x = new double [n];
  y = new double [n];
  
  //  elastic stiffness matrix
  elmatstiff (d,ipp);
  //  elastic compliance matrix
  elmatcompl (c,ipp);
  
  //  initialization of increment of consistency parameter
  dgamma=0.0;
  
  //  elastic strain
  subv (epsn,epsp,epse);
  //  trial stress computation
  mxv (d,epse,sig);
  
  //  initial stresses
  if (ic){
    for (j=0; j < ncompstr; j++)
      sig[j] += ic[ipp][3+j];
  }

  //  conversion of vector to tensor notation
  vector_tensor (sig,sigt,ip[ipp].ssst,stress);
  //  copy of hardening parameters 
  //copyv (q,qold);
  
  //  evaluation of the yield function
  f = yieldfunction (ipp,im,sigt,q);
  
  //  evaluation of hardening/softening function
  hardsoftfunction (ipp,im,sigt,q,hs);
  
  if (f>err){
    //  stress is outside from yield surface
    //  stress return algorithm must be used
    
    //  backup of the trial stresses (will be used on the right hand side)
    copyv (sig,sigtrial);
    
    //  main iteration loop
    for (i=0;i<ni;i++){
      
      //fprintf (Out,"\n %ld",i);

      nullv (a,n*n);
      nullv (x,n);
      nullv (y,n);
      
      //  Jacobian matrix assembling
      //  submatrix A_11
      
      //  matrix of second derivatives of plastic potential with respect to stresses
      dgdsigmadsigma (ipp, im, sigt, dgdsds);
      
      mxm (d,dgdsds,auxm);
      cmulm (dgamma,auxm);
      
      //fprintf (Out,"\n d . dgdsds . dgamma  %le",auxm[0][0]);

      for (j=0;j<ncompstr;j++){
	for (k=0;k<ncompstr;k++){
	  a[j*n+k]=auxm[j][k];
	}
	a[j*n+j]+=1.0;
      }
      
      //  submatrix A_12
      
      //  vector of first derivatives of plastic potential with respect to stresses
      dgdsigma (ipp, im, sigt, q, dgds);
      
      mxv (d,dgds,auxv);
      
      //fprintf (Out,"\n d . dgds   %le",auxv[0]);
      
      for (j=0;j<ncompstr;j++){
	a[j*n+ncompstr]=auxv[j];
      }
      
      //  submatrix A_21
      
      //  vector of first derivatives of yield function with respect to stresses
      dfdsigma (ipp, im, sigt, q, dfds);
      
      //fprintf (Out,"\n dfds    %le",dfds[0]);

      for (j=0;j<ncompstr;j++){
	a[ncompstr*n+j]=dfds[j];
      }
      
      //  submatrix A_22
      //  there is no contribution to the A_22 submatrix
      
      
      //  submatrices generated by hardening terms
      if (ncomphard>0){
	
	//  submatrix A_13
	
	//  matrix of second derivatives of plastic potential with respect to stresses and hardening parameters
	//dgdsigmadq (ipp,im,sigt,q,dgdsdq);
	
	//mxm (d,dgdsdq,auxm2);
	
	for (j=0;j<ncompstr;j++){
	  for (k=0;k<ncomphard;k++){
	    a[j*n+ncompstr+1+k]=auxm2[j][k]*dgamma;
	  }
	}
	
	//fprintf (Out,"\n d . dgdsdq . dgamma    %le",auxm2[0][0]*dgamma);
	
	//  submatrix A_31

	//  matrix of derivatives of hardening function with respect to stresses
	//dhdsigma (ipp,im,epsp,epsn,sigt,dhds);
	
	//fprintf (Out,"\n dhds   %le",dhds[0][0]);
	
	for (j=0;j<ncomphard;j++){
	  for (k=0;k<ncompstr;k++){
	    a[(ncompstr+1+j)*n+k]=dhds[j][k];
	  }
	}
	
	//  submatrix A_23
	
	//  vector of derivatives of yield function with respect to hardening parameters
	//dfdqpar (ipp,im,sigt,q,dfdq);
	
	//fprintf (Out,"\n dfdq    %le",dfdq[0]);
	
	for (j=0;j<ncomphard;j++){
	  a[ncompstr*n+ncompstr+1+j]=dfdq[j];
	}
	
	//  submatrix A_32
	
	//  vector of derivatives of hardening function with respect to consistency parameter
	//dhdgamma (ipp,im,epsp,sigt,q,dhdc);
	
	//fprintf (Out,"\n dhdc   %le",dhdc[0]);

	for (j=0;j<ncomphard;j++){
	  a[(ncompstr+1+j)*n+ncompstr]=0.0-dhdc[j];
	}
	

	//  submatrix A_33
	
	//  matrix of derivatives of hardening function with respect to hardening parameters
	//dhdqpar (ipp,im,dhdq);
	
	//fprintf (Out,"\n dhdq   %le",dhdq[0][0]);

	for (j=0;j<ncomphard;j++){
	  for (k=0;k<ncomphard;k++){
	    a[(ncompstr+1+j)*n+ncompstr+1+k]=dhdq[j][k];
	  }
	  a[(ncompstr+1+j)*n+ncompstr+1+j]-=1.0;
	}
      }
      
      
      //  right hand side assembling
      //  subvector y_1
      
      for (j=0;j<ncompstr;j++){
	y[j]=sigtrial[j]-sig[j]-dgamma*auxv[j];
      }
      
      //  subvector y_2
      y[ncompstr]=0.0-f;
      
      //  subvector y_3
      for (j=0;j<ncomphard;j++){
	//y[ncompstr+1+j]=q[j]-qold[j];
	//y[ncompstr+1+j]=q[j]-hs[j];
      }
      
      //fprintf (Out,"\n prava strana \n");
      //for (j=0;j<n;j++){
      //fprintf (Out,"  %le",y[j]);
      //}
      
      //fprintf (Out,"\n matice");
      //for (j=0;j<n;j++){
      //fprintf (Out,"\n");
      //for (k=0;k<n;k++){
      //fprintf (Out,"  %le",a[j*n+k]);
      //}
      //}

      //  solution of linearized system of equations
      gemp (a,x,y,n,1,1.0e-10,1);
      
      //fprintf (Out,"\n reseni \n");
      //for (j=0;j<n;j++){
      //fprintf (Out,"  %le",x[j]);
      //}
      
      //  norm of stresses
      norstr=0.0;
      //  norm of stress increments
      norstrincr=0.0;
      
      for (j=0;j<ncompstr;j++){
	norstrincr+=x[j]*x[j];
	norstr+=sig[j]*sig[j];
      }
      
      norconspar=x[ncompstr]*x[ncompstr];
      
      //  norm of hardening parameters
      norhardpar=0.0;
      //  norm of increments of hardening parameters
      norhardparincr=0.0;
      
      for (j=0;j<ncomphard;j++){
	norhardpar+=q[j]*q[j];
      }
      for (j=ncompstr+1;j<ncompstr+1+ncomphard;j++){
	norhardparincr+=x[j]*x[j];
      }
      
      
      //  stress update
      for (j=0;j<ncompstr;j++){
	sig[j]+=x[j];
	auxv[j]=x[j];
      }
      
      //  consistency parameter update
      dgamma+=x[ncompstr];
      
      //  hardening parameter update
      for (j=0;j<ncomphard;j++){
	q[j]+=x[ncompstr+1+j];
      }
      
	
      //fprintf (Out,"\n");
      //for (j=0;j<ncompstr;j++){
      //fprintf (Out,"   %le",sig[j]);
      //}
      //fprintf (Out,"       %le",dgamma);
      //for (j=0;j<ncomphard;j++){
      //fprintf (Out,"  %le",q[j]);
      //}
      
      //  plastic strains update
      mxv (c,auxv,auxv2);
      
      for (j=0;j<ncompstr;j++){
	epsp[j]-=auxv2[j];
      }


      //  conversion of vector to tensor notation
      vector_tensor (sig,sigt,ip[ipp].ssst,stress);

      //updateq (ipp,im,ido,dgamma,epsn,epsp,sigt,q);
	
      //  evaluation of the yield function
      f = yieldfunction (ipp,im,sigt,q);
      
      //  evaluation of hardening/softening function
      hardsoftfunction (ipp,im,sigt,q,hs);


      //for (j=0;j<ncompstr;j++){
      //fprintf (Out,"   %le",epsp[j]);
      //}



      
      j=0;  k=0;
      if (norstr>zero){
	j++;
	if (norstrincr/norstr<err)  k++;
      }
      if (norhardpar>zero){
	j++;
	if (norhardparincr/norhardpar<err)  k++;
      }
      if (dgamma>zero){
	j++;
	if (norconspar/dgamma<err)  k++;
      }
      
      
      if (j==k){
	/*
	  fprintf (stdout,"\n bod %ld  yield function  %le   q %le    epsp %le %le %le   nor %le",
	  ipp,f,q[0],epsp[0],epsp[1],epsp[2], epsp[0]*epsp[0]+epsp[1]*epsp[1]+epsp[2]*epsp[2]);
	*/
	break;
      }
    }
  }
  
  //  update of consistency parameter
  gamma+=dgamma;
  
  //  storing resulting stresses
  storestress (0,ipp,sig);
  
  delete [] a;
  delete [] x;
  delete [] y;
}



/**
   Function returns stresses on sufrace of plasticity.
   Newton method is used.

   gamma,epsp and q will be replaced by new values

   @param ipp - integration point id
   @param im - id of material type (for combination of material models, default value is 0)
   @param ido - first index in eqother array (for combination of material models, default value is 0)
   @param gamma - consistency parameter
   @param epsn - total strain components
   @param epsp - plastic strain components
   @param q - hardening parameters
   @param ni - maximum number of iterations
   @param err - required error
   
   JK, 16.2.2007
*/
void mechmat::newton_stress_return_2 (long ipp,long im,long ido,double &gamma,vector &epsn,vector &epsp,vector &q,long ni,double err)
{
  long i,j,k,n,ncompstr,ncomphard;
  double zero,f,dgamma,norstr,norstrincr,norconspar,norhardpar,norhardparincr;


  //  computer zero
  zero=Mp->zero;

  //  number of strain components
  ncompstr = epsn.n;
  //  number of hardening components
  ncomphard = q.n;
  //  number of unknowns in the Newton method
  n = ncompstr + 1 + ncomphard;
  //  unknowns in the Newton-Raphson method are organized as follows:
  //  increments of stresses (ncompstr components)
  //  increment of consistency parameter (1 component)
  //  increments of hardening/softening parameters (ncomphard components)
  //
  //  hardening parameters may be omitted
  vector qold(ncomphard);

  //  array containing matrix of the system (Jacobian matrix)
  double *a;
  //  array containing solution of the system
  double *x;
  //  array containing right hand side
  double *y;
  //  stiffness matrix of the material
  matrix d(ncompstr,ncompstr);
  //  compliance matrix of the material
  matrix c(ncompstr,ncompstr);
  //  vector of elastic strains
  vector epse(ncompstr);
  //  vector of stresses
  vector sig(ncompstr);
  //  stress tensor
  matrix sigt(3,3);
  //  backup of the trial stresses
  vector sigtrial(ncompstr);
  //  backup of attained hardening parameters
  //vector qold(ncomphard);
  //  matrix of second derivatives of plastic potential with respect to stresses
  matrix dgdsds(ncompstr,ncompstr);
  //  vector of derivatives of plastic potential with respect to stresses
  vector dgds(ncompstr);
  //  vector of derivatives of yield function with respect to stresses
  vector dfds(ncompstr);
  //  matrix of second derivatives of plastic potential with respect to stresses and hardening parameters
  matrix dgdsdq(ncompstr,ncomphard);
  //  vector of derivatives of yield function with respect to hardening parameters
  vector dfdq(ncomphard);
  //  matrix of derivatives of hardening function with respect to stresses
  matrix dhds (ncompstr,ncomphard);
  //  vector of derivatives of hardening function with respect to consistency parameter
  vector dhdc (ncomphard);
  //  matrix of derivatives of hardening function with respect to hardening parameters
  matrix dhdq(ncomphard,ncomphard);
  //  hardening/softening parameters
  vector hs(ncomphard);

  //  auxiliary matrices
  matrix auxm(ncompstr,ncompstr),auxm2(ncompstr,ncomphard);
  //  auxiliary vector
  vector auxv(ncompstr),auxv2(ncompstr);


  //  allocation of memory for Newton method
  a = new double [n*n];
  x = new double [n];
  y = new double [n];
  
  //  elastic stiffness matrix
  elmatstiff (d,ipp);
  //  elastic compliance matrix
  elmatcompl (c,ipp);
  
  //  initialization of increment of consistency parameter
  dgamma=0.0;
  
  //  elastic strain
  subv (epsn,epsp,epse);
  //  trial stress computation
  mxv (d,epse,sig);
  
  //  initial stresses
  if (ic){
    for (j=0; j < ncompstr; j++)
      sig[j] += ic[ipp][3+j];
  }

  //  conversion of vector to tensor notation
  vector_tensor (sig,sigt,ip[ipp].ssst,stress);
  //  copy of hardening parameters 
  copyv (q,qold);
  
  //  evaluation of the yield function
  f = yieldfunction (ipp,im,sigt,q);
  
  //  evaluation of hardening/softening function
  hardsoftfunction (ipp,im,sigt,q,hs);

  if (f>err){
    //  stress is outside from yield surface
    //  stress return algorithm must be used
    
    //  backup of the trial stresses (will be used on the right hand side)
    copyv (sig,sigtrial);
    
    //  main iteration loop
    for (i=0;i<ni;i++){
      
      //fprintf (Out,"\n %ld",i);

      nullv (a,n*n);
      nullv (x,n);
      nullv (y,n);
      
      //  Jacobian matrix assembling
      //  submatrix A_11
      
      //  matrix of second derivatives of plastic potential with respect to stresses
      dgdsigmadsigma (ipp, im, sigt, dgdsds);
      
      mxm (d,dgdsds,auxm);
      cmulm (dgamma,auxm);
      
      //fprintf (Out,"\n d . dgdsds . dgamma  %le",auxm[0][0]);

      for (j=0;j<ncompstr;j++){
	for (k=0;k<ncompstr;k++){
	  a[j*n+k]=auxm[j][k];
	}
	a[j*n+j]+=1.0;
      }
      
      //  submatrix A_12
      
      //  vector of first derivatives of plastic potential with respect to stresses
      dgdsigma (ipp, im, sigt, q, dgds);
      
      mxv (d,dgds,auxv);
      
      //fprintf (Out,"\n d . dgds   %le",auxv[0]);
      
      for (j=0;j<ncompstr;j++){
	a[j*n+ncompstr]=auxv[j];
      }
      
      //  submatrix A_21
      
      //  vector of first derivatives of yield function with respect to stresses
      dfdsigma (ipp, im, sigt, q, dfds);
      
      //fprintf (Out,"\n dfds    %le",dfds[0]);

      for (j=0;j<ncompstr;j++){
	a[ncompstr*n+j]=dfds[j];
      }
      
      //  submatrix A_22
      //  there is no contribution to the A_22 submatrix
      
      
      //  submatrices generated by hardening terms
      if (ncomphard>0){
	
	//  submatrix A_13
	
	//  matrix of second derivatives of plastic potential with respect to stresses and hardening parameters
	dgdsigmadq (ipp,im,sigt,q,dgdsdq);
	
	mxm (d,dgdsdq,auxm2);
	
	for (j=0;j<ncompstr;j++){
	  for (k=0;k<ncomphard;k++){
	    a[j*n+ncompstr+1+k]=auxm2[j][k]*dgamma;
	  }
	}
	
	//fprintf (Out,"\n d . dgdsdq . dgamma    %le",auxm2[0][0]*dgamma);
	
	//  submatrix A_31
	
	//  matrix of derivatives of hardening function with respect to stresses
	dhdsigma (ipp,im,sigt,q,dhds);
	
	
	//fprintf (Out,"\n dhds   %le",dhds[0][0]);
	
	for (j=0;j<ncomphard;j++){
	  for (k=0;k<ncompstr;k++){
	    a[(ncompstr+1+j)*n+k]=dhds[k][j]*dgamma;
	  }
	}
	
	//  submatrix A_23
	
	//  vector of derivatives of yield function with respect to hardening parameters
	dfdqpar (ipp,im,sigt,q,dfdq);
	
	//fprintf (Out,"\n dfdq    %le",dfdq[0]);
	
	for (j=0;j<ncomphard;j++){
	  a[ncompstr*n+ncompstr+1+j]=dfdq[j];
	}
	
	//  submatrix A_32
	
	//  vector of derivatives of hardening function with respect to consistency parameter
	dhdgamma (ipp,im,epsp,sigt,q,dhdc);
	
	//fprintf (Out,"\n dhdc   %le",dhdc[0]);
	
	for (j=0;j<ncomphard;j++){
	  a[(ncompstr+1+j)*n+ncompstr]=dhdc[j];
	}
	
	//  submatrix A_33
	
	//  matrix of derivatives of hardening function with respect to hardening parameters
	dhdqpar (ipp,im,sigt,q,dhdq);
	
	//fprintf (Out,"\n dhdq   %le",dhdq[0][0]);
	
	for (j=0;j<ncomphard;j++){
	  for (k=0;k<ncomphard;k++){
	    a[(ncompstr+1+j)*n+ncompstr+1+k]=dhdq[j][k];
	  }
	  a[(ncompstr+1+j)*n+ncompstr+1+j]-=1.0;
	}

      }
      
      //  right hand side assembling
      //  subvector y_1
      
      for (j=0;j<ncompstr;j++){
	y[j]=sigtrial[j]-sig[j]-dgamma*auxv[j];
      }
      
      //  subvector y_2
      y[ncompstr]=0.0-f;
      
      //  subvector y_3
      for (j=0;j<ncomphard;j++){
	//y[ncompstr+1+j]=q[j]-hs[j];
      }
      
      //fprintf (Out,"\n prava strana \n");
      //for (j=0;j<n;j++){
      //fprintf (Out,"  %le",y[j]);
      //}
      
      //fprintf (Out,"\n matice");
      //for (j=0;j<n;j++){
      //fprintf (Out,"\n");
      //for (k=0;k<n;k++){
      //fprintf (Out,"  %le",a[j*n+k]);
      //}
      //}

      //  solution of linearized system of equations
      gemp (a,x,y,n,1,1.0e-10,1);
      
      //fprintf (Out,"\n reseni \n");
      //for (j=0;j<n;j++){
      //fprintf (Out,"  %le",x[j]);
      //}
      
      //  norm of stresses
      norstr=0.0;
      //  norm of stress increments
      norstrincr=0.0;
      
      for (j=0;j<ncompstr;j++){
	norstrincr+=x[j]*x[j];
	norstr+=sig[j]*sig[j];
      }
      
      norconspar=x[ncompstr]*x[ncompstr];
      
      //  norm of hardening parameters
      norhardpar=0.0;
      //  norm of increments of hardening parameters
      norhardparincr=0.0;
      
      for (j=0;j<ncomphard;j++){
	norhardpar+=q[j]*q[j];
      }
      for (j=ncompstr+1;j<ncompstr+1+ncomphard;j++){
	norhardparincr+=x[j]*x[j];
      }
      
      
      //  stress update
      for (j=0;j<ncompstr;j++){
	sig[j]+=x[j];
	auxv[j]=x[j];
      }
      
      //  consistency parameter update
      dgamma+=x[ncompstr];
      
      //  hardening parameter update
      for (j=0;j<ncomphard;j++){
	q[j]+=x[ncompstr+1+j];
      }
      
	
      //fprintf (Out,"\n");
      //for (j=0;j<ncompstr;j++){
      //fprintf (Out,"   %le",sig[j]);
      //}
      //fprintf (Out,"       %le",dgamma);
      //for (j=0;j<ncomphard;j++){
      //fprintf (Out,"  %le",q[j]);
      //}
      
      //  plastic strains update
      mxv (c,auxv,auxv2);
      
      for (j=0;j<ncompstr;j++){
	epsp[j]-=auxv2[j];
      }


      //  conversion of vector to tensor notation
      vector_tensor (sig,sigt,ip[ipp].ssst,stress);

      //  evaluation of the yield function
      f = yieldfunction (ipp,im,sigt,q);
      
      //  evaluation of hardening/softening function
      hardsoftfunction (ipp,im,sigt,q,hs);


      //for (j=0;j<ncompstr;j++){
      //fprintf (Out,"   %le",epsp[j]);
      //}



      
      j=0;  k=0;
      if (norstr>zero){
	j++;
	if (norstrincr/norstr<err)  k++;
      }
      if (norhardpar>zero){
	j++;
	if (norhardparincr/norhardpar<err)  k++;
      }
      if (dgamma>zero){
	j++;
	if (norconspar/dgamma<err)  k++;
      }
      
      
      if (j==k){
	/*
	  fprintf (stdout,"\n bod %ld  yield function  %le   q %le    epsp %le %le %le   nor %le",
	  ipp,f,q[0],epsp[0],epsp[1],epsp[2], epsp[0]*epsp[0]+epsp[1]*epsp[1]+epsp[2]*epsp[2]);
	*/
	break;
      }
    }
  }
  
  fprintf (stdout,"\n\n parametr zpevneni  %lf",q[0]);
  fprintf (stdout,"\n parametr konzistence  %lf\n",gamma);
  
  //  update of consistency parameter
  gamma+=dgamma;
  
  //  storing resulting stresses
  storestress (0,ipp,sig);
  
  delete [] a;
  delete [] x;
  delete [] y;
}















/**
  Function computes D \delta \epsilon
  it is used for right hand side computation in time dependent problems.

  @param ipp - integration point pointer
  @param im - material index
  @param ido - index of internal variables for given material in the ipp other array
  @param sig - stress components
  @param q - internal variables
  @param dt - time increment
   
   28.10.2001
*/
void mechmat::stiff_deps_vispl (long ipp,long im,long ido,vector &sig,vector &q,double dt)
{
  long i,ncomp,ncomph,ncompo;
  //  number of strain/stress components
  ncomp=sig.n;
  //  number of internal parameters
  ncomph=q.n;
  
  //  number of components of array eqother belonging to viscous material
  ncompo = givencompother(ipp,im+1);
  
  double f,g,dlambda,cs;
  vector epsp(ncomp),dq(ncomph),dh(ncomph);
  matrix d(ncomp,ncomp),sigt(3,3),epspt(3,3), h(ncomph, ncomph);
  
  vector_tensor (sig,sigt,ip[ipp].ssst,stress);
  //  value of yield function
  f = yieldfunction (ipp,im+2,sigt,q);
  //  derivative of yield function with respect to stress components
  //  function rewrites array sig by the derivatives of yield function with respect to stress components
  dfdsigma (ipp,im+2,sigt,q,sig);
  //  derivative of yield function with respect to hardening parameters
  dfdqpar (ipp,im+2,sigt,q,dq);
  
  switch (ip[ipp].tm[im+1]){
  case simvisplas:{
    //  viscous function value
    g = svipl[ip[ipp].idm[im+1]].gfun (f);
    break;
  }
  case simvisc:{
    //  viscous function value
    g = svis[ip[ipp].idm[im+1]].gfun (f);
    break;
  }
  case lemaitr:{
    //  cumulative deformation
    cs=ip[ipp].other[ido+3*ncomp];
    //  viscous function value
    g = lmtr[ip[ipp].idm[im+1]].gfun (f,cs);
    break;
  }
  default:{
    print_err("unknown material type is required",__FILE__,__LINE__,__func__);
  }
  }
  
  dlambda = 0.0;
  tensor_vector (sig,sigt,ip[ipp].ssst,stress);
  if (g>0.0){
    //  lambda increment
    dlambda = dt*g;
    //  irreversible strain increment
    for (i=0;i<ncomp;i++){
      epsp[i]=dlambda*sig[i];
    }

    //  hardening parameter increment
    plasticmoduli (ipp, im+2, epsp, sigt, h);
    mxv (h,dq,dh);
    for (i=0;i<ncomph;i++){
      dh[i]=-dlambda*dh[i];
    }
  }

  //  storage of new values
  switch (ip[ipp].tm[im+1]){
  case simvisc:{
    //  irreversible strain increment
    for (i=0;i<ncomp;i++){
      ip[ipp].eqother[ido+ncompo+i]=epsp[i];
    }
    //  internal variable increment
    //ip[ipp].other[ido+3*ncomp]+=dh[0];
    
    ip[ipp].eqother[ido+ncompo+ncomp]+=dlambda;
    for (i=0;i<ncomph;i++){
      ip[ipp].eqother[ido+ncompo+ncomp+1+i]+=dh[i];
    }
    //store_interparam (ipp,im,ido,dh);
    
    break;
  }
  case simvisplas:{
    //  irreversible strain increment
    for (i=0;i<ncomp;i++){
      ip[ipp].eqother[ido+ncomp+i]=epsp[i];
    }
    //  internal variable increment
    ip[ipp].eqother[ido+3*ncomp]+=dh[0];
    break;
  }
  case lemaitr:{
    //  irreversible strain increment
    for (i=0;i<ncomp;i++){
      ip[ipp].eqother[ido+i+ncomp]=epsp[i];
    }
    //  cumulative irreversible strain computation

    vector_tensor (epsp,epspt,ip[ipp].ssst,strain);
    cumulstrain (epspt,cs);
    ip[ipp].eqother[ido+ncomp*3]=cs;
    //  internal variable increment
    for (i=0;i<ncomph;i++){
      ip[ipp].eqother[ido+i+ncomp*3+1]+=dh[i];
    }
    break;
  }
  default:{
    print_err("unknown material type is requiered",__FILE__,__LINE__,__func__);
  }
  }

  //  stress increments
  matstiff (d,ipp);
  mxv (d,epsp,sig);
  //storestress (0,ipp,sig);

}


/**
   Function computes contribution to the right hand side
   in creep and shrinkage analysis.
   
   @param ipp - integration point pointer
   
   9.11.2001
*/
void mechmat::stiff_eps (long ipp)
{
  long i,ncomp=ip[ipp].ncompstr;
  double time,
    complian,
    strength,
    shrink;
  vector eps(ncomp),sig(ncomp);
  
  time=Mp->time/3600.0/24.0;
  
  switch (ip[ipp].tm[0]){
  case aci:{
    aci78mod[ip[ipp].idm[0]].compliance (time,complian,strength,shrink);
    break;
  }
  case cebfip:{
    cebfip78mod[ip[ipp].idm[0]].compliance (time,complian,strength,shrink);
    break;
  }
  default:
    print_err("unknown material type is requiered",__FILE__,__LINE__,__func__);
  }
  
  
  //  strain components
  for (i=0;i<ncomp;i++){
    ip[ipp].strain[i]=ip[ipp].stress[i]*complian;
  }
  
}


/**
   Function returns damage function parameters for actual values of total strains
   for scalar damage models.

   @param ipp - integration point pointer
   @param eps - total strain components
   @param im  - index of material type for given ip
   @param kappa - damage function parameters

   Created 27.1.2003
   Tomas Koudelka

*/
void mechmat::damfuncpar(long ipp, long im, vector &eps, vector &kappa)
{
  switch (ip[ipp].tm[im])
  {
    case scaldamage:
    {
      scdam[ip[ipp].idm[im]].damfuncpar(ipp, eps, kappa);
      break;
    }
    case scaldamagecc:
    {
      scdamcc[ip[ipp].idm[im]].damfuncpar(ipp, eps, kappa);
      break;
    }
    case scaldamage_time:
    {
      scdamtime[ip[ipp].idm[im]].damfuncpar(ipp, eps, kappa);
      break;
    }
    default:
      print_err("unknown scalar damage model is required",__FILE__,__LINE__,__func__);
  }
  return;
}



/**
   Function returns damage function for actual values of kappa
   for scalar damage models.

   @param ipp   - integration point pointer
   @param im - material index
   @param ido - index of internal variables for given material in the ipp other array
   @param kappa - damage function parameters
   @param eps   - strain vector

   Created 27.1.2003
   Tomas Koudelka
*/
double mechmat::damfunction(long ipp, long im, long ido,vector &kappa, vector &eps)
{
  double ret = 0.0;

  switch (ip[ipp].tm[im])
  {
    case scaldamage:
    {
      ret = scdam[ip[ipp].idm[im]].damfunction(ipp,kappa);
      break;
    }
    case scaldamagecc:
    {
      ret = scdamcc[ip[ipp].idm[im]].damfunction(ipp,kappa,eps);
      break;
    }
    case scaldamage_time:
    {
      ret = scdamtime[ip[ipp].idm[im]].damfunction(ipp,kappa);
      break;
    }
    default:
      print_err("unknown scalar damage model is required",__FILE__,__LINE__,__func__);
  }
  return ret;
}


/**
   Function returns limit elastic strain
   for scalar damage models.

   @param ipp   - integration point pointer
   @param im   - index of damage material in the ip.tm array

   Created 23.2.2003
   Frdric DUFOUR

*/
double mechmat::epsefunction(long ipp, long im)
{
  double ret = 0.0;

  switch (ip[ipp].tm[im])
  {
    case scaldamage:
    {
      ret = scdam[ip[ipp].idm[im]].epsefunction(ipp);
      break;
    }
    case scaldamagecc:
    {
      ret = scdamcc[ip[ipp].idm[im]].epsefunction();
      break;
    }
    case scaldamage_time:
    {
      ret = scdamtime[ip[ipp].idm[im]].epsefunction(ipp);
      break;
    }
    default:
      print_err("unknown scalar damage model in function",__FILE__,__LINE__,__func__);
  }
  return ret;
}


/**
   Function returns stresses for scalar damage models and value of damage function.

   @param ipp    - integration point pointer
   @param im     - material index
   @param ido    - index of internal variables for given material in the ipp other array
   @param eps    - total strain components
   @param kappa  - damage function parameters from previous stesp
   @param omegao - damage parameters from the previous step
   @param sigma  - result stresses
   @param d      - elastic stiffness matrix with respect to timedependent Young modulus

   27.1.2003
   Author Tomas Koudelka
*/
double mechmat::scal_dam_sol (long ipp,long im,long ido,vector &eps,vector &kappa, vector &omegao, vector &sigma, matrix &d)
{
  long nk = kappa.n;
//  long ncomp=ip[ipp].ncompstr;
  double epse, omega;
  vector updkappa(nk);
//  matrix d(ncomp, ncomp);


  if (nk > 1)
  {
    print_err("unsupported number of componetnts of vector kappa", __FILE__, __LINE__,__func__);
    return 0.0;
  }

  if ((Mp->matmodel == local) || ((Mm->ip[ipp].hmt & 2) == 0)) // model is not nonlocal
  // local model is used
  {
    damfuncpar(ipp, im, eps, updkappa);
    if (updkappa[0] > kappa[0])
    {
      kappa[0] = updkappa[0];
      epse = epsefunction(ipp,im) ;
      if (kappa[0] > epse)
        omega = damfunction(ipp, im, ido, kappa,eps);
      else
        omega = 0.0;
      if (omega < omegao[0])
        omega = omegao[0];
    }
    else
      omega = omegao[0];

    mxv(d, eps, sigma);
    cmulv(1.0-omega, sigma);
    return omega;
  }

  if (Mp->matmodel == nonlocal)
  // nonlocal model is used
  {
    if (Mp->nonlocphase == 1)
    // compute local values for averaging -> nonlocphase=1 -> no stresses are required
    {
      damfuncpar(ipp, im, eps, updkappa);
      kappa[0] = updkappa[0]; // the updating of the kappa is required
      fillv(0.0, sigma);
      return 0.0;
    }
    if (Mp->nonlocphase == 2)
    // compute stresses from the averaged nonlocal values -> nonlocphase=2
    {
      updkappa[0] = ip[ipp].nonloc[0];
      if (updkappa[0] > kappa[0])
      {
        kappa[0] = updkappa[0];
        epse = epsefunction(ipp, im);
        if (kappa[0] > epse)
          omega = damfunction(ipp, im, ido,kappa,eps);
        else
          omega = 0.0;
        if (omega < omegao[0])
          omega = omegao[0];
      }
      else
        omega = omegao[0];

      mxv(d, eps, sigma);
      cmulv(1.0-omega, sigma);
      return omega;
    }
  }

  // Following command is only for formal purposes (compiler warnings)
  return 0.0;
}


/*
void mechmat::visc_cutting_plane (long ipp,double &gamma,vector &epsn,vector &epsp,vector &q,
                                  double gt,double gdt)
  //
  //  gamma,epsp and q will be replaced by new values
  //
  //  ipp - integration point pointer
  //  gamma - consistency parameter
  //  epsn - total strain components
  //  epsp - plastic strain components
  //  q - hardening parameters
  //  gt - time t_{n+1} in global time discretization
  //  gdt - time step in global time discretization
  //
  // 7.10.2001
{
  long i,j,stop,ncomp=epsn.n,ncomph=q.n,ni;
  double f,denom,denomq,dgamma,err,gf,dg,eta,tr,dt,t;
  vector epsptr(ncomp),epsa(ncomp),qtr(ncomph),hp(ncomph),sig(ncomp);
  matrix d(ncomp,ncomp);

  ni = Mp->nicp;
  err = Mp->errcp;

  //  initialization
  i = copyv (epsp,epsptr);
  i = copyv (q,qtr);
  dgamma=0.0;
  stop=0;
  t=gt-gdt;

  //  main iteration loop
  for (i=0;i<ni;i++){
    //  elastic strain
    subv (epsn,epsptr,epsa);
    //  elastic stiffness matrix
    elmatstiff (d,ipp);
    //  trial stress computation
    mxv (d,epsa,sig);

    switch (ip[ipp].tm){
    case simvisplas:{
      f = svipl1d[ip[ipp].idm].yieldfunction (sig,qtr);
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown material type is required in function visc_cutting_plane (mechmat.cpp).\n");
    }
    }
    
    if (f<err)  break;
    
    switch (ip[ipp].tm){
    case simvisplas:{
      svipl1d[ip[ipp].idm].deryieldfsigma (sig,sig);
      svipl1d[ip[ipp].idm].deryieldfq (q,hp);
      dg=svipl1d[ip[ipp].idm].dergfun (f);
      gf=svipl1d[ip[ipp].idm].gfun (f);
      eta=svipl1d[ip[ipp].idm].eta;
      break;
    }
    default:{
      fprintf (stderr,"\n\n unknown material type is required in function visc_cutting_plane (mechmat.cpp).\n");
    }
    }
    
    mxv (d,sig,epsa);
    scprd (sig,epsa,denom);

    scprd (q,q,denomq);
    denom+=denomq;
    
    
    tr=eta/dg/denom;
    dt=tr*(1.0-exp(-1.0*gdt/tr));
    t+=dt;
    if (t-gt>=0.0){
      dt-=t-gt;  stop=1;
    }
    
    dgamma = dt*gf/eta;
    
    //  value updating
    for (j=0;j<ncomp;j++){
      epsptr[j]+=dgamma*sig[j];
    }
    for (j=0;j<ncomph;j++){
      qtr[j]-=dgamma*hp[j]*q[j];
    }
    
    if (stop==1)  break;
  }
  
  //fprintf (out,"\n number of performed iterations %ld",i);

  i = copyv (epsptr,epsp);
  i = copyv (qtr,q);

  
  //  elastic strain
  subv (epsn,epsptr,epsa);
  //  elastic stiffness matrix
  elmatstiff (d,ipp);
  //  trial stress computation
  mxv (d,epsa,sig);

  storestress (ipp,sig);

}
*/
/*
void mechmat::visc_closest_point_proj (long ipp,vector &epsn,vector &epsp,double &gamma,vector &q)
  //  d - odpovida male delte
  //  dd odpovida velke delte
  //
  //  ncomp - number of strain/stress components
  //  nhp - number of hardening parameters
  //
  //  epsn(ncomp,1) - vector of new strain
  //  epsp(ncomp,1) - vector of attained irreversible strain
  //  q(nhp,1) - vector of attained hardening parameters
  //  gamma(1,1) - plastic parameter
  //
  //  r(ncomp,1) - vector of direction of plastic increment (for associative plasticity is equal to df/dsigma)
  //  rs(ncomp,ncomp) - derivative of vector r with respect to sigma
  //  rq(ncomp,nhp) - derivative of vector r with respect to q
  //  h(nhp,1) - vector of direction of hardening parameters increment (for associative plasticity is equal to df/dq)
  //  hs(nhp,ncomp) - derivative of vector h with respect to sigma
  //  hq(nhp,nhp)) - derivative of vector h with respect to q
  //  f(1,1) - value of plastic function
  //  fs(1,ncomp) - derivate of plastic function with respect to sigma
  //  fq(1,nhp) - derivative of plastic function with respect to q
  //
  //  res(ncomp+nhp,1) - vector of residuals
  //
{
  long ni,i,j,ncomp=epsn.n,nhp=q.n;
  double err,ddt,f,dg,norres,dgamma,ddgamma;
  vector r(ncomp),h(nhp),sig(ncomp),res(ncomp+nhp),fs(ncomp),fq(nhp);
  vector epsptr(ncomp),ddepsptr(ncomp),qtr(nhp),ddqtr(nhp),epsa(ncomp);
  matrix sm(ncomp,ncomp),cm(ncomp,ncomp),rs(ncomp,ncomp),rq(ncomp,nhp),hs(nhp,ncomp),hq(nhp,nhp);
  matrix b(ncomp+nhp,ncomp+nhp),lhs(ncomp+nhp,2),rhs(ncomp+nhp,2);

  ni = Mp->nicppvp;
  err = Mp->errcppvp;
  ddt = Mp->ddtcppvp;

  copyv (epsp,epsptr);
  copyv (q,qtr);

  //  trial elastic strain
  subv (epsn,epsptr,epsa);
  //  stiffness matrix of the material
  elmatstiff (sm,ipp);
  //  trial stress
  mxv (sm,epsa,sig);

  switch (ip[ipp].tm){
  case simvisplas:{
    f=svipl1d[ip[ipp].idm].yieldfunction (sig,qtr);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown material type is required in function");
    fprintf (stderr,"\n mechmat::visc_closest_point_proj (mechmat.cpp).\n");
  }
  }
  
  if (f>err){
    
    dgamma=0.0;

    for (i=0;i<ni;i++){
      
      //  trial elastic strain
      subv (epsn,epsptr,epsa);
      //  stiffness matrix of the material
      elmatstiff (sm,ipp);
      //  trial stress
      mxv (sm,epsa,sig);
      
      switch (ip[ipp].tm){
      case simvisplas:{
        f=svipl1d[ip[ipp].idm].yieldfunction (sig,qtr);
        svipl1d[ip[ipp].idm].deryieldfsigma (sig,r);
        svipl1d[ip[ipp].idm].deryieldfsigma (sig,fs);
        dg=svipl1d[ip[ipp].idm].dergfun (f);
        break;
      }
      default:{
        fprintf (stderr,"\n\n unknown material type is required in function");
        fprintf (stderr,"\n mechmat::visc_closest_point_proj (mechmat.cpp).\n");
      }
      }
      
      //  vector of residuals
      for (j=0;j<ncomp;j++){
        res[j]=epsp[j]-epsptr[j]+dgamma*r[j];
      }
      for (j=0;j<nhp;j++){
        res[j+ncomp]=q[j]-qtr[j]+dgamma*h[j];
      }
      
      //  norm of vector of residuals
      scprd (res,res,norres);
      
      fprintf (stderr,"\n norm of res  %e",norres);
      
      if (norres<err)  break;
      
      //  compliance matrix of the material
      matcompl (cm,ipp,ncomp);
      
      //  matrix B
      auxmat (b,cm,rs,rq,hs,hq,dgamma);
      auxrhs (rhs,res,r,h);

      //  solution of auxiliary system of equations
      gemp (b.a,lhs.a,rhs.a,ncomp+nhp,1,Mp->zero,1);
      
      //  increment of gamma
      ddgamma = increments (epsa,ddqtr,lhs,fs,fq,dg,ddt);
      

      matcompl (cm,ipp,ncomp);
      mxv (cm,epsa,ddepsptr);

      addv (epsptr,ddepsptr,epsptr);
      addv (qtr,ddqtr,qtr);
    }
  }

  subv (epsn,epsptr,epsa);
  elmatstiff (sm,ipp);
  mxv (sm,epsa,sig);
  storestress (ipp,sig);
}

*/

/**
  Function returns radius of nonlocal material model.
   
  @param ipp - integration point pointer
  @param im - material index
  
  @retval Function returns real number representing radius for nonlocal model.
*/
double mechmat::nonlocradius (long ipp,long im)
{
  double r;
  
  switch (ip[ipp].tm[im]){
  case nonlocplastmat :
  {
    r = nlplast[ip[ipp].idm[im]].r;
    break;
  }
  case nonlocdamgmat :
  {
    r = nldamg[ip[ipp].idm[im]].r;
    break;
  }
  case nonlocalmicroM4 :
  {
    r = nmpM4[ip[ipp].idm[im]].r;
    break;
  }
  default:{
    print_err("wrong material type is required",__FILE__,__LINE__,__func__);
  }
  }
  
  return r;
}

/**
   Function computes %matrix of thermal dilatancy of the material
   in the required integration point.

   @param d - %matrix of thermal dilatancy
   @param ipp - number of integration point

   JK, 29.11.2002
*/
void mechmat::matdilat (matrix &d,long ipp)
{
  long i, iddm = ip[ipp].nm-1;

  switch (ip[ipp].tm[iddm]){
  case therisodilat:{
    i=ip[ipp].idm[iddm];
    tidilat[i].matdilat (d,ip[ipp].ssst);
    break;
  }
  case therisodilattime:{
    i=ip[ipp].idm[iddm];
    tidilattime[i].matdilat (d,ip[ipp].ssst);
    break;
  }
  default:{
    print_err("unknown material type is required",__FILE__,__LINE__,__func__);
  }
  }

}

/**
   Function updates values at integration points,
   it is used for nonlinear computations.
   
*/
void mechmat::updateipval (void)
{
  long i,j,ipp,nip;
  
  //tady pridano 17.7.2006 TKr
  //updates only used elements(int. points)
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      ipp=Mt->elements[i].ipp[0][0];
      nip=Mt->give_tnip (i);
      for (j=0;j<nip;j++){
	updateipvalmat (ipp,0,0);
	ipp++;
      }
    }
  }
}


/**
   Function updates values at integration points,
   it is used for nonlinear computations.
   
   @param ipp - number of integration point
   @param im - index of material type
   @param ido - index in array other
   
*/
void mechmat::updateipvalmat (long ipp,long im,long ido)
{
  switch (ip[ipp].tm[im]){
  case elisomat :
    break;
  case simplas1d:{
    spl1d[ip[ipp].idm[im]].updateval (ipp,ido);
    break;
  }
  case jflow:{
    j2f[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case jflow2:{
    j2f2[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case mohrcoul:{
    mcoul[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case mohrcoulparab:{
    mcpar[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case boermaterial:{
    boerm[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case druckerprager:{
    drprm[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case chenplast:{
    chplast[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case modcamclaymat:{
    cclay[ip[ipp].idm[im]].updateval (ipp,ido);
    break;
  }
  case microplaneM4:{
    mpM4[ip[ipp].idm[im]].updateval (ipp,im,ido);
  }
  case microsimp:{
    mpSIM[ip[ipp].idm[im]].updateval (ipp,im,ido);
  }
  case microfibro:{
    mpfib[ip[ipp].idm[im]].updateval (ipp,im,ido);
  }
  case shefpl:{
    shpl[ip[ipp].idm[im]].updateval (ipp,ido);
    break;
  }
  case glasgowdamage:{
    glasgdam[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case creep_damage:{
    crdam[ip[ipp].idm[im]].updateval (ipp, im, ido);
    break;
  }
  case time_switchmat:{
    tswmat[ip[ipp].idm[im]].updateval (ipp, im, ido);
    break;
  }
  case graphm:{
    break;
  }
  case graphmtime:{
    break;
  }
  case geoelast:{
    geoel[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case scaldamage:{
    scdam[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case scaldamagecc:{
    scdamcc[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }    
  case ortodamage:{
    ortdam[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case ortodamagerot:{
    ortdamrot[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case anisodamage:{
    anidam[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }
  case anisodamagerot:{
    anidamrot[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  }

  case nonlocdamgmat:{
    updateipvalmat (ipp,im+1,ido);
    break;
  }
  case nonlocplastmat:{
    updateipvalmat (ipp,im+1,ido);
    break;
  }
  case nonlocalmicroM4:{
    updateipvalmat (ipp,im+1,ido);
    break;
  }
    
  case damage_plasticity:{
    dampl[ip[ipp].idm[im]].updateval (ipp);
    break;
  }
  case simvisc:
  case simvisplas:
  case lemaitr:
  case viscoplasticity:
    break;
  case creeprs:
  case creepb3:
  case creepdpl:{
    creep_updateval (ipp,im,ido);
    break;
  }
  case creepbaz:
    crbaz[ip[ipp].idm[im]].updateval ();
    break;
  case creepeffym:
    creffym[ip[ipp].idm[im]].updateval (ipp,im,ido);
    break;
  case elasttimemat:
    break;
  case scaldamage_time:
    break;
  case lenjonespot:
    break;
    
  case contmat:
    break;
    
  case consolidation:{
    csol[ip[ipp].idm[im]].updateval (ipp,im,ido);
  }
  default:{
    print_err ("unknown material type is required",__FILE__,__LINE__,__func__);
    break;
  }
  }
}



/**
  Function initializes values of internal material variables
  at integration point ipp for given material im.
   
  @param ipp - integration point pointer
  @param im  - index of material type for given ip
  @param ido - index of internal variables for given material in the ipp eqother array
   
*/
void mechmat::initvalues (long ipp,long im,long ido)
{
  switch (ip[ipp].tm[im])
  {
    case elisomat:
    case simplas1d:
    case jflow:
    case jflow2:
    case microplaneM4:
    case nonlocalmicroM4:
    case microsimp:
    case microfibro:
    case mohrcoul:
    case mohrcoulparab:
    case boermaterial:
    case druckerprager:
    case modcamclaymat:
    case shefpl:
    case chenplast:
    case glasgowmechmat:
    case glasgowdamage:
      break;
    case creep_damage:
      crdam[ip[ipp].idm[im]].initvalues(ipp, im, ido);
      break;
    case time_switchmat:
      tswmat[ip[ipp].idm[im]].initvalues(ipp, im, ido);
      break;
    case simvisc:
    case simvisplas:
    case lemaitr:
    case scaldamage:
    case scaldamagecc:
      break;
    case scaldamage_time:
      scdamtime[ip[ipp].idm[im]].initvalues(ipp, im, ido);
    case aci:
    case cebfip:
    case graphm:
    case graphmtime:
    case geoelast:
      break;
    case ortodamage:
      ortdam[ip[ipp].idm[im]].initvalues(ipp, ido);
      break;
    case ortodamagerot:
      ortdamrot[ip[ipp].idm[im]].initvalues(ipp, ido);
      break;
    case anisodamage:
      anidam[ip[ipp].idm[im]].initvalues(ipp, ido);
      break;
    case anisodamagerot:
      anidamrot[ip[ipp].idm[im]].initvalues(ipp, ido);
      break;
    case creeprs:
    case creepb3:
    case creepdpl:{
      creep_initmaterialmodel(ipp,im,ido);
    }
    case creepbaz:
    case consolidation:
    case winklerpasternak:
    case therisodilat:
    case nonlocplastmat:
    case nonlocdamgmat:
    case damage_plasticity:
    case viscoplasticity:
    case elasttimemat:
    case creepeffym:
    case rcmatmodnorm:
      break;
    case therisodilattime:
      tidilattime[ip[ipp].idm[im]].initvalues(ipp, ido);
      break;
    default:
      print_err("unknown material type is required", __FILE__,__LINE__,__func__);
  }
}



/**
  Function returns consistency parameter for models of plasticity.

  @param ipp - integration point number in the mechmat ip array.
  @param im - material index
  @param ido - index of internal variables for given material in the ipp other array

*/
double mechmat::give_consparam (long ipp, long im, long ido)
{
  long i;
  double cp;
  
  i = ip[ipp].idm[im];
  
  switch (ip[ipp].tm[im]){
  case simplas1d:{
    cp = spl1d[i].give_consparam (ipp,ido);
    break;
  }
  case jflow:{
    cp = j2f[i].give_consparam (ipp,ido);
    break;
  }
  case mohrcoul:{
    cp = mcoul[i].give_consparam (ipp,ido);
    break;
  }
  case mohrcoulparab:{
    cp = mcpar[i].give_consparam (ipp,ido);
    break;
  }
  case boermaterial:{
    cp = boerm[i].give_consparam (ipp,ido);
    break;
  }
  case druckerprager:{
    cp = drprm[i].give_consparam (ipp,ido);
    break;
  }
  case modcamclaymat:{
    cp = cclay[i].give_consparam (ipp,ido);
    break;
  }
  default:{
    print_err("unknown material type is required",__FILE__,__LINE__,__func__);
  }
  }
  
  return cp;
}



/**
  Function stores volume belonging to the given integration point.

  @param ipp - integration point number in the mechmat ip array.
  @param vol - volume belonging to the integration point  
  
*/
void mechmat::refresh_plast (long im, long ido)
{
  if (plast == 0)
    for (long i=0;i<tnip;i++)
      if ( give_consparam(i,im,ido) ){
	plast = 1;
	return;
      }
}



/**
  Function stores volume belonging to the given integration point.

  @param ipp - integration point number in the mechmat ip array.
  @param vol - volume belonging to the integration point  
  
*/
void mechmat::storeipvol (long ipp,double vol)
{
  ipv[ipp]=vol;
}

/**
   Function allocates array for eigenstrains.
   
   3.3.2004, JK
*/
void mechmat::alloceigstrains ()
{
  long i,j,nc;
  
  if (eigstrains==NULL){
    eigstrains = new double* [tnip];
    for (i=0;i<tnip;i++){
      nc=ip[i].ncompstr;
      eigstrains[i] = new double [nc];
      for (j=0;j<nc;j++){
	eigstrains[i][j]=0.0;
      }
    }
  }
  
}

/**
   Function allocates array for eigenstresses.
   
   JK, 27.11.2006
*/
void mechmat::alloceigstresses ()
{
  long i,j,nc;
  
  if (eigstresses==NULL){
    eigstresses = new double* [tnip];
    for (i=0;i<tnip;i++){
      nc=ip[i].ncompstr;
      eigstresses[i] = new double [nc];
      for (j=0;j<nc;j++){
	eigstresses[i][j]=0.0;
      }
    }
  }
  
}

/**
   Function allocates array for eigenstrains.
   
   3.3.2004, JK
*/
void mechmat::alloctempstrains ()
{
  long i,j,nc;
  
  if (tempstrains==NULL){
    tempstrains = new double* [tnip];
    for (i=0;i<tnip;i++){
      nc=ip[i].ncompstr;
      tempstrains[i] = new double [nc];
      for (j=0;j<nc;j++){
	tempstrains[i][j]=0.0;
      }
    }
  }
  
}



/**
   Function computes strains without eigenstrains.
   
   3.3.2004, JK
*/
void mechmat::eigstrmod ()
{
  long i,j,k,nc,ipp,nip;
  
  if (eigstrains != NULL){
    for (i=0;i<Mt->ne;i++){
      if (Gtm->leso[i]==1){
	//  first integration point on element
	ipp=Mt->elements[i].ipp[0][0];
	//  total number of integration points on element
	nip = Mt->give_tnip (i);
	for (j=0;j<nip;j++){
	  //  number of components in strain array
	  nc=ip[ipp].ncompstr;
	  for (k=0;k<nc;k++)
	    ip[ipp].strain[k]-=eigstrains[ipp][k];
	  ipp++;
	}
      }
    }
  }
  if (tempstrains != NULL){
    for (i=0;i<Mt->ne;i++){
      if (Gtm->leso[i]==1){
	//  first integration point on element
	ipp=Mt->elements[i].ipp[0][0];
	//  total number of integration points on element
	nip = Mt->give_tnip (i);
	for (j=0;j<nip;j++){
	  //  number of components in strain array
	  nc=ip[ipp].ncompstr;
	  for (k=0;k<nc;k++)
	    ip[ipp].strain[k]-=tempstrains[ipp][k];
	  ipp++;
	}
      }
    }
  }
}



/**
   Function computes the total strains at all integration points.
   
   3.3.2004, JK
*/
void mechmat::totstrains ()
{
  long i,j,k,nc,ipp,nip;
  
  if (eigstrains != NULL){
    for (i=0;i<Mt->ne;i++){
      if (Gtm->leso[i]==1){
	//  first integration point on element
	ipp=Mt->elements[i].ipp[0][0];
	//  total number of integration points on element
	nip = Mt->give_tnip (i);
	for (j=0;j<nip;j++){
	  //  number of components in strain array
	  nc=ip[ipp].ncompstr;
	  for (k=0;k<nc;k++)
	    ip[ipp].strain[k]+=eigstrains[ipp][k];
	  ipp++;
	}
      }
    }
  }
  if (tempstrains != NULL){
    for (i=0;i<Mt->ne;i++){
      if (Gtm->leso[i]==1){
	//  first integration point on element
	ipp=Mt->elements[i].ipp[0][0];
	//  total number of integration points on element
	nip = Mt->give_tnip (i);
	for (j=0;j<nip;j++){
	  //  number of components in strain array
	  nc=ip[ipp].ncompstr;
	  for (k=0;k<nc;k++)
	    ip[ipp].strain[k]+=tempstrains[ipp][k];
	  ipp++;
	}
      }
    }
  }
}



/**
   Function cleans all arrays defined at integration points.
   
   JK, 25.9.2004
*/
void mechmat::clean_ip ()
{
  long i;
  for (i=0;i<tnip;i++){
    ip[i].clean (Mb->nlc);
  }
}



/**
   Function saves data from integration points into auxiliary file
   in text format.
   
   @param aux - pointer to auxiliary file
   @param selelems - selection of elements whose eqother array will be saved
   @param selother - selection of components of saved eqother array which will be saved on selected elements
                     for each range or list item in selelems an individual selection of eqother components
                     is enabled
   
   JK, 19.9.2004
   TKo, 9.2008
*/
void mechmat::save_intpoints_txt (FILE *aux, sel &selelems, sel *selother)
{
  long i,j,n,nip,ir,ipp;
  sel selno; // selection with no selected components
  int prec = (int)Mp->hdbcont.prec;
  
  selno.n=1;
  fprintf(aux, "\n");
  for (i=0;i<Mt->ne;i++)
  {    
    ipp = Mt->elements[i].ipp[0][0];
    nip = Mt->give_totnip(i);
    selelems.presence_id(i, ir);
    if (ir < 0)
      n = 0;    
    else
      n = selother[ir].give_nselcomp(ip[ipp].ncompeqother);
    for (j=0; j<nip; j++)
    {
      fprintf (aux,"%ld %ld %ld %ld\n",ipp, Mb->nlc, ip[ipp].ncompstr, n);
      if (ir < 0)
        ip[ipp].save_data_txt(aux,Mb->nlc, selno);
      else
        ip[ipp].save_data_txt(aux,Mb->nlc, selother[ir]);

      // writing TRFEL quantities
      if (inittempr)        
        fprintf (aux,"1 %.*le ",prec, inittempr[ipp]);
      else
        fprintf (aux,"0 ");
      if (tempr)        
        fprintf (aux,"1 %.*le ",prec, tempr[ipp]);
      else
        fprintf (aux,"0 ");
      if (moist)        
        fprintf (aux,"1 %.*le",prec, moist[ipp]);
      else
        fprintf (aux,"0");
      fprintf(aux, "\n");
      ipp++;
    }
  }
}



/**
   Function saves data from integration points into several files
   in text format.
   
   @param ni       - time step id
   @param selelems - selection of elements whose eqother array will be saved
   @param selother - selection of components of saved eqother array which will be saved on selected elements
                     for each range or list item in selelems an individual selection of eqother components
                     is enabled
   
   JK, 19.9.2004
   TKo, 9.2008
*/
void mechmat::save_intpoints_txt (long ni, sel &selelems, sel *selother)
{
  long i,j,k,ir,n,nip,ipp;
  sel selno; // selection with no selected components
  selno.n=1;
  FILE *aux;
  char name[FNAMELEN+20];
  char emsg[FNAMELEN+100];
  int prec = (int)Mp->hdbcont.prec;

  sprintf(name, "%s.%ld.strain.bac",Mp->hdbcont.hdbnames, ni);
  aux = fopen(name,"wt");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++)
  {
    fprintf (aux,"%ld %ld %ld\n",i, Mb->nlc, ip[i].ncompstr);
    for (j=0;j<Mb->nlc*ip[i].ncompstr;j++)
      fprintf (aux, "%.*le\n",prec , ip[i].strain[j]);
  }
  fclose(aux);

  sprintf(name, "%s.%ld.stress.bac",Mp->hdbcont.hdbnames, ni);
  aux = fopen(name,"wt");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++)
  {
    fprintf (aux,"%ld %ld %ld\n",i, Mb->nlc, ip[i].ncompstr);
    for (j=0;j<Mb->nlc*ip[i].ncompstr;j++)
      fprintf (aux, "%.*le\n",prec , ip[i].stress[j]);
  }
  fclose(aux);

  sprintf(name, "%s.%ld.other.bac",Mp->hdbcont.hdbnames, ni);
  aux = fopen(name,"wt");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<Mt->ne;i++)
  {    
    ipp = Mt->elements[i].ipp[0][0];
    nip = Mt->give_totnip(i);
    selelems.presence_id(i, ir);    
    if (ir < 0)
      n = 0;
    else
      n = selother[ir].give_nselcomp(ip[ipp].ncompeqother);
    for (j=0; j<nip; j++)
    {
      fprintf (aux,"%ld %ld\n",ipp, n);
      if (n==0) 
      {
        ipp++;
        continue;
      }
      for (k=0;k<ip[ipp].ncompeqother;k++)
      {
        if (selother[ir].presence_id(k))
          fprintf (aux, "%.*le\n",prec , ip[ipp].eqother[k]);
      }
      ipp++;
    }
  }
  fclose(aux);
 
 // writing TRFEL quantities
  if (tempr || inittempr || moist)
  {
    sprintf(name, "%s.%ld.trfquant.bac",Mp->hdbcont.hdbnames, ni);
    aux = fopen(name,"wt");
    if (aux==NULL)
    {
      sprintf(emsg, "cannot open backup file %s", name);
      print_err(emsg, __FILE__, __LINE__, __func__);
      abort();
    }
    for (i=0;i<tnip;i++)
    {
      fprintf (aux,"%ld ",i);
      if (inittempr)        
        fprintf (aux,"1 %.*le ",prec , inittempr[i]);
      else
        fprintf (aux,"0 ");
      if (tempr)        
        fprintf (aux,"1 %.*le ",prec , tempr[i]);
      else
        fprintf (aux,"0 ");
      if (moist)        
        fprintf (aux,"1 %.*le",prec , moist[i]);
      else
        fprintf (aux,"0");
      fprintf(aux, "\n");
    }
    fclose(aux);
  }
}



/**
   Function saves data from integration points into auxiliary file
   in binary format.
   
   @param aux - pointer to auxiliary file
   @param selelems - selection of elements whose eqother array will be saved
   @param selother - selection of components of saved eqother array which will be saved on selected elements
                     for each range or list item in selelems an individual selection of eqother components
                     is enabled
   
   TKo, 9.2008
*/
void mechmat::save_intpoints_bin (FILE *aux, sel &selelems, sel *selother)
{
  long i,j,n,nip,ir,ipp;
  sel selno; // selection with no selected components
  char one = 1;
  char zero = 0;
  
  selno.n=1;
  for (i=0;i<Mt->ne;i++)
  {    
    ipp = Mt->elements[i].ipp[0][0];
    nip = Mt->give_totnip(i);
    selelems.presence_id(i, ir);
    
    if (ir < 0)
      n = 0;
    else
      n = selother[ir].give_nselcomp(ip[ipp].ncompeqother);
    for (j=0; j<nip; j++)
    {
      fwrite(&ipp, sizeof(ipp), 1, aux);
      fwrite(&Mb->nlc, sizeof(Mb->nlc), 1, aux);
      fwrite(&ip[ipp].ncompstr, sizeof(ip[ipp].ncompstr), 1, aux);
      fwrite(&n, sizeof(n), 1, aux);
      if (ir < 0)
        ip[ipp].save_data_bin(aux,Mb->nlc, selno);
      else
        ip[ipp].save_data_bin(aux,Mb->nlc, selother[ir]);

      // writing TRFEL quantities
      if (inittempr)        
      {
        fwrite (&one, sizeof(one), 1, aux);
        fwrite (&inittempr[ipp], sizeof(inittempr[ipp]), 1, aux);
      }
      else
        fwrite (&zero, sizeof(zero), 1, aux);
      if (tempr)        
      {
        fwrite (&one, sizeof(one), 1, aux);
        fwrite (&tempr[ipp], sizeof(tempr[ipp]), 1, aux);
      }
      else
        fwrite (&zero, sizeof(zero), 1, aux);
      if (moist)        
      {
        fwrite (&one, sizeof(one), 1, aux);
        fwrite (&moist[ipp], sizeof(moist[ipp]), 1, aux);
      }
      else
        fwrite (&zero, sizeof(zero), 1, aux);
      ipp++;
    }
  }
}



/**
   Function saves data from integration points into several files
   in binary format.
   
   @param ni       - time step id
   @param selelems - selection of elements whose eqother array will be saved
   @param selother - selection of components of saved eqother array which will be saved on selected elements
                     for each range or list item in selelems an individual selection of eqother components
                     is enabled
   
   TKo 9.2008
*/
void mechmat::save_intpoints_bin (long ni, sel &selelems, sel *selother)
{
  long i,j,k,ir,n,nip,ipp;
  sel selno; // selection with no selected components
  selno.n=1;
  FILE *aux;
  char name[FNAMELEN+20];
  char emsg[FNAMELEN+100];
  char one = 1;
  char zero = 0;

  sprintf(name, "%s.%ld.strain.bac",Mp->hdbcont.hdbnames, ni);
  aux = fopen(name,"wb");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++){
    fwrite(&i, sizeof(i), 1, aux);
    fwrite(&Mb->nlc, sizeof(Mb->nlc), 1, aux);
    fwrite(&ip[i].ncompstr, sizeof(ip[i].ncompstr), 1, aux);
    fwrite (ip[i].strain, sizeof(*ip[i].strain),Mb->nlc*ip[i].ncompstr, aux);
  }
  fclose(aux);

  sprintf(name, "%s.%ld.stress.bac",Mp->hdbcont.hdbnames, ni);
  aux = fopen(name,"wb");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++)
  {
    fwrite(&i, sizeof(i), 1, aux);
    fwrite(&Mb->nlc, sizeof(Mb->nlc), 1, aux);
    fwrite(&ip[i].ncompstr, sizeof(ip[i].ncompstr), 1, aux);
    fwrite(ip[i].stress, sizeof(*ip[i].stress),Mb->nlc*ip[i].ncompstr, aux);
  }
  fclose(aux);

  sprintf(name, "%s.%ld.other.bac",Mp->hdbcont.hdbnames, ni);
  aux = fopen(name,"wb");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<Mt->ne;i++)
  {    
    ipp = Mt->elements[i].ipp[0][0];
    nip = Mt->give_totnip(i);
    selelems.presence_id(i, ir);    
    if (ir < 0)
      n = 0;
    else
      n = selother[ir].give_nselcomp(ip[ipp].ncompeqother);
    for (j=0; j<nip; j++)
    {
      fwrite(&ipp, sizeof(ipp), 1, aux);
      fwrite(&n, sizeof(n), 1, aux);
      if (n==0) 
      {
        ipp++;
        continue;
      }
      for (k=0;k<ip[ipp].ncompeqother;k++)
      {
        if (selother[ir].presence_id(k))
          fwrite (ip[ipp].eqother+k, sizeof(*ip[ipp].eqother), 1, aux);
      }
      ipp++;
    }
  }
  fclose(aux);

  // writing TRFEL quantities
  sprintf(name, "%s.%ld.trfquant.bac",Mp->hdbcont.hdbnames, ni);
  aux = fopen(name,"wb");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (ipp=0;ipp<tnip;ipp++)
  {
    fwrite(&ipp, sizeof(ipp), 1, aux);
    if (inittempr)        
    {
      fwrite (&one, sizeof(one), 1, aux);
      fwrite (&inittempr[ipp], sizeof(inittempr[ipp]), 1, aux);
    }
    else
      fwrite (&zero, sizeof(zero), 1, aux);
    if (tempr)        
    {
      fwrite (&one, sizeof(one), 1, aux);
      fwrite (&tempr[ipp], sizeof(tempr[ipp]), 1, aux);
    }
    else
      fwrite (&zero, sizeof(zero), 1, aux);
    if (moist)        
    {
      fwrite (&one, sizeof(one), 1, aux);
      fwrite (&moist[ipp], sizeof(moist[ipp]), 1, aux);
    }
    else
      fwrite (&zero, sizeof(zero), 1, aux);
  }
  fclose(aux);
}



/**
   Function restores data from auxiliary text file into integration points.
   
   @param aux - pointer to auxiliary file
   @param selelems - selection of elements whose eqother array will be restored
   @param selother - selection of components of saved eqother array which will be resored on selected elements
                     for each range or list item in selelems an individual selection of eqother components
                     is enabled
   @param selid    - array of indeces of positions in eqother array to which will be 
                     restored selected saved eqother components
                     for each range or list item in selother an individual selection of position 
                     of eqother components is enabled for each selection of elements
   
   JK, 19.9.2004
   TKo 9.2008
*/
void mechmat::restore_intpoints_txt (FILE *aux, sel &selelems, sel *selother, long **selid)
{
  long i,n,ir,ipp;
  long nlc, ncompstr;
  sel selno; // selection with no selected components
  long flag;
  double tmp;
  
  selno.n=1;
  for (i=0;i<tnip;i++){
    fscanf (aux,"%ld %ld %ld %ld",&ipp, &nlc, &ncompstr, &n);
    if ((ipp < 0) || (ipp >= tnip))
    {
      print_err("invalid integration point number", __FILE__, __LINE__, __func__);
      abort();
    }
    if (nlc != Mb->nlc)
    {
      print_err("invalid number of load cases", __FILE__, __LINE__, __func__);
      abort();
    }
    if (ncompstr != ip[ipp].ncompstr)
    {
      print_err("incompatible number of stress/strain components", __FILE__, __LINE__, __func__);
      abort();
    }
    selelems.presence_id(elip[ipp], ir);
    if (ir < 0)
      ip[ipp].restore_data_txt(aux, nlc, n, selno, NULL);
    else
      ip[ipp].restore_data_txt(aux, nlc, n, selother[ir], selid[ir]);

    // restoring TRFEL quantities
    fscanf (aux,"%ld",&flag);
    if (flag)
    {        
      fscanf(aux,"%le", &tmp);
      if (inittempr)
        inittempr[ipp] = tmp;
    }
    fscanf (aux,"%ld",&flag);
    if (flag)
    {        
      fscanf(aux,"%le", &tmp);
      if (tempr)
        tempr[ipp] = tmp;
    }
    fscanf (aux,"%ld",&flag);
    if (flag)
    {        
      fscanf(aux,"%le", &tmp);
      if (moist)
        moist[ipp] = tmp;
    }
  }
}



/**
   Function restores data from several text files into integration points.
   
   @param selelems - selection of elements whose eqother array will be restored
   @param selother - selection of components of saved eqother array which will be resored on selected elements
                     for each range or list item in selelems an individual selection of eqother components
                     is enabled
   @param selid    - array of indeces of positions in eqother array to which will be 
                     restored selected saved eqother components
                     for each range or list item in selother an individual selection of position 
                     of eqother components is enabled for each selection of elements
   
   JK, 19.9.2004
   TKo 9.2008
*/
void mechmat::restore_intpoints_txt (sel &selelems, sel *selother, long **selid)
{
  long i,j,k,n,ir,ik,is,ipp;
  long nlc, ncompstr;
  double tmp;
  long flag;
  FILE *aux;
  char name[FNAMELEN+20];
  char emsg[FNAMELEN+100];

  // restoring of strain arrays
  sprintf(name, "%s.strain.bac",Mp->hdbcont.hdbnamer);
  aux = fopen(name,"rt");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++){
    fscanf (aux,"%ld %ld %ld",&ipp, &nlc, &ncompstr);
    if ((ipp < 0) || (ipp >= tnip))
    {
      print_err("invalid integration point number", __FILE__, __LINE__, __func__);
      abort();
    }
    if (nlc != Mb->nlc)
    {
      print_err("invalid number of load cases", __FILE__, __LINE__, __func__);
      abort();
    }
    if (ncompstr != ip[ipp].ncompstr)
    {
      print_err("incompatible number of stress/strain components", __FILE__, __LINE__, __func__);
      abort();
    }
    for (j=0; j<Mb->nlc*ip[i].ncompstr; j++)
      fscanf (aux, "%le", ip[i].strain+j);
  }
  // restoring of stress arrays
  sprintf(name, "%s.stress.bac",Mp->hdbcont.hdbnamer);
  aux = fopen(name,"rt");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++){
    fscanf (aux,"%ld %ld %ld",&ipp, &nlc, &ncompstr);
    if ((ipp < 0) || (ipp >= tnip))
    {
      print_err("invalid integration point number", __FILE__, __LINE__, __func__);
      abort();
    }
    if (nlc != Mb->nlc)
    {
      print_err("invalid number of load cases", __FILE__, __LINE__, __func__);
      abort();
    }
    if (ncompstr != ip[ipp].ncompstr)
    {
      print_err("incompatible number of stress/strain components", __FILE__, __LINE__, __func__);
      abort();
    }
    for (j=0;j<nlc*ncompstr;j++)
      fscanf (aux, "%le", ip[i].stress+j);
  }
  // restoring of other arrays
  sprintf(name, "%s.other.bac",Mp->hdbcont.hdbnamer);
  aux = fopen(name,"rt");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++)
  {
    fscanf (aux,"%ld %ld",&ipp, &n);
    if ((ipp < 0) || (ipp >= tnip))
    {
      print_err("invalid integration point number", __FILE__, __LINE__, __func__);
      abort();
    }
    selelems.presence_id(elip[ipp], ir);    
    for (k=0;k<n;k++)
    {
      fscanf (aux, "%le", &tmp);
      if (ir < 0)
        continue;
      if (selother[ir].presence_id(k,ik))
      {
        is = selid[ir][ik]+k-selother[ir].id1[ik]; // storage index in eqother array 
        if (is >= ip[ipp].ncompeqother)
          print_err("invalid index for eqother restoring is required", __FILE__, __LINE__, __func__);
        else
          ip[ipp].eqother[is] = tmp;
      }
    }
    ipp++;
  }
  fclose(aux);

  // restoring TRFEL quantities
  sprintf(name, "%s.trfquant.bac",Mp->hdbcont.hdbnamer);
  aux = fopen(name,"rt");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++)
  {
    fscanf (aux, "%ld", &ipp);
    if ((ipp < 0) || (ipp >= tnip))
    {
      print_err("invalid integration point number", __FILE__, __LINE__, __func__);
      abort();
    }
    fscanf (aux, "%ld", &flag);
    if (flag)
    {        
      fscanf(aux, "%le", &tmp);
      if (inittempr)
        inittempr[ipp] = tmp;
    }
    fscanf (aux, "%ld", &flag);
    if (flag)
    {        
      fscanf(aux, "%le", &tmp);
      if (tempr)
        tempr[ipp] = tmp;
    }
    fscanf (aux, "%ld", &flag);
    if (flag)
    {        
      fscanf(aux, "%le", &tmp);
      if (moist)
        moist[ipp] = tmp;
    }
  }
  fclose(aux);
}



/**
   Function restores data from auxiliary binary file into integration points.
   
   @param aux - pointer to auxiliary file
   @param selelems - selection of elements whose eqother array will be restored
   @param selother - selection of components of saved eqother array which will be resored on selected elements
                     for each range or list item in selelems an individual selection of eqother components
                     is enabled
   @param selid    - array of indeces of positions in eqother array to which will be 
                     restored selected saved eqother components
                     for each range or list item in selother an individual selection of position 
                     of eqother components is enabled for each selection of elements
   
   JK, 19.9.2004
   TKo 9.2008
*/
void mechmat::restore_intpoints_bin (FILE *aux, sel &selelems, sel *selother, long **selid)
{
  long i, ipp, nlc, ncompstr, n;
  long ir;
  char flag;
  double tmp;
  sel selno; // selection with no selected components
  
  for (i=0;i<tnip;i++){
    fread(&ipp, sizeof(ipp), 1, aux);
    fread(&nlc, sizeof(nlc), 1, aux);
    fread(&ncompstr, sizeof(ncompstr), 1, aux);
    fread(&n, sizeof(n), 1, aux);
    if ((ipp < 0) || (ipp >= tnip))
    {
      print_err("invalid integration point number", __FILE__, __LINE__, __func__);
      abort();
    }
    if (nlc != Mb->nlc)
    {
      print_err("invalid number of load cases", __FILE__, __LINE__, __func__);
      abort();
    }
    if (ncompstr != ip[ipp].ncompstr)
    {
      print_err("incompatible number of stress/strain components", __FILE__, __LINE__, __func__);
      abort();
    }
    selelems.presence_id(elip[ipp], ir);
    if (ir < 0)
      ip[ipp].restore_data_bin(aux, Mb->nlc, n, selno, NULL);
    else
      ip[ipp].restore_data_bin(aux, Mb->nlc, n, selother[ir], selid[ir]);

    // restoring TRFEL quantities
    fread (&flag, sizeof(flag), 1, aux);
    if (flag)
    {        
      fread(&tmp, sizeof(tmp), 1, aux);
      if (inittempr)
        inittempr[ipp] = tmp;
    }
    fread (&flag, sizeof(flag), 1, aux);
    if (flag)
    {        
      fread(&tmp, sizeof(tmp), 1, aux);
      if (tempr)
        tempr[ipp] = tmp;
    }
    fread (&flag, sizeof(flag), 1, aux);
    if (flag)
    {        
      fread(&tmp, sizeof(tmp), 1, aux);
      if (moist)
        moist[ipp] = tmp;
    }
  }
}



/**
   Function restores data from several binary files into integration points.
   
   @param selelems - selection of elements whose eqother array will be restored
   @param selother - selection of components of saved eqother array which will be resored on selected elements
                     for each range or list item in selelems an individual selection of eqother components
                     is enabled
   @param selid    - array of indeces of positions in eqother array to which will be 
                     restored selected saved eqother components
                     for each range or list item in selother an individual selection of position 
                     of eqother components is enabled
   
   JK, 19.9.2004
   TKo 9.2008
*/
void mechmat::restore_intpoints_bin (sel &selelems, sel *selother, long **selid)
{
  long i,k,n,ir,ik,is,ipp;
  long nlc, ncompstr;
  double tmp;
  char flag;
  FILE *aux;
  char name[FNAMELEN+20];
  char emsg[FNAMELEN+100];

  // restoring of strain arrays
  sprintf(name, "%s.strain.bac",Mp->hdbcont.hdbnamer);
  aux = fopen(name,"rb");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++){
    fread(&ipp, sizeof(ipp), 1, aux);
    fread(&nlc, sizeof(nlc), 1, aux);
    fread(&ncompstr, sizeof(ncompstr), 1, aux);
    if ((ipp < 0) || (ipp >= tnip))
    {
      print_err("invalid integration point number", __FILE__, __LINE__, __func__);
      abort();
    }
    if (nlc != Mb->nlc)
    {
      print_err("invalid number of load cases", __FILE__, __LINE__, __func__);
      abort();
    }
    if (ncompstr != ip[ipp].ncompstr)
    {
      print_err("incompatible number of stress/strain components", __FILE__, __LINE__, __func__);
      abort();
    }
    fread (ip[ipp].strain, sizeof(*ip[ipp].strain),Mb->nlc*ip[ipp].ncompstr, aux);
  }
  // restoring of stress arrays
  sprintf(name, "%s.stress.bac",Mp->hdbcont.hdbnamer);
  aux = fopen(name,"rb");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++){
    fread(&ipp, sizeof(ipp), 1, aux);
    fread(&nlc, sizeof(nlc), 1, aux);
    fread(&ncompstr, sizeof(ncompstr), 1, aux);
    if ((ipp < 0) || (ipp >= tnip))
    {
      print_err("invalid integration point number", __FILE__, __LINE__, __func__);
      abort();
    }
    if (nlc != Mb->nlc)
    {
      print_err("invalid number of load cases", __FILE__, __LINE__, __func__);
      abort();
    }
    if (ncompstr != ip[ipp].ncompstr)
    {
      print_err("incompatible number of stress/strain components", __FILE__, __LINE__, __func__);
      abort();
    }
    fread (ip[ipp].stress, sizeof(*ip[ipp].stress),Mb->nlc*ip[ipp].ncompstr, aux);
  }
  // restoring of other arrays
  sprintf(name, "%s.other.bac",Mp->hdbcont.hdbnamer);
  aux = fopen(name,"rb");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++){
    fread (&ipp, sizeof(ipp), 1, aux);
    fread (&n, sizeof(n), 1, aux);
    if ((ipp < 0) || (ipp >= tnip))
    {
      print_err("invalid integration point number", __FILE__, __LINE__, __func__);
      abort();
    }
    selelems.presence_id(ipp, ir);    
    for (k=0;k<n;k++)
    {
      fread (&tmp, sizeof(tmp), 1, aux);
      if (ir < 0)
        continue;
      if (selother[ir].presence_id(k,ik))
      {
        is = selid[ir][ik]+k-selother[ir].id1[ik]; // storage index in eqother array 
        if (is >= ip[ipp].ncompeqother)
          print_err("invalid index for eqother restoring is required", __FILE__, __LINE__, __func__);
        else
          ip[ipp].eqother[is] = tmp;
      }
    }   
  }
  fclose(aux);

  // restoring TRFEL quantities
  sprintf(name, "%s.trfquant.bac",Mp->hdbcont.hdbnamer);
  aux = fopen(name,"rb");
  if (aux==NULL)
  {
    sprintf(emsg, "cannot open backup file %s", name);
    print_err(emsg, __FILE__, __LINE__, __func__);
    abort();
  }
  for (i=0;i<tnip;i++)
  {
    fread (&ipp, sizeof(ipp), 1, aux);
    if ((ipp < 0) || (ipp >= tnip))
    {
      print_err("invalid integration point number", __FILE__, __LINE__, __func__);
      abort();
    }
    fread (&flag, sizeof(flag), 1, aux);
    if (flag)
    {        
      fread(&tmp, sizeof(tmp), 1, aux);
      if (inittempr)
        inittempr[ipp] = tmp;
    }
    fread (&flag, sizeof(flag), 1, aux);
    if (flag)
    {        
      fread(&tmp, sizeof(tmp), 1, aux);
      if (tempr)
        tempr[ipp] = tmp;
    }
    fread (&flag, sizeof(flag), 1, aux);
    if (flag)
    {        
      fread(&tmp, sizeof(tmp), 1, aux);
      if (moist)
        moist[ipp] = tmp;
    }
  }
  fclose(aux);
}



/**
  Function returns actual value of damage parameter for
  scalar isotropic damage models.

  @param ipp - integration point number in the mechmat ip array.
  @param im - material index
  @param ido - index of internal variables for given material in the ipp other array

  @retval Function returns actual value of damage parameter. It should be in range <0,1>

  TKo
*/
double mechmat::give_dampar (long ipp,long im,long ido)
{
  double dp;
  
  switch (ip[ipp].tm[im]){
  case scaldamage:{
    dp = ip[ipp].eqother[ido+1];
    break;
  }
  case scaldamage_time:{
    dp = ip[ipp].eqother[ido+1];
    break;
  }
  default:{
    print_err ("unknown type of damage model is required",__FILE__,__LINE__,__func__);
  }
  }
  
  return dp;
}



/**
  Function returns actual value of equivalent strain norm for scalar damage models.
  It is used in nonlocal models and thus it returns value from OTHER array 
  i.e. from the disequilibrium state !!!

  @param ipp - integration point number in the mechmat ip array.
  @param im - material index
  @param ido - index of internal variables for given material in the ipp other array
  @param kappa - output parameter

  @retval Function does not return anything

  Created by TKo   10.2009
*/
void mechmat::give_kappa(long ipp,long im,long ido, vector &kappa)
{
  switch (ip[ipp].tm[im])
  {
    case scaldamage:
      kappa[0] = ip[ipp].other[ido];
      break;
    case scaldamage_time:
      kappa[0] = ip[ipp].other[ido];
      break;
    case scaldamagecc:
      kappa[0] = ip[ipp].other[ido];
//      kappa[1] = ip[ipp].other[ido+1];
      break;
    default:
      print_err ("unknown type of damage model is required",__FILE__,__LINE__,__func__);
  }  
  return;
}



/**

  @param ipp - integration point number in the mechmat ip array.

*/
double mechmat::give_first_derivative (long ipp,double r)
{
  double d;
  
  switch (ip[ipp].tm[0]){
  case lenjonespot:{
    d = lenjon[ip[ipp].idm[0]].first_derivative (r);
    break;
  }
  default:{
    print_err("unknown material model is required",__FILE__,__LINE__,__func__);
  }
  }
  
  return d;
}



/**

  @param ipp - integration point number in the mechmat ip array.

*/
double mechmat::give_second_derivative (long ipp,double r)
{
  double d;
  
  switch (ip[ipp].tm[0]){
  case lenjonespot:{
    d = lenjon[ip[ipp].idm[0]].second_derivative (r);
    break;
  }
  default:{
    print_err("unknown material model is required",__FILE__,__LINE__,__func__);
  }
  }

  return d;
}



/**
   Function initializes material models with initial values.
   
   21.6.2004, JK
*/
void mechmat::initmaterialmodels (void)
{
  long i,j,nip,ipp,nm,ido;
  elemtype te;
  
  for (i=0;i<Mt->ne;i++){
    if (Gtm->leso[i]==1){
      te = Mt->give_elem_type (i);
      nip = Mt->give_tnip (i);
      //  number of the first integration point
      ipp=Mt->elements[i].ipp[0][0];
      for(j=0; j<nip; j++){
	nm = ip[ipp+j].nm;
	initvalues(ipp+j, 0, 0);
	if (ip[ipp].hmt & 1){
	  // thermal dilatancy is present and ncompeqother 
	  switch (ip[ipp].tm[nm-1]){
	  case therisodilat:
	    break;
	  case therisodilattime:    
	    ido = givencompeqother(ipp+j, 0);
	    ido -= givencompeqother(ipp+j, nm-1);
	    initvalues(ipp+j, nm-1, ido);
	    break;
	  default:{
	    print_err("unknown thermal material type is required",__FILE__,__LINE__,__func__);
	  }
	  }
	}
      }
    }
  }
}



/**
  Function searches index of nonlocal material type at given integration point.

  @param ipp - integration point id

  @retval >= 0 in case that the nonlocal material type was found
  @retval < 0  in case that the integration point has no nonlocal material type

  TKo 8.2008

*/
long mechmat::givenonlocid(long ipp)
{
  long i, ret=-1;
  for(i=0; i<ip[ipp].nm; i++)
  {
    switch(ip[ipp].tm[i])
    {
      case nonlocplastmat :
      case nonlocdamgmat :
      case nonlocalmicroM4 :
        ret = i;
        break;
      default:
        break;
    }
    if (ret >= 0)
      break;
  }
  return ret;
}
